@@ -74,7 +74,7 @@ static per_thread_data * get_ptd(JNIEnv *env, jclass cls)
 static void setup_panic_parse_options(Parse_Options opts)
 {
 	parse_options_set_repeatable_rand(opts, FALSE);
-	parse_options_set_disjunct_cost(opts, 3.0f);
+	parse_options_set_disjunct_cost(opts, 3.7f);
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
 	parse_options_set_max_parse_time(opts, 60);
@@ -84,7 +84,6 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_linkage_limit(opts, 100);
 	parse_options_set_verbosity(opts, 0);
 	parse_options_set_spell_guess(opts, FALSE);
-	parse_options_set_screen_width(opts, 4096);
 }
 
 static inline void test(void)
@@ -144,16 +143,18 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	 * for grammar learning. That is, for the 'any' language.
 	 */
 	parse_options_set_repeatable_rand(ptd->opts, FALSE);
-	parse_options_set_disjunct_cost(ptd->opts, 2.0f);
-	parse_options_set_max_sentence_length(ptd->opts, MAX_SENTENCE-3);
+
+	/* A cost of 2.7 allows the usual cost-2 connectors, plus the
+	 * assorted fractional costs, without going to cost 3.0, which
+	 * is used only during panic-parsing.
+	 */
+	parse_options_set_disjunct_cost(ptd->opts, 2.7f);
 	parse_options_set_max_parse_time(ptd->opts, 30);
 	parse_options_set_linkage_limit(ptd->opts, 1000);
 	parse_options_set_short_length(ptd->opts, 10);
 	parse_options_set_verbosity(ptd->opts, 0);
 	parse_options_set_spell_guess(ptd->opts, FALSE);
-	parse_options_set_screen_width(ptd->opts, 4096);
 
-	parse_options_set_panic_mode(ptd->opts, TRUE);
 	ptd->panic_parse_opts = parse_options_create();
 	setup_panic_parse_options(ptd->panic_parse_opts);
 
@@ -199,7 +200,6 @@ static void finish(per_thread_data *ptd)
 
 static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 {
-	int maxlen;
 	Parse_Options opts = ptd->opts;
 	int jverbosity = parse_options_get_verbosity(opts);
 
@@ -219,18 +219,6 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	if (ptd->sent == NULL)
 		return;
 
-	maxlen = parse_options_get_max_sentence_length(ptd->opts);
-	if (maxlen < sentence_length(ptd->sent))
-	{
-		if (jverbosity > 0) {
-			prt_error("Error: JNI: Sentence length (%d words) exceeds maximum allowable (%d words)\n",
-				sentence_length(ptd->sent), maxlen);
-		}
-		sentence_delete(ptd->sent);
-		ptd->sent = NULL;
-		return;
-	}
-
 	/* First parse with cost 0 or 1 and no null links or fat links */
 	parse_options_set_disjunct_cost(opts, 2.0f);
 	parse_options_set_min_null_count(opts, 0);
@@ -239,16 +227,21 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 
 	ptd->num_linkages = sentence_parse(ptd->sent, ptd->opts);
 
+	/* If failed bad. Give up. */
+	if (ptd->num_linkages < 0)
+	{
+		sentence_delete(ptd->sent);
+		ptd->sent = NULL;
+		return;
+	}
+
 	/* If failed, try again with null links */
 	if (0 == ptd->num_linkages)
 	{
 		if (jverbosity > 0) prt_error("Warning: JNI: No complete linkages found.\n");
-		if (parse_options_get_allow_null(opts))
-		{
-			parse_options_set_min_null_count(opts, 1);
-			parse_options_set_max_null_count(opts, sentence_length(ptd->sent));
-			ptd->num_linkages = sentence_parse(ptd->sent, opts);
-		}
+		parse_options_set_min_null_count(opts, 1);
+		parse_options_set_max_null_count(opts, sentence_length(ptd->sent));
+		ptd->num_linkages = sentence_parse(ptd->sent, opts);
 	}
 
 	if (jverbosity > 0)
@@ -261,8 +254,7 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	}
 
 	if ((ptd->num_linkages == 0) &&
-	    parse_options_resources_exhausted(opts) &&
-	    parse_options_get_panic_mode(opts))
+	    parse_options_resources_exhausted(opts))
 	{
 		parse_options_print_total_time(opts);
 		if (jverbosity > 0) prt_error("Warning: JNI: Entering \"panic\" mode...\n");
@@ -705,7 +697,7 @@ JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getLinkString(JNIEnv *env, jclass cls)
 {
 	per_thread_data *ptd = get_ptd(env, cls);
-	char *s = linkage_print_diagram(ptd->linkage);
+	char *s = linkage_print_diagram(ptd->linkage, true, 8100);
 	jstring j = (*env)->NewStringUTF(env, s);
 	linkage_free_diagram(s);
 	return j;
