@@ -71,14 +71,14 @@ struct DIS_node_struct
 {
 	CON_list * cl;     /* the list of children */
 	List_o_links * lol;/* the links that comprise this region of the graph */
-	int word;          /* the word defining this node */
+	size_t word;          /* the word defining this node */
 };
 
 struct CON_node_struct
 {
 	DIS_list * dl;     /* the list of children */
 	DIS_list * current;/* defines the current child */
-	int word;          /* the word defining this node */
+	size_t word;          /* the word defining this node */
 };
 
 struct DIS_list_struct
@@ -159,10 +159,10 @@ static void replace_link_name(Link *l, const char *s)
 {
 	/* XXX can get some perf improvement by avoiding strlen! */
 	char * t;
-	exfree((char *) l->name, sizeof(char)*(strlen(l->name)+1));
+	exfree((char *) l->link_name, sizeof(char)*(strlen(l->link_name)+1));
 	t = (char *) exalloc(sizeof(char)*(strlen(s)+1));
 	strcpy(t, s);
-	l->name = t;
+	l->link_name = t;
 }
 #endif /* USE_FAT_LINKAGES */
 
@@ -206,10 +206,10 @@ static void build_digraph(analyze_context_t *actx, Parse_info pi)
 		}
 
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
-		lol->next = actx->word_links[lp->l];
-		actx->word_links[lp->l] = lol;
+		lol->next = actx->word_links[lp->lw];
+		actx->word_links[lp->lw] = lol;
 		lol->link = link;
-		lol->word = lp->r;
+		lol->word = lp->rw;
 		p = lp->lc->priority;
 		if (p == THIN_priority) {
 			lol->dir = 0;
@@ -220,10 +220,10 @@ static void build_digraph(analyze_context_t *actx, Parse_info pi)
 		}
 
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
-		lol->next = actx->word_links[lp->r];
-		actx->word_links[lp->r] = lol;
+		lol->next = actx->word_links[lp->rw];
+		actx->word_links[lp->rw] = lol;
 		lol->link = link;
-		lol->word = lp->l;
+		lol->word = lp->lw;
 		p = lp->rc->priority;
 		if (p == THIN_priority) {
 			lol->dir = 0;
@@ -742,9 +742,9 @@ static inline int cost_for_length(int length)
  * Computes the cost of the current parse of the current sentence,
  * due to the length of the links.
  */
-static int link_cost(Parse_info pi)
+static size_t link_cost(Parse_info pi)
 {
-	int lcost, i;
+	size_t lcost, i;
 	lcost =  0;
 	for (i = 0; i < pi->N_links; i++)
 	{
@@ -791,7 +791,11 @@ const char * intersect_strings(Sentence sent, const char * s, const char * t)
 	const char *w, *s0;
 	char u0[MAX_TOKEN_LENGTH]; /* Links are *always* less than 10 chars long */
 	char *u;
-	if (strcmp(s,t)==0) return s;  /* would work without this */
+
+	if (islower((int) *s)) s++;  /* skip the head-dependent indicator */
+	if (islower((int) *t)) t++;  /* skip the head-dependent indicator */
+
+	if (strcmp(s,t) == 0) return s;  /* would work without this */
 	i = strlen(s);
 	j = strlen(t);
 	if (j > i) {
@@ -811,7 +815,7 @@ const char * intersect_strings(Sentence sent, const char * s, const char * t)
 		}
 		s++; t++; u++;
 	}
-	if (d==0) {
+	if (d == 0) {
 		return s0;
 	} else {
 		strcpy(u, s);   /* get the remainder of s */
@@ -827,7 +831,7 @@ const char * intersect_strings(Sentence sent, const char * s, const char * t)
  */
 static void compute_link_names(Sentence sent)
 {
-	int i;
+	size_t i;
 	Parse_info pi = sent->parse_info;
 
 	for (i = 0; i < pi->N_links; i++)
@@ -873,6 +877,8 @@ static int strictly_smaller_name(const char * s, const char * t)
 	return (strictness > 0);
 }
 
+#define UNUSED_CONNECTION ((size_t) -1)
+
 /**
  * This fills in the sublinkage->link[].name field.  We assume that
  * link_array[].name have already been filled in.  As above, in the
@@ -888,10 +894,11 @@ static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 	int i;
 	const char * s;
 	Parse_info pi = sent->parse_info;
+	count_context_t * ctxt = sent->count_ctxt;
 
 	for (i = 0; i < pi->N_links; i++)
 	{
-		if (sublinkage->link[i]->l == -1) continue;
+		if (sublinkage->link[i]->lw == UNUSED_CONNECTION) continue;
 		/* NULL's here are quite unexpected -- I think there's a bug
 		 * elsewhere in the code. But for now, punt.  Here's a sentence
 		 * that triggers a NULL -- "His convalescence was relatively brief
@@ -900,9 +907,9 @@ static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 		 */
 		if (NULL == sublinkage->link[i]->lc) continue;
 		if (NULL == sublinkage->link[i]->rc) continue;
-		if (!do_match(sent, sublinkage->link[i]->lc, sublinkage->link[i]->rc, 0, 0))
+		if (!do_match(ctxt, sublinkage->link[i]->lc, sublinkage->link[i]->rc, 0, 0))
 		{
-			replace_link_name(sublinkage->link[i], pi->link_array[i].name);
+			replace_link_name(sublinkage->link[i], pi->link_array[i].link_name);
 		}
 		else
 		{
@@ -910,8 +917,8 @@ static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 				connector_get_string(sublinkage->link[i]->lc),
 				connector_get_string(sublinkage->link[i]->rc));
 
-			if (strictly_smaller_name(s, pi->link_array[i].name))
-				replace_link_name(sublinkage->link[i], pi->link_array[i].name);
+			if (strictly_smaller_name(s, pi->link_array[i].link_name))
+				replace_link_name(sublinkage->link[i], pi->link_array[i].link_name);
 			else
 				replace_link_name(sublinkage->link[i], s);
 		}
@@ -990,7 +997,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 		free_DIS_tree(d_root);
 		for (i = 0; i < pi->N_links; i++)
 		{
-			pi->link_array[i].name = "";
+			pi->link_array[i].link_name = "";
 		}
 		return li;
 	}
@@ -1005,6 +1012,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 
 	compute_link_names(sent);
 
+	accum.d_type_array = xalloc(pi->N_links * sizeof(D_type_list *));
 	for (i=0; i<pi->N_links; i++) accum.d_type_array[i] = NULL;
 
 	/* loop through all the sub linkages */
@@ -1013,8 +1021,8 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 		for (i=0; i<pi->N_links; i++)
 		{
 			actx->patch_array[i].used = actx->patch_array[i].changed = FALSE;
-			actx->patch_array[i].newl = pi->link_array[i].l;
-			actx->patch_array[i].newr = pi->link_array[i].r;
+			actx->patch_array[i].newl = pi->link_array[i].lw;
+			actx->patch_array[i].newr = pi->link_array[i].rw;
 			copy_full_link(&sublinkage->link[i], &(pi->link_array[i]));
 		}
 		fill_patch_array_DIS(actx, d_root, NULL);
@@ -1023,13 +1031,13 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 		{
 			if (actx->patch_array[i].changed || actx->patch_array[i].used)
 			{
-				sublinkage->link[i]->l = actx->patch_array[i].newl;
-				sublinkage->link[i]->r = actx->patch_array[i].newr;
+				sublinkage->link[i]->lw = actx->patch_array[i].newl;
+				sublinkage->link[i]->rw = actx->patch_array[i].newr;
 			}
-			else if ((actx->dfs_root_word[pi->link_array[i].l] != -1) &&
-					 (actx->dfs_root_word[pi->link_array[i].r] != -1))
+			else if ((actx->dfs_root_word[pi->link_array[i].lw] != -1) &&
+					 (actx->dfs_root_word[pi->link_array[i].rw] != -1))
 			{
-				sublinkage->link[i]->l = -1;
+				sublinkage->link[i]->lw = UNUSED_CONNECTION;
 			}
 		}
 
@@ -1040,13 +1048,13 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 		}
 
 		/* 'analyze_pass' logic added ALB 1/97 */
-		if (analyze_pass==PP_FIRST_PASS) {
+		if (analyze_pass == PP_FIRST_PASS) {
 			post_process_scan_linkage(postprocessor,opts,sent,sublinkage);
 			if (!advance_DIS(d_root)) break;
 			else continue;
 		}
 
-		pp = post_process(postprocessor, opts, sent, sublinkage, TRUE);
+		pp = do_post_process(postprocessor, opts, sent, sublinkage, TRUE);
 
 		if (pp==NULL) {
 			if (postprocessor != NULL) li.N_violations = 1;
@@ -1056,7 +1064,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 			   ancestry for a link in each of its sentences is consistent. */
 
 			for (i=0; i<pi->N_links; i++) {
-				if (sublinkage->link[i]->l == -1) continue;
+				if (sublinkage->link[i]->lw == -1) continue;
 				if (accum.d_type_array[i] == NULL) {
 					accum.d_type_array[i] = copy_d_type(pp->d_type_array[i]);
 				} else {
@@ -1084,6 +1092,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 	for (i=0; i<pi->N_links; ++i) {
 		free_d_type(accum.d_type_array[i]);
 	}
+	xfree(accum.d_type_array, pi->N_links * sizeof(D_type_list *));
 
 	/* if (display_on && (li.N_violations != 0) &&
 	   (verbosity > 3) && should_print_messages)
@@ -1102,7 +1111,7 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
  */
 Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze_pass)
 {
-	int i;
+	size_t i;
 	Linkage_info li;
 	PP_node * pp;
 	Postprocessor * postprocessor;
@@ -1137,7 +1146,7 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	 * For this to work, however, you have to call "build_digraph"
 	 * first (as in analyze_fat_linkage), and then "free_digraph".
 	 */
-	pp = post_process(postprocessor, opts, sent, sublinkage, TRUE);
+	pp = do_post_process(postprocessor, opts, sent, sublinkage, TRUE);
 
 	memset(&li, 0, sizeof(li));
 	li.N_violations = 0;
@@ -1178,7 +1187,7 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 
 void extract_thin_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 {
-	int i;
+	size_t i;
 	Parse_info pi = sent->parse_info;
 
 #ifdef USE_FAT_LINKAGES
@@ -1327,8 +1336,8 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 		for (i = 0; i < pi->N_links; i++)
 		{
 			actx->patch_array[i].used = actx->patch_array[i].changed = FALSE;
-			actx->patch_array[i].newl = pi->link_array[i].l;
-			actx->patch_array[i].newr = pi->link_array[i].r;
+			actx->patch_array[i].newl = pi->link_array[i].lw;
+			actx->patch_array[i].newr = pi->link_array[i].rw;
 			copy_full_link(&sublinkage->link[i], &(pi->link_array[i]));
 		}
 		fill_patch_array_DIS(actx, d_root, NULL);
@@ -1337,13 +1346,13 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 		{
 			if (actx->patch_array[i].changed || actx->patch_array[i].used)
 			{
-				sublinkage->link[i]->l = actx->patch_array[i].newl;
-				sublinkage->link[i]->r = actx->patch_array[i].newr;
+				sublinkage->link[i]->lw = actx->patch_array[i].newl;
+				sublinkage->link[i]->rw = actx->patch_array[i].newr;
 			}
-			else if ((actx->dfs_root_word[pi->link_array[i].l] != -1) &&
-					   (actx->dfs_root_word[pi->link_array[i].r] != -1))
+			else if ((actx->dfs_root_word[pi->link_array[i].lw] != -1) &&
+					   (actx->dfs_root_word[pi->link_array[i].rw] != -1))
 			{
-				sublinkage->link[i]->l = -1;
+				sublinkage->link[i]->lw = UNUSED_CONNECTION;
 			}
 		}
 
@@ -1357,7 +1366,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 		N_thin_links = 0;
 		for (i = 0; i < pi->N_links; ++i)
 		{
-			if (sublinkage->link[i]->l == -1) continue;
+			if (sublinkage->link[i]->lw == UNUSED_CONNECTION) continue;
 			N_thin_links++;
 		}
 
@@ -1369,7 +1378,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 
 		for (i = 0, j = 0; i < pi->N_links; ++i)
 		{
-			if (sublinkage->link[i]->l == -1) continue;
+			if (sublinkage->link[i]->lw == UNUSED_CONNECTION) continue;
 			linkage->sublinkage[num_sublinkages].link[j++] =
 				excopy_link(sublinkage->link[i]);
 		}
