@@ -12,6 +12,7 @@
 
 #ifdef USE_FAT_LINKAGES
 
+#include "analyze-linkage.h"
 #include "and.h"
 #include "api-structures.h"
 #include "dict-structures.h"
@@ -380,17 +381,17 @@ static inline int and_hash_disjunct(Disjunct *d)
  * Check here that the connectors are from some small set.
  * This will disallow, for example "the and their dog ran".
  */
-static int is_appropriate(Sentence sent, Disjunct * d)
+static bool is_appropriate(count_context_t *ctxt, Connector_set *acs, Disjunct * d)
 {
 	Connector * c;
 
-	if (sent->dict->andable_connector_set == NULL) return TRUE;
+	if (acs == NULL) return TRUE;
 	/* if no set, then everything is considered andable */
 	for (c = d->right; c!=NULL; c=c->next) {
-		if (!match_in_connector_set(sent, sent->dict->andable_connector_set, c, '+')) return FALSE;
+		if (!match_in_connector_set(ctxt, acs, c, '+')) return FALSE;
 	}
 	for (c = d->left; c!=NULL; c=c->next) {
-		if (!match_in_connector_set(sent, sent->dict->andable_connector_set, c, '-')) return FALSE;
+		if (!match_in_connector_set(ctxt, acs, c, '-')) return FALSE;
 	}
 	return TRUE;
 }
@@ -608,12 +609,15 @@ static void put_disjunct_into_table(Sentence sent, Disjunct *d)
 static void extract_all_fat_links(Sentence sent, Disjunct * d)
 {
 	Connector * cl, * cr, *tl, *tr;
+	Connector_set * andablecs = sent->dict->andable_connector_set;
+	count_context_t *ctxt = sent->count_ctxt;
+
 	tl = d->left;
 	d->left = NULL;
 	for (cr = d->right; cr!=NULL; cr = cr->next) {
 		tr = cr->next;
 		cr->next = NULL;
-		if (is_appropriate(sent, d)) put_disjunct_into_table(sent, d);
+		if (is_appropriate(ctxt, andablecs, d)) put_disjunct_into_table(sent, d);
 		cr->next = tr;
 	}
 	d->left = tl;
@@ -623,7 +627,7 @@ static void extract_all_fat_links(Sentence sent, Disjunct * d)
 	for (cl = d->left; cl!=NULL; cl = cl->next) {
 		tl = cl->next;
 		cl->next = NULL;
-		if (is_appropriate(sent, d)) put_disjunct_into_table(sent, d);
+		if (is_appropriate(ctxt, andablecs, d)) put_disjunct_into_table(sent, d);
 		cl->next = tl;
 	}
 	d->right = tr;
@@ -634,7 +638,7 @@ static void extract_all_fat_links(Sentence sent, Disjunct * d)
 			tr = cr->next;
 			cl->next = cr->next = NULL;
 
-			if (is_appropriate(sent, d)) put_disjunct_into_table(sent, d);
+			if (is_appropriate(ctxt, andablecs, d)) put_disjunct_into_table(sent, d);
 
 			cl->next = tl;
 			cr->next = tr;
@@ -695,7 +699,7 @@ static void compute_matchers_for_a_label(Sentence sent, int k)
 	while(d != NULL) {
 		i = 0;
 		for (c=d->left; c != NULL; c = c->next) {
-         cs = skip_utf8_upper(c->string);
+			cs = skip_utf8_upper(c->string);
 			j = strlen(cs);
 			if (j > lengths[i]) lengths[i] = j;
 			i++;
@@ -803,10 +807,13 @@ static void connector_for_disjunct(Sentence sent, Disjunct * d, Connector * c)
  * fat link, in two possible positions.  Does not effect d.
  * The cost of d is inherited by all of the disjuncts in the result.
  */
-static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
+static Disjunct * build_fat_link_substitutions(Sentence sent,
+                         count_context_t* ctxt, Disjunct *d)
 {
 	Connector * cl, * cr, *tl, *tr, *wc, work_connector;
 	Disjunct *d1, *wd, work_disjunct, *d_list;
+	Connector_set * andablecs = sent->dict->andable_connector_set;
+
 	if (d==NULL) return NULL;
 	wd = &work_disjunct;
 	wc = init_connector(&work_connector);
@@ -817,7 +824,7 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
 	for (cr = d->right; cr!=NULL; cr = cr->next) {
 		tr = cr->next;
 		cr->next = NULL;
-		if (is_appropriate(sent, d)) {
+		if (is_appropriate(ctxt, andablecs, d)) {
 			connector_for_disjunct(sent, d, wc);
 			wd->left = tl;
 			wd->right = wc;
@@ -841,7 +848,7 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
 	for (cl = d->left; cl!=NULL; cl = cl->next) {
 		tl = cl->next;
 		cl->next = NULL;
-		if (is_appropriate(sent, d)) {
+		if (is_appropriate(ctxt, andablecs, d)) {
 			connector_for_disjunct(sent, d, wc);
 			wd->left = tl;
 			wd->right = wc;
@@ -865,7 +872,7 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
 			tl = cl->next;
 			tr = cr->next;
 			cl->next = cr->next = NULL;
-			if (is_appropriate(sent, d)) {
+			if (is_appropriate(ctxt, andablecs, d)) {
 				connector_for_disjunct(sent, d, wc);
 				wd->left = tl;
 				wd->right = wc;
@@ -893,14 +900,13 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
  */
 Disjunct * explode_disjunct_list(Sentence sent, Disjunct *d)
 {
-   Disjunct *d1;
+	Disjunct *d1 = NULL;
+	count_context_t * ctxt = sent->count_ctxt;
 
-   d1 = NULL;
-
-   for (; d!=NULL; d = d->next) {
-	   d1 = catenate_disjuncts(d1, build_fat_link_substitutions(sent, d));
-   }
-   return d1;
+	for (; d!=NULL; d = d->next) {
+	   d1 = catenate_disjuncts(d1, build_fat_link_substitutions(sent, ctxt, d));
+	}
+	return d1;
 }
 
 /**
@@ -957,12 +963,13 @@ Disjunct * build_AND_disjunct_list(Sentence sent, const char * s)
 	int lab;
 	Disjunct *d_list, *d1, *d3, *d, *d_copy;
 	Connector *c1, *c2, *c3;
+	count_context_t *ctxt = sent->count_ctxt;
 
 	d_list = NULL;  /* where we put the list we're building */
 
 	for (lab = 0; lab < sent->and_data.LT_size; lab++) {
 		for (d = sent->and_data.label_table[lab]; d!=NULL; d=d->next) {
-			d1 = build_fat_link_substitutions(sent, d);
+			d1 = build_fat_link_substitutions(sent, ctxt, d);
 			d_copy = copy_disjunct(d);  /* also include the thing itself! */
 			d_copy->next = d1;
 			d1 = d_copy;
@@ -1179,12 +1186,12 @@ int set_has_fat_down(Sentence sent)
 		if (pi->link_array[link].lc->priority == DOWN_priority)
 		{
 			N_fat ++;
-			pi->has_fat_down[pi->link_array[link].l] = TRUE;
+			pi->has_fat_down[pi->link_array[link].lw] = TRUE;
 		}
 		else if (pi->link_array[link].rc->priority == DOWN_priority)
 		{
 			N_fat ++;
-			pi->has_fat_down[pi->link_array[link].r] = TRUE;
+			pi->has_fat_down[pi->link_array[link].rw] = TRUE;
 		}
 	}
 	return (N_fat > 0);
@@ -1228,7 +1235,7 @@ static void build_image_array(Sentence sent)
 		{
 			if (end < 0)
 			{
-				word = pi->link_array[link].l;
+				word = pi->link_array[link].lw;
 				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].lc;
 				other_end_con = pi->link_array[link].rc;
@@ -1237,7 +1244,7 @@ static void build_image_array(Sentence sent)
 			}
 			else
 			{
-				word = pi->link_array[link].r;
+				word = pi->link_array[link].rw;
 				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].rc;
 				other_end_con = pi->link_array[link].lc;
@@ -1358,9 +1365,9 @@ static Disjunct * find_subdisjunct(Sentence sent, Disjunct * dis, int label)
 	return d;
 }
 
-static int x_match(Sentence sent, Connector *a, Connector *b)
+static int x_match(count_context_t * ctxt, Connector *a, Connector *b)
 {
-   return do_match(sent, a, b, 0, 0);
+	return do_match(ctxt, a, b, 0, 0);
 }
 
 /**
@@ -1376,6 +1383,7 @@ int is_canonical_linkage(Sentence sent)
 	Disjunct *dis, *chosen_d;
 	Image_node * in;
 	Parse_info pi = sent->parse_info;
+	count_context_t * ctxt = sent->count_ctxt;
 
 	init_connector(&dummy_connector);
 	dummy_connector.priority = UP_priority;
@@ -1437,17 +1445,17 @@ int is_canonical_linkage(Sentence sent)
 					}
 
 					/* I hope using x_match here is right */
-					if (!x_match(sent, &dummy_connector, in->c)) break;  
+					if (!x_match(ctxt, &dummy_connector, in->c)) break;  
 				} else if (place > 0) {
 					for (c=dis->right; place > 1; place--) {
 						c = c->next;
 					}
-					if (!x_match(sent, c, in->c)) break;	/* Ditto above comment  --DS 07/97*/
+					if (!x_match(ctxt, c, in->c)) break;	/* Ditto above comment  --DS 07/97*/
 				} else {
 					for (c=dis->left; place < -1; place++) {
 						c = c->next;
 					}
-					if (!x_match(sent, c, in->c)) break;  /* Ditto Ditto */
+					if (!x_match(ctxt, c, in->c)) break;  /* Ditto Ditto */
 				}
 			}
 
@@ -1461,6 +1469,8 @@ int is_canonical_linkage(Sentence sent)
 	return (w == pi->N_words);
 }
 
+#define UNUSED_CONNECTION ((size_t) -1)
+
 /**
  * This takes as input link_array[], sublinkage->link[]->l and
  * sublinkage->link[]->r (and also has_fat_down[word], which has been
@@ -1480,23 +1490,23 @@ void compute_pp_link_array_connectors(Sentence sent, Sublinkage *sublinkage)
 	{
 		for (link=0; link<pi->N_links; link++)
 		{
-			if (sublinkage->link[link]->l == -1) continue;
+			if (sublinkage->link[link]->lw == UNUSED_CONNECTION) continue;
 			if (end < 0)
 			{
-				word = pi->link_array[link].l;
+				word = pi->link_array[link].lw;
 				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].lc;
 				dis = pi->chosen_disjuncts[word];
-				mydis = pi->chosen_disjuncts[sublinkage->link[link]->l];
+				mydis = pi->chosen_disjuncts[sublinkage->link[link]->lw];
 				clist = dis->right;
 			}
 			else
 			{
-				word = pi->link_array[link].r;
+				word = pi->link_array[link].rw;
 				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].rc;
 				dis = pi->chosen_disjuncts[word];
-				mydis = pi->chosen_disjuncts[sublinkage->link[link]->r];
+				mydis = pi->chosen_disjuncts[sublinkage->link[link]->rw];
 				clist = dis->left;
 			}
 
@@ -1540,10 +1550,10 @@ void compute_pp_link_array_connectors(Sentence sent, Sublinkage *sublinkage)
 				mycon = mydis->right;
 			} else {
 				printf("word = %d\n", word);
-				printf("fat link: [%d, %d]\n",
-					   pi->link_array[link].l, pi->link_array[link].r);
-				printf("thin link: [%d, %d]\n",
-					   sublinkage->link[link]->l, sublinkage->link[link]->r);
+				printf("fat link: [%zu, %zu]\n",
+					   pi->link_array[link].lw, pi->link_array[link].rw);
+				printf("thin link: [%zu, %zu]\n",
+					   sublinkage->link[link]->lw, sublinkage->link[link]->rw);
 				assert(FALSE, "There should be a fat UP link here");
 			}
 
