@@ -59,53 +59,43 @@ struct Resources_s
 
 struct Parse_Options_s
 {
+	/* General options */
 	int verbosity;         /* Level of detail to give about the computation 0 */
 	char * debug;          /* comma-sparated function names to debug "" */
 	char * test;           /* comma-sparated features to test "" */
+	Resources resources;   /* For deciding when to abort the parsing */
+
+	/* Options governing the tokenizer (sentence-splitter) */
+	bool use_spell_guess;  /* Perform spell-guessing of unknown words. */ 
+
+	/* Choice of the parser to use */
 	bool use_sat_solver;   /* Use the Boolean SAT based parser */
 	bool use_viterbi;      /* Use the Viterbi decoder-based parser */
-	size_t linkage_limit;     /* The maximum number of linkages processed 100 */
+
+	/* Options governing the parser internals operation */
 	double disjunct_cost;  /* Max disjunct cost to allow */
 	int min_null_count;    /* The minimum number of null links to allow */
 	int max_null_count;    /* The maximum number of null links to allow */
-	int null_block;        /* consecutive blocks of this many words are
-	                          considered as one null link  (default=1) */
 	bool islands_ok;       /* If TRUE, then linkages with islands
 	                          (separate component of the link graph)
 	                          will be generated (default=FALSE) */
-	size_t twopass_length; /* min length for two-pass post processing */
-	size_t max_sentence_length;
+	bool use_cluster_disjuncts; /* Attempt using a broader list of disjuncts */
 	size_t short_length;   /* Links that are limited in length can be
 	                        * no longer than this.  Default = 6 */
 	bool all_short;        /* If true, there can be no connectors that are exempt */
-	bool use_spell_guess;  /* Perform spell-guessing of unknown words. */ 
 	bool repeatable_rand;  /* Reset rand number gen after every parse. */
-	Cost_Model cost_model; /* For sorting linkages in post_processing */
-	Resources resources;   /* For deciding when to abort the parsing */
 
-	/* Flags governing the command-line client; not used by parser */
-	bool display_short;
-	bool display_word_subscripts;  /* as in "dog.n" as opposed to "dog" */
-	bool display_link_subscripts;  /* as in "Ss" as opposed to "S" */
-	bool display_walls;
 #ifdef USE_FAT_LINKAGES
 	bool use_fat_links;     /* Look for fat linkages */
-	bool display_union;     /* print squashed version of linkage with conjunction? */
 #endif /* USE_FAT_LINKAGES */
-	bool allow_null;        /* true if we allow null links in parsing */
-	bool use_cluster_disjuncts; /* if true, atttempt using a borader list of disjuncts */
-	bool echo_on;           /* true if we should echo the input sentence */
-	bool batch_mode;        /* if true, process sentences non-interactively */
-	bool panic_mode;        /* if true, parse in "panic mode" after all else fails */
-	int screen_width;       /* width of screen for displaying linkages */
-	bool display_on;        /* if true, output graphical linkage diagram */
-	bool display_postscript;/* if true, output postscript linkage */
-	ConstituentDisplayStyle display_constituents; /* style for displaying constituent structure */
-	bool display_bad;       /* if true, bad linkages are displayed */
-	bool display_disjuncts; /* if true, print disjuncts that were used */
-	bool display_links;     /* if true, a list o' links is printed out */
+
+	/* Options governing post-processing */
+	size_t twopass_length; /* min sent length for two-pass post processing */
+	Cost_Model cost_model; /* For sorting linkages in post_processing */
+
+	/* Options governing the generation of linkages. */
+	size_t linkage_limit;  /* The maximum number of linkages processed 100 */
 	bool display_morphology;/* if true, print morpho analysis of words */
-	bool display_senses;    /* if true, sense candidates are printed out */
 };
 
 struct Connector_set_s
@@ -116,7 +106,7 @@ struct Connector_set_s
 
 struct Afdict_class_struct
 {
-	size_t mem_elems;     /* number of memory elements alolocated */
+	size_t mem_elems;     /* number of memory elements allocated */
 	size_t length;        /* number of strings */
 	char const ** string;
 };
@@ -138,6 +128,9 @@ struct Dictionary_s
 	/* Affixes are used during the tokenization stage. */
 	Dictionary      affix_table;
 	Afdict_class *  afdict_class;
+#ifdef USE_ANYSPLIT
+	struct anysplit_params * anysplit;
+#endif
 
 	/* If not null, then use spelling guesser for unknown words */
 	void *          spell_checker; /* spell checker handle */
@@ -213,6 +206,27 @@ struct Link_s
 	const char * link_name; /* Spelling of full link name */
 };
 
+/**
+ * N_words:
+ *   The number of words in the current sentence.  Computed by
+ *   separate_sentence().
+ *
+ * N_links:
+ *   The number of links in the current linkage.  Computed by
+ *   extract_linkage().
+ *
+ * chosen_disjuncts[]
+ *   This is an array pointers to disjuncts, one for each word, that is
+ *   computed by extract_links().  It represents the chosen disjuncts
+ *   for the current linkage.  It is used to compute the cost of the
+ *   linkage, and also by compute_chosen_words() to compute the
+ *   chosen_words[].
+ *
+ * link_array[]
+ *   This is an array of links.  These links define the current linkage.
+ *    It is computed by extract_links().  It is used by analyze_linkage()
+ *    to compute pp_linkage[].
+ */
 struct Parse_info_struct
 {
 	unsigned int   x_table_size;
@@ -221,8 +235,9 @@ struct Parse_info_struct
 	Parse_set *    parse_set;
 	int            N_words;
 	Disjunct **    chosen_disjuncts;
-	int            N_links;
-	Link           link_array[MAX_LINKS];
+	size_t         N_links;
+	size_t         lasz;
+	Link           *link_array;
 
 #ifdef USE_FAT_LINKAGES
 	/* Points to the image structure for each word.
@@ -258,26 +273,23 @@ struct Sentence_s
 	                               This is not the same as num alloced
 	                               because some may be non-canonical. */
 	size_t num_valid_linkages;  /* number with no pp violations */
-	bool   null_links;          /* null links allowed */
 	size_t null_count;          /* number of null links in linkages */
 	Parse_info     parse_info;  /* set of parses for the sentence */
 	Linkage_info * link_info;   /* array of valid and invalid linkages (sorted) */
 
 	/* Tokenizer internal/private state */
-	bool   * post_quote;
+	bool   * post_quote;        /* Array, one entry per word, true if quote */
 	int    t_start;             /* start word of the current token sequence */
 	int    t_count;             /* word count in the current token sequence */
 
+#ifdef USE_FAT_LINKAGES
 	/* parser internal/private state */
-#ifdef USE_FAT_LINKAGES
-	analyze_context_t * analyze_ctxt; /* private state  used for analyzing */
-#endif /* USE_FAT_LINKAGES */
-	count_context_t * count_ctxt; /* private state info used for counting */
-	match_context_t * match_ctxt; /* private state info used for matching */
-
-#ifdef USE_FAT_LINKAGES
-	int    num_thin_linkages;   /* valid linkages which are not fat */
 	/* Obsolete fat-linkage state */
+	analyze_context_t * analyze_ctxt; /* private state  used for analyzing */
+	count_context_t * count_ctxt; /* private state info used for counting */
+
+	bool   null_links;          /* null links allowed */
+	int    num_thin_linkages;   /* valid linkages which are not fat */
 	char * is_conjunction;      /* Array of flags, one per word; set to
 	                               TRUE if conjunction, as defined by dictionary */
 	char** deletable;           /* deletable regions in a sentence with conjunction */
@@ -349,6 +361,7 @@ struct Postprocessor_s
 	pp_linkset *set_of_links_in_an_active_rule;/*used in *some* linkage of sent*/
 	int *relevant_contains_one_rules;        /* -1-terminated list of indices  */
 	int *relevant_contains_none_rules;
+
 	/* the following maintain state during a call to post_process() */
 	String_set *sentence_link_name_set;        /* link names seen for sentence */
 	bool visited[MAX_SENTENCE];                 /* for the depth-first search */
