@@ -101,10 +101,14 @@ Parse_Options parse_options_create(void)
 	po->debug = (char *)"";
 	po->test = (char *)"";
 	po->linkage_limit = 100;
-	po->disjunct_cost = 2.0;   /* Maybe should be 1.0 ?? */
+
+	/* A cost of 2.7 allows the usual cost-2 connectors, plus the
+	 * assorted fractional costs, without going to cost 3.0, which
+	 * is used only during panic-parsing.
+	 */
+	po->disjunct_cost = 2.7; /* 3.0 is needed for Russian dicts */
 	po->min_null_count = 0;
 	po->max_null_count = 0;
-	po->null_block = 1;
 	po->islands_ok = false;
 	po->use_spell_guess = true;
 	po->use_sat_solver = false;
@@ -124,31 +128,13 @@ Parse_Options parse_options_create(void)
 	po->short_length = 10;
 	po->all_short = false;
 	po->twopass_length = 30;
-	po->max_sentence_length = MAX_SENTENCE-3;
 	po->repeatable_rand = true;
 	po->resources = resources_create();
-	po->display_short = true;
-	po->display_word_subscripts = true;
-	po->display_link_subscripts = true;
-	po->display_walls = false;
 #ifdef USE_FAT_LINKAGES
 	po->use_fat_links = false;
-	po->display_union = false;
 #endif /* USE_FAT_LINKAGES */
-	po->allow_null = true;
 	po->use_cluster_disjuncts = false;
-	po->echo_on = false;
-	po->batch_mode = false;
-	po->panic_mode = false;
-	po->screen_width = 79;
-	po->display_on = true;
-	po->display_postscript = false;
-	po->display_constituents = NO_DISPLAY;
-	po->display_bad = false;
-	po->display_disjuncts = false;
-	po->display_links = false;
 	po->display_morphology = false;
-	po->display_senses = false;
 
 	return po;
 }
@@ -332,14 +318,6 @@ int parse_options_get_max_null_count(Parse_Options opts) {
 	return opts->max_null_count;
 }
 
-
-void parse_options_set_null_block(Parse_Options opts, int dummy) {
-	opts->null_block = dummy;
-}
-int parse_options_get_null_block(Parse_Options opts) {
-	return opts->null_block;
-}
-
 void parse_options_set_islands_ok(Parse_Options opts, bool dummy) {
 	opts->islands_ok = dummy;
 }
@@ -396,46 +374,6 @@ int parse_options_get_max_memory(Parse_Options opts) {
 	return opts->resources->max_memory;
 }
 
-void parse_options_set_max_sentence_length(Parse_Options opts, int dummy) {
-	opts->max_sentence_length = dummy;
-}
-
-int parse_options_get_max_sentence_length(Parse_Options opts) {
-	return opts->max_sentence_length;
-}
-
-void parse_options_set_echo_on(Parse_Options opts, int dummy) {
-	opts->echo_on = dummy;
-}
-
-int parse_options_get_echo_on(Parse_Options opts) {
-	return opts->echo_on;
-}
-
-void parse_options_set_batch_mode(Parse_Options opts, int dummy) {
-	opts->batch_mode = dummy;
-}
-
-int parse_options_get_batch_mode(Parse_Options opts) {
-	return opts->batch_mode;
-}
-
-void parse_options_set_panic_mode(Parse_Options opts, int dummy) {
-	opts->panic_mode = dummy;
-}
-
-int parse_options_get_panic_mode(Parse_Options opts) {
-	return opts->panic_mode;
-}
-
-void parse_options_set_allow_null(Parse_Options opts, bool dummy) {
-	opts->allow_null = dummy;
-}
-
-bool parse_options_get_allow_null(Parse_Options opts) {
-	return opts->allow_null;
-}
-
 void parse_options_set_use_cluster_disjuncts(Parse_Options opts, bool dummy) {
 	opts->use_cluster_disjuncts = dummy;
 }
@@ -444,75 +382,6 @@ bool parse_options_get_use_cluster_disjuncts(Parse_Options opts) {
 	return opts->use_cluster_disjuncts;
 }
 
-void parse_options_set_screen_width(Parse_Options opts, int dummy) {
-	opts->screen_width = dummy;
-}
-
-int parse_options_get_screen_width(Parse_Options opts) {
-	return opts->screen_width;
-}
-
-
-void parse_options_set_display_on(Parse_Options opts, int dummy) {
-	opts->display_on = dummy;
-}
-
-int parse_options_get_display_on(Parse_Options opts) {
-	return opts->display_on;
-}
-
-void parse_options_set_display_postscript(Parse_Options opts, int dummy) {
-	opts->display_postscript = dummy;
-}
-
-int parse_options_get_display_postscript(Parse_Options opts)
-{
-	return opts->display_postscript;
-}
-
-void parse_options_set_display_constituents(Parse_Options opts, ConstituentDisplayStyle dummy)
-{
-	if (dummy > MAX_STYLES)
-	{
-		prt_error("Possible values for constituents: \n"
-	             "   0 (no display)\n"
-	             "   1 (treebank style, multi-line indented)\n"
-	             "   2 (flat tree, square brackets)\n"
-	             "   3 (flat treebank style)\n");
-		opts->display_constituents = NO_DISPLAY;
-	}
-	else opts->display_constituents = dummy;
-}
-
-ConstituentDisplayStyle parse_options_get_display_constituents(Parse_Options opts)
-{
-	return opts->display_constituents;
-}
-
-void parse_options_set_display_bad(Parse_Options opts, int dummy) {
-	opts->display_bad = dummy;
-}
-
-int parse_options_get_display_bad(Parse_Options opts) {
-	return opts->display_bad;
-}
-
-void parse_options_set_display_disjuncts(Parse_Options opts, int dummy) {
-	opts->display_disjuncts = dummy;
-}
-
-int parse_options_get_display_disjuncts(Parse_Options opts) {
-	return opts->display_disjuncts;
-}
-
-void parse_options_set_display_links(Parse_Options opts, int dummy) {
-	opts->display_links = dummy;
-}
-
-int parse_options_get_display_links(Parse_Options opts) {
-	return opts->display_links;
-}
-
 int parse_options_get_display_morphology(Parse_Options opts) {
 	return opts->display_morphology;
 }
@@ -521,36 +390,6 @@ void parse_options_set_display_morphology(Parse_Options opts, int dummy) {
 	opts->display_morphology = dummy;
 }
 
-void parse_options_set_display_senses(Parse_Options opts, int dummy) {
-	opts->display_senses = dummy;
-}
-
-int parse_options_get_display_senses(Parse_Options opts) {
-	return opts->display_senses;
-}
-
-void parse_options_set_display_walls(Parse_Options opts, bool dummy) {
-	opts->display_walls = dummy;
-}
-
-bool parse_options_get_display_walls(Parse_Options opts) {
-	return opts->display_walls;
-}
-
-int parse_options_get_display_union(Parse_Options opts) {
-#ifdef USE_FAT_LINKAGES
-	return opts->display_union;
-#else
-	return 0;
-#endif /* USE_FAT_LINKAGES */
-}
-
-void parse_options_set_display_union(Parse_Options opts, int dummy) {
-#ifdef USE_FAT_LINKAGES
-	opts->display_union = dummy;
-#endif /* USE_FAT_LINKAGES */
-}
-
 bool parse_options_timer_expired(Parse_Options opts) {
 	return resources_timer_expired(opts->resources);
 }
@@ -605,9 +444,9 @@ static void linkage_info_delete(Linkage_info *link_info, int sz)
 #ifdef USE_FAT_LINKAGES
 static void free_andlists(Sentence sent)
 {
-	int L;
+	size_t L;
 	Andlist * andlist, * next;
-	for(L=0; L<sent->num_linkages_post_processed; L++) {
+	for (L=0; L<sent->num_linkages_post_processed; L++) {
 		/* printf("%d ", sent->link_info[L].canonical);  */
 		/* if (sent->link_info[L].canonical==0) continue; */
 		andlist = sent->link_info[L].andlist;
@@ -634,7 +473,9 @@ static void free_post_processing(Sentence sent)
 	}
 }
 
-static void post_process_linkages(Sentence sent, Parse_Options opts)
+static void post_process_linkages(Sentence sent, match_context_t* mchxt, 
+                                  count_context_t* ctxt,
+                                  Parse_Options opts)
 {
 	int *indices;
 	size_t in, block_bottom, block_top;
@@ -650,7 +491,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 
 	free_post_processing(sent);
 
-	overflowed = build_parse_set(sent, sent->null_count, opts);
+	overflowed = build_parse_set(sent, mchxt, ctxt, sent->null_count, opts);
 	print_time(opts, "Built parse set");
 
 	if (overflowed && (1 < opts->verbosity))
@@ -906,7 +747,7 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
  */
 static void set_is_conjunction(Sentence sent)
 {
-	int w;
+	size_t w;
 	const char * s;
 	for (w=0; w<sent->length; w++) {
 		s = sent->word[w].alternatives[0];
@@ -993,16 +834,14 @@ void sentence_delete(Sentence sent)
 	if (sent->parse_info) free_parse_info(sent->parse_info);
 	free_post_processing(sent);
 	post_process_close_sentence(sent->dict->postprocessor);
+
 #ifdef USE_FAT_LINKAGES
 	free_deletable(sent);
 	free_effective_dist(sent);
-#endif /* USE_FAT_LINKAGES */
-
-	free_count(sent);
-#ifdef USE_FAT_LINKAGES
 	free_analyze(sent);
 	if (sent->is_conjunction) xfree(sent->is_conjunction, sizeof(char)*sent->length);
 #endif /* USE_FAT_LINKAGES */
+
 	global_rand_state = sent->rand_state;
 	xfree((char *) sent, sizeof(struct Sentence_s));
 }
@@ -1013,12 +852,14 @@ int sentence_length(Sentence sent)
 	return sent->length;
 }
 
-int sentence_null_count(Sentence sent) {
+int sentence_null_count(Sentence sent)
+{
 	if (!sent) return 0;
 	return sent->null_count;
 }
 
-int sentence_num_thin_linkages(Sentence sent) {
+int sentence_num_thin_linkages(Sentence sent)
+{
 	if (!sent) return 0;
 #ifdef USE_FAT_LINKAGES
 	return sent->num_thin_linkages;
@@ -1027,72 +868,82 @@ int sentence_num_thin_linkages(Sentence sent) {
 #endif /* USE_FAT_LINKAGES */
 }
 
-int sentence_num_linkages_found(Sentence sent) {
+int sentence_num_linkages_found(Sentence sent)
+{
 	if (!sent) return 0;
 	return sent->num_linkages_found;
 }
 
-int sentence_num_valid_linkages(Sentence sent) {
+int sentence_num_valid_linkages(Sentence sent)
+{
 	if (!sent) return 0;
 	return sent->num_valid_linkages;
 }
 
-int sentence_num_linkages_post_processed(Sentence sent) {
+int sentence_num_linkages_post_processed(Sentence sent)
+{
 	if (!sent) return 0;
 	return sent->num_linkages_post_processed;
 }
 
-int sentence_num_violations(Sentence sent, int i) {
+int sentence_num_violations(Sentence sent, LinkageIdx i)
+{
 	if (!sent) return 0;
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0;
+	if (sent->num_linkages_alloced <= i) return 0; /* bounds check */
 	return sent->link_info[i].N_violations;
 }
 
-int sentence_and_cost(Sentence sent, int i) {
+int sentence_and_cost(Sentence sent, LinkageIdx i) {
 #ifdef USE_FAT_LINKAGES
 	if (!sent) return 0;
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0;
+	if (sent->num_linkages_alloced <= i) return 0; /* bounds check */
 	return sent->link_info[i].and_cost;
 #else
 	return 0;
 #endif /* USE_FAT_LINKAGES */
 }
 
-double sentence_disjunct_cost(Sentence sent, int i)
+double sentence_disjunct_cost(Sentence sent, LinkageIdx i)
 {
 	if (!sent) return 0.0;
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0.0;
+	if (sent->num_linkages_alloced <= i) return 0.0; /* bounds check */
 	return sent->link_info[i].disjunct_cost;
 }
 
-int sentence_link_cost(Sentence sent, int i)
+int sentence_link_cost(Sentence sent, LinkageIdx i)
 {
 	if (!sent) return 0;
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0;
+	if (sent->num_linkages_alloced <= i) return 0; /* bounds check */
 	return sent->link_info[i].link_cost;
 }
 
 /* ============================================================== */
 /* A kind of morphism post-processing */
 
-#define INFIX_MARK '='
-
-static inline Boolean is_AFFIXTYPE_STEM(const char *a, size_t len)
-	{ return (SUBSCRIPT_MARK == a[len]) && (INFIX_MARK == a[len+1]); }
-static inline Boolean is_AFFIXTYPE_PREFIX(const char *a, size_t len)
-	{ return INFIX_MARK == a[len-1]; }
-static inline Boolean is_AFFIXTYPE_SUFFIX(const char *a)
-	{ return (INFIX_MARK == a[0] && '\0' != a[1]); }
-static inline Boolean is_AFFIXTYPE_EMPTY(const char *a, size_t len)
-	{ return (len == 5) && (0 == strcmp(a, EMPTY_WORD_MARK)); }
+static inline bool
+	is_AFFIXTYPE_STEM(char infix_mark, const char *a, size_t len)
+	{ return (SUBSCRIPT_MARK == a[len]) && (infix_mark == a[len+1]); }
+static inline bool
+	is_AFFIXTYPE_PREFIX(char infix_mark, const char *a, size_t len)
+	{ return infix_mark == a[len-1]; }
+static inline bool
+	is_AFFIXTYPE_SUFFIX(char infix_mark, const char *a)
+	{ return (infix_mark == a[0] && '\0' != a[1]); }
+static inline bool
+	is_AFFIXTYPE_EMPTY(char infix_mark, const char *a, size_t len)
+	{ return (0 == strcmp(a, EMPTY_WORD_MARK)); }
 
 /**
  * This routine solves the problem of mis-linked alternatives,
@@ -1128,17 +979,24 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 {
 	size_t lk, i;
 	Parse_info pi = sent->parse_info;
-	Dictionary afdict = sent->dict->affix_table;
+	Dictionary afdict = sent->dict->affix_table; /* for INFIX_MARK only */
+	char infix_mark = INFIX_MARK;
+	int * matched_alts = NULL;       /* number of morphemes that have matched
+												 * the chosen disjuncts for the
+												 * unsplit_word, so far (index: ai) */
+	size_t matched_alts_num = 0;     /* matched_alts number of elements */
+#define MATCHED_ALTS_MIN_INC 16     /* don't allocate matched_alts often */
 
-	/* Skip checking, if dictionary specifies neither prefixes nor sufixes.
-	 * This special-cases English, more or less. */
+#if 0	/* Now it can always be enabled. Anyway, it is needed for "any". */
+	/* Skip checking, if dictionary specifies neither prefixes nor sufixes. */
 	if (NULL == afdict) return;
 	if ((0 == AFCLASS(afdict, AFDICT_PRE)->length) &&
-		 (0 == AFCLASS(afdict, AFDICT_MPRE)->length) &&
-		 (0 == AFCLASS(afdict, AFDICT_SUF)->length))
+	    (0 == AFCLASS(afdict, AFDICT_MPRE)->length) &&
+	    (0 == AFCLASS(afdict, AFDICT_SUF)->length))
 	{
 		return;
 	}
+#endif
 
 	for (lk = 0; lk < sent->num_linkages_alloced; lk++)
 	{
@@ -1149,10 +1007,7 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 		const char * unsplit = NULL;     /* unsplit word */
 		char affix_types[MAX_SENTENCE+1];/* affix types sequence */
 		char * affix_types_p = affix_types;
-		Boolean match_found = false;     /* djw matched a morpheme */
-		int matched_alts[MAX_SENTENCE];  /* number of morphemes that have matched
-		                                  * the chosen disjuncts for the
-		                                  * unsplit_word, so far (index: ai) */
+		bool match_found = false;        /* djw matched a morpheme */
 
 		/* Don't bother with linkages that already failed post-processing... */
 		if (0 != lifo->N_violations)
@@ -1160,19 +1015,19 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 		extract_links(lifo->index, pi);
 		for (i=0; i<sent->length; i++)
 		{
-			const char * djw;             /* disjunct word - the chosen word */
-			size_t djwlen;                /* disjunct total length */
-			size_t len;                   /* disjunct length w/o subscript*/
-			const char * mark;            /* char position of SUBSCRIPT_MARK */
-			Boolean empty_word = false;   /* is this an empty word? */
+			const char * djw;          /* disjunct word - the chosen word */
+			size_t djwlen;             /* disjunct total length */
+			size_t len;                /* disjunct length w/o subscript */
+			const char * mark;         /* char position of SUBSCRIPT_MARK */
+			bool empty_word = false;   /* is this an empty word? */
 			Disjunct * cdj = pi->chosen_disjuncts[i];
 
-			lgdebug(+4, ">%zu Checking word %zu/%zu\n", lk, i, sent->length);
+			lgdebug(+4, "Linkage %zu, word %zu/%zu\n", lk+1, i, sent->length);
 
 			/* Ignore island words */
 			if (NULL == cdj)
 			{
-				lgdebug(4, "%zu ignored island word\n", lk);
+				lgdebug(4, "%zu ignored island word\n", lk+1);
 				unsplit = NULL; /* mark it as island */
 				continue;
 			}
@@ -1183,14 +1038,24 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 				unsplit_i = i;
 				unsplit = sent->word[i].unsplit_word;
 				numalt = altlen(sent->word[i].alternatives);
+				if (numalt > matched_alts_num)
+				{
+					matched_alts_num = numalt + MATCHED_ALTS_MIN_INC;
+					matched_alts =
+						realloc(matched_alts, sizeof(*matched_alts)*matched_alts_num);
+				}
 
-				for (ai = 0; ai < numalt; ai++) matched_alts[ai] = 0;
-				lgdebug(4, "%zu unsplit word %s, numalt %d\n",
-						lk, unsplit, (int)numalt);
+				lgdebug(4, "%zu unsplit word %s, alts:", lk+1, unsplit);
+				for (ai = 0; ai < numalt; ai++)
+				{
+					matched_alts[ai] = 0;
+					lgdebug(4, " %zu:%s", ai, sent->word[i].alternatives[ai]);
+				}
+				lgdebug(4, "\n");
 			}
 			if (NULL == unsplit)
 			{
-				lgdebug(4, "%zu ignore morphemes of an island word\n", lk);
+				lgdebug(4, "%zu ignore morphemes of an island word\n", lk+1);
 				continue;
 			}
 
@@ -1200,23 +1065,23 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 			len = NULL != mark ? (size_t)(mark - djw) : djwlen;
 
 			/* Find morpheme type */
-			if (is_AFFIXTYPE_EMPTY(djw, djwlen))
+			if (is_AFFIXTYPE_EMPTY(infix_mark, djw, djwlen))
 			{
 				/* Ignore the empty word */;
 				empty_word = true;
 			}
 			else
-			if (is_AFFIXTYPE_SUFFIX(djw))
+			if (is_AFFIXTYPE_SUFFIX(infix_mark, djw))
 			{
 				*affix_types_p = AFFIXTYPE_SUFFIX;
 			}
 			else
-			if (is_AFFIXTYPE_STEM(djw, len))
+			if (is_AFFIXTYPE_STEM(infix_mark, djw, len))
 			{
 				*affix_types_p = AFFIXTYPE_STEM;
 			}
 			else
-			if (is_AFFIXTYPE_PREFIX(djw, len))
+			if (is_AFFIXTYPE_PREFIX(infix_mark, djw, len))
 			{
 				*affix_types_p = AFFIXTYPE_PREFIX;
 			}
@@ -1225,12 +1090,12 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 				*affix_types_p = AFFIXTYPE_WORD;
 			}
 
-			lgdebug(4, "%zu djw=%s affixtype=%c wordlen=%zu\n",
-			        lk, djw, empty_word ? 'E' : *affix_types_p, len);
+			lgdebug(4, "%zu djw=%s affixtype=%c\n",
+			        lk+1, djw, empty_word ? 'E' : *affix_types_p);
 
 			if (! empty_word) affix_types_p++;
 
-			lgdebug(4, "%zu djw %s matched alternatives no.:", lk, djw);
+			lgdebug(4, "%zu djw %s matched alt#:", lk+1, djw);
 			match_found = false;
 
 			/* Compare the chosen word djw to the alternatives */
@@ -1246,18 +1111,19 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 				s = a;
 try_again:
 				t = djw;
-				//lgdebug(4, "\n%d COMPARING alt%d s %s djw %s: ", lk, (int)ai, s, t);
+				//lgdebug(4, "\n%d COMPARING alt%d s %s djw %s: ", lk+1, (int)ai, s, t);
 				/* Rules of match:
 				 * A morpheme with a subscript needs an exact match to djw.
 				 * A morpheme w/o a subscript needs an exact match to djw
 				 * disregarding its subscript.
 				 * XXX To check: words that contain a dot as part of them. */
-				while ((*s != '\0') && (*s == *t)) {s++; t++;}
+				while ((*s != '\0' && *s != '[') && (*s == *t)) {s++; t++;}
 				/* Possibilities:
-				 *** Match (the last one is for words ending with [!]):
+				 *** Match (the last two are for words ending with [...]):
 				 * s==\0 && t==\0
 				 * s==\0 && t==SUBSCRIPT_MARK
 				 * s==\0 && t==[
+				 * s==[ && t==[
 				 *** Continue to check:
 				 * s==.	&& t==SUBSCRIPT_MARK
 				 */
@@ -1266,8 +1132,10 @@ try_again:
 					s++; t++;
 					while ((*s != '\0') && (*s == *t)) {s++; t++;}
 				}
-				if ((*s == '\0') &&
-					((*t == '\0') || (*t == SUBSCRIPT_MARK) || (*t == '[')))
+				if (((*s == '\0') &&
+				   ((*t == '\0') || (*t == SUBSCRIPT_MARK) || (*t == '[')))
+				   ||
+				   ((*s == '[') && (*t == '[')))
 				{
 					//lgdebug(4, "EQUAL\n");
 					lgdebug(4, " %zu", ai);
@@ -1284,7 +1152,7 @@ try_again:
 					{
 						downcase_utf8_str(downcased, a, MAX_WORD);
 						lgdebug(4, "\n");
-						lgdebug(4, "%zu downcasing %s>%s\n", lk, a, downcased);
+						lgdebug(4, "%zu downcasing %s>%s\n", lk+1, a, downcased);
 						s = downcased;
 						goto try_again;
 					}
@@ -1306,7 +1174,7 @@ try_again:
 				int num_morphemes = i - unsplit_i + 1;
 
 				lgdebug(4, "%zu end of input word, num_morphemes %d\n",
-						lk, num_morphemes);
+						lk+1, num_morphemes);
 				*affix_types_p++ = AFFIXTYPE_END;
 
 				/* Make sure that there exists an alternative in
@@ -1325,10 +1193,10 @@ try_again:
 
 				if (! match_found)
 				{
-					lgdebug(4, "%zu morphemes are missing in this linkage\n", lk);
+					lgdebug(4, "%zu morphemes are missing in this linkage\n", lk+1);
 					break;
 				}
-				lgdebug(4, "%zu Perfect match\n", lk);
+				lgdebug(4, "%zu Perfect match\n", lk+1);
 			}
 		}
 		*affix_types_p = '\0';
@@ -1338,21 +1206,21 @@ try_again:
 		 * due to morpheme islands, so skip this check. */
 		if (match_found && 0 == sent->null_count &&
 			(NULL != afdict) && (NULL != afdict->regex_root) &&
-			(NULL == match_regex(afdict, affix_types)))
+			(NULL == match_regex(afdict->regex_root, affix_types)))
 		{
 			/* Morpheme type combination not valid */
 			match_found = false;
 			/* XXX we should have a better way to notify */
 			if (0 < verbosity)
 				printf("Warning: Invalid morpheme type combination %s, "
-				       "run with verbosity=4 to debug\n", affix_types);
+				       "run with !bad and !verbosity=4 to debug\n", affix_types);
 		}
 		else
-			lgdebug(4, "%zu morpheme type combination %s\n", lk, affix_types);
+			lgdebug(4, "%zu morpheme type combination %s\n", lk+1, affix_types);
 
 		if (match_found)
 		{
-			lgdebug(4, "%zu SUCCEEDED\n", lk);
+			lgdebug(4, "%zu SUCCEEDED\n", lk+1);
 		}
 		else
 		{
@@ -1360,20 +1228,28 @@ try_again:
 			sent->num_valid_linkages --;
 			lifo->N_violations ++;
 			lifo->pp_violation_msg = "Invalid morphism construction.";
-			lgdebug(4, "%zu FAILED, remaining %zu\n", lk, sent->num_valid_linkages);
+			lgdebug(4, "%zu FAILED, remaining %zu\n", lk+1,
+			        sent->num_valid_linkages);
 		}
 	}
+
+	free(matched_alts);
 }
 
 static void chart_parse(Sentence sent, Parse_Options opts)
 {
 	int nl;
+	match_context_t * mchxt;
+	count_context_t * ctxt;
 
 	/* Build lists of disjuncts */
 	prepare_to_parse(sent, opts);
+	if (resources_exhausted(opts->resources)) return;
 
-	init_fast_matcher(sent);
-	init_count(sent);
+	mchxt = alloc_fast_matcher(sent);
+	ctxt = alloc_count_context(sent->length);
+	print_time(opts, "Initialized fast matcher");
+	if (resources_exhausted(opts->resources)) return;
 
 	/* A parse set may have been already been built for this sentence,
 	 * if it was previously parsed.  If so we free it up before
@@ -1386,7 +1262,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		s64 total;
 		if (resources_exhausted(opts->resources)) break;
 		sent->null_count = nl;
-		total = do_parse(sent, sent->null_count, opts);
+		total = do_parse(sent, mchxt, ctxt, sent->null_count, opts);
 
 		if (verbosity > 1)
 		{
@@ -1401,7 +1277,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		sent->num_linkages_found = (int) total;
 		print_time(opts, "Counted parses");
 
-		post_process_linkages(sent, opts);
+		post_process_linkages(sent, mchxt, ctxt, opts);
 		sane_morphism(sent, opts);
 		if (sent->num_valid_linkages > 0) break;
 
@@ -1410,8 +1286,8 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		if (PARSE_NUM_OVERFLOW < total) break;
 	}
 
-	free_count(sent);
-	free_fast_matcher(sent);
+	free_count_context(ctxt);
+	free_fast_matcher(mchxt);
 }
 
 static void free_sentence_disjuncts(Sentence sent)
@@ -1433,20 +1309,7 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	debug = opts->debug;
 	test = opts->test;
 
-	if ('\0' != test[0]) /* remind the developer this is not a normal mode */
-	{
-		/* Static variable - reentrancy is not needed */
-		static Boolean batch_in_progress = FALSE;
-
-		/* In batch mode warn only once */
-		if (! batch_in_progress)
-		{
-			fflush(stdout);
-			fprintf(stderr, "Warning: Tests enabled: %s\n", test);
-			if (parse_options_get_batch_mode(opts))
-				batch_in_progress = 1;
-		}
-	}
+	sent->num_valid_linkages = 0;
 
 	/* If the sentence has not yet been split, do so now.
 	 * This is for backwards compatibility, for existing programs
@@ -1473,10 +1336,8 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 #endif /* USE_FAT_LINKAGES */
 	resources_reset_space(opts->resources);
 
-	if (resources_exhausted(opts->resources)) {
-		sent->num_valid_linkages = 0;
+	if (resources_exhausted(opts->resources))
 		return 0;
-	}
 
 #ifdef USE_FAT_LINKAGES
 	init_analyze(sent);
