@@ -42,13 +42,13 @@ struct count_context_s
 #endif /* USE_FAT_LINKAGES */
 	Word *  local_sent;
 	int     null_block;
-	Boolean islands_ok;
-	Boolean null_links;
+	bool    islands_ok;
+	bool    null_links;
 	int     table_size;
 	int     log2_table_size;
 	Table_connector ** table;
 	Resources current_resources;
-	Boolean exhausted;
+	bool    exhausted;
 	int     checktimer;  /* Avoid excess system calls */
 };
 
@@ -70,20 +70,18 @@ static void free_table(count_context_t *ctxt)
 	ctxt->table_size = 0;
 }
 
-static void init_table(Sentence sent)
+static void init_table(count_context_t *ctxt, size_t sent_len)
 {
 	unsigned int shift;
 	/* A piecewise exponential function determines the size of the
 	 * hash table. Probably should make use of the actual number of
 	 * disjuncts, rather than just the number of words.
 	 */
-	count_context_t *ctxt = sent->count_ctxt;
-
 	if (ctxt->table) free_table(ctxt);
 
-	if (sent->length >= 10)
+	if (sent_len >= 10)
 	{
-		shift = 12 + (sent->length) / 6 ; 
+		shift = 12 + (sent_len) / 6 ; 
 	} 
 	else
 	{
@@ -100,14 +98,14 @@ static void init_table(Sentence sent)
 }
 
 #ifdef USE_FAT_LINKAGES
-void count_set_effective_distance(Sentence sent)
+void count_set_effective_distance(count_context_t* ctxt, Sentence sent)
 {
-	sent->count_ctxt->effective_dist = sent->effective_dist;
+	ctxt->effective_dist = sent->effective_dist;
 }
 
-void count_unset_effective_distance(Sentence sent)
+void count_unset_effective_distance(count_context_t* ctxt)
 {
-	sent->count_ctxt->effective_dist = NULL;
+	ctxt->effective_dist = NULL;
 }
 
 /*
@@ -123,7 +121,7 @@ void count_unset_effective_distance(Sentence sent)
  * is different depending on which of the two priority cases is being
  * considered.  See the comments below. 
  */
-int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
+bool do_match(count_context_t *ctxt, Connector *a, Connector *b, int aw, int bw)
 {
 	const char *s, *t;
 	int x, y;
@@ -134,7 +132,7 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 	s = a->string;
 	t = b->string;
 
-	while(isupper((int)*s) || isupper((int)*t))
+	while (isupper((int)*s) || isupper((int)*t))
 	{
 		if (*s != *t) return FALSE;
 		s++;
@@ -149,8 +147,6 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 	if (aw == 0 && bw == 0) {
 		dist = 0;
 	} else {
-		count_context_t *ctxt;
-		ctxt = sent->count_ctxt;
 		assert(aw < bw, "match() did not receive params in the natural order.");
 		dist = ctxt->effective_dist[aw][bw];
 	}
@@ -224,17 +220,14 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 
 /*
  * Returns TRUE if s and t match according to the connector matching
- * rules.  The connector strings must be properly formed, starting with
- * zero or more upper case letters, followed by some other letters, and
- * The algorithm is symmetric with respect to a and b.
- *
- * It works as follows:  The labels must match.  
- * The sequence of upper case letters must match exactly.  After these comes
- * a sequence of lower case letters or "*"s. 
+ * rules.
  */
-int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
+bool do_match(count_context_t* ctxt, Connector *a, Connector *b, int aw, int bw)
 {
-   return prune_match(bw - aw, a, b);
+	int dist = bw - aw;
+	assert(aw < bw, "do_match() did not receive params in the natural order.");
+	if (dist > a->length_limit || dist > b->length_limit) return false;
+	return easy_match(a->string, b->string);
 }
 #endif /* not USE_FAT_LINKAGES */
 
@@ -292,10 +285,10 @@ find_table_pointer(count_context_t *ctxt,
 }
 
 /** returns the count for this quintuple if there, -1 otherwise */
-s64 table_lookup(Sentence sent, 
+s64 table_lookup(count_context_t * ctxt, 
                  int lw, int rw, Connector *le, Connector *re, unsigned int cost)
 {
-	Table_connector *t = find_table_pointer(sent->count_ctxt, lw, rw, le, re, cost);
+	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, cost);
 
 	if (t == NULL) return -1; else return t->count;
 }
@@ -320,15 +313,17 @@ static void table_update(count_context_t *ctxt, int lw, int rw,
  * Returns 0 if and only if this entry is in the hash table 
  * with a count value of 0.
  */
-static s64 pseudocount(Sentence sent,
+static s64 pseudocount(count_context_t * ctxt,
                        int lw, int rw, Connector *le, Connector *re, unsigned int cost)
 {
 	s64 count;
-	count = table_lookup(sent, lw, rw, le, re, cost);
+	count = table_lookup(ctxt, lw, rw, le, re, cost);
 	if (count == 0) return 0; else return 1;
 }
 
-static s64 do_count(Sentence sent, int lw, int rw,
+static s64 do_count(match_context_t *mchxt, 
+                    count_context_t *ctxt,
+                    int lw, int rw,
                     Connector *le, Connector *re, int null_count)
 {
 	s64 total;
@@ -340,8 +335,6 @@ static s64 do_count(Sentence sent, int lw, int rw,
 	Match_node * m, *m1;
 	Table_connector *t;
 
-	count_context_t *ctxt = sent->count_ctxt;
-
 	if (null_count < 0) return 0;  /* can this ever happen?? */
 
 	t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
@@ -404,10 +397,10 @@ static s64 do_count(Sentence sent, int lw, int rw,
 			{
 				if (d->left == NULL)
 				{
-					total += do_count(sent, w, rw, d->right, NULL, null_count-1);
+					total += do_count(mchxt, ctxt, w, rw, d->right, NULL, null_count-1);
 				}
 			}
-			total += do_count(sent, w, rw, NULL, NULL, null_count-1);
+			total += do_count(mchxt, ctxt, w, rw, NULL, NULL, null_count-1);
 			t->count = total;
 		}
 		return t->count;
@@ -435,7 +428,7 @@ static s64 do_count(Sentence sent, int lw, int rw,
 
 	for (w = start_word; w < end_word; w++)
 	{
-		m1 = m = form_match_list(sent, w, le, lw, re, rw);
+		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m != NULL; m = m->next)
 		{
 			unsigned int null_count_p1;
@@ -451,25 +444,25 @@ static s64 do_count(Sentence sent, int lw, int rw,
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
 				Lmatch = (le != NULL) && (d->left != NULL) && 
-				         do_match(sent, le, d->left, lw, w);
+				         do_match(ctxt, le, d->left, lw, w);
 				Rmatch = (d->right != NULL) && (re != NULL) && 
-				         do_match(sent, d->right, re, w, rw);
+				         do_match(ctxt, d->right, re, w, rw);
 
 				rightcount = leftcount = 0;
 				if (Lmatch)
 				{
-					leftcount = pseudocount(sent, lw, w, le->next, d->left->next, lcost);
-					if (le->multi) leftcount += pseudocount(sent, lw, w, le, d->left->next, lcost);
-					if (d->left->multi) leftcount += pseudocount(sent, lw, w, le->next, d->left, lcost);
-					if (le->multi && d->left->multi) leftcount += pseudocount(sent, lw, w, le, d->left, lcost);
+					leftcount = pseudocount(ctxt, lw, w, le->next, d->left->next, lcost);
+					if (le->multi) leftcount += pseudocount(ctxt, lw, w, le, d->left->next, lcost);
+					if (d->left->multi) leftcount += pseudocount(ctxt, lw, w, le->next, d->left, lcost);
+					if (le->multi && d->left->multi) leftcount += pseudocount(ctxt, lw, w, le, d->left, lcost);
 				}
 
 				if (Rmatch)
 				{
-					rightcount = pseudocount(sent, w, rw, d->right->next, re->next, rcost);
-					if (d->right->multi) rightcount += pseudocount(sent, w,rw,d->right,re->next, rcost);
-					if (re->multi) rightcount += pseudocount(sent, w, rw, d->right->next, re, rcost);
-					if (d->right->multi && re->multi) rightcount += pseudocount(sent, w, rw, d->right, re, rcost);
+					rightcount = pseudocount(ctxt, w, rw, d->right->next, re->next, rcost);
+					if (d->right->multi) rightcount += pseudocount(ctxt, w,rw,d->right,re->next, rcost);
+					if (re->multi) rightcount += pseudocount(ctxt, w, rw, d->right->next, re, rcost);
+					if (d->right->multi && re->multi) rightcount += pseudocount(ctxt, w, rw, d->right, re, rcost);
 				}
 
 				/* total number where links are used on both sides */
@@ -477,39 +470,39 @@ static s64 do_count(Sentence sent, int lw, int rw,
 
 				if (leftcount > 0) {
 					/* evaluate using the left match, but not the right */
-					pseudototal += leftcount * pseudocount(sent, w, rw, d->right, re, rcost);
+					pseudototal += leftcount * pseudocount(ctxt, w, rw, d->right, re, rcost);
 				}
 				if ((le == NULL) && (rightcount > 0)) {
 					/* evaluate using the right match, but not the left */
-					pseudototal += rightcount * pseudocount(sent, lw, w, le, d->left, lcost);
+					pseudototal += rightcount * pseudocount(ctxt, lw, w, le, d->left, lcost);
 				}
 
 				/* now pseudototal is 0 implies that we know that the true total is 0 */
 				if (pseudototal != 0) {
 					rightcount = leftcount = 0;
 					if (Lmatch) {
-						leftcount = do_count(sent, lw, w, le->next, d->left->next, lcost);
-						if (le->multi) leftcount += do_count(sent, lw, w, le, d->left->next, lcost);
-						if (d->left->multi) leftcount += do_count(sent, lw, w, le->next, d->left, lcost);
-						if (le->multi && d->left->multi) leftcount += do_count(sent, lw, w, le, d->left, lcost);
+						leftcount = do_count(mchxt, ctxt, lw, w, le->next, d->left->next, lcost);
+						if (le->multi) leftcount += do_count(mchxt, ctxt, lw, w, le, d->left->next, lcost);
+						if (d->left->multi) leftcount += do_count(mchxt, ctxt, lw, w, le->next, d->left, lcost);
+						if (le->multi && d->left->multi) leftcount += do_count(mchxt, ctxt, lw, w, le, d->left, lcost);
 					}
 
 					if (Rmatch) {
-						rightcount = do_count(sent, w, rw, d->right->next, re->next, rcost);
-						if (d->right->multi) rightcount += do_count(sent, w,rw,d->right,re->next, rcost);
-						if (re->multi) rightcount += do_count(sent, w, rw, d->right->next, re, rcost);
-						if (d->right->multi && re->multi) rightcount += do_count(sent, w, rw, d->right, re, rcost);
+						rightcount = do_count(mchxt, ctxt, w, rw, d->right->next, re->next, rcost);
+						if (d->right->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right,re->next, rcost);
+						if (re->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right->next, re, rcost);
+						if (d->right->multi && re->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right, re, rcost);
 					}
 
 					total += leftcount*rightcount;  /* total number where links are used on both sides */
 
 					if (leftcount > 0) {
 						/* evaluate using the left match, but not the right */
-						total += leftcount * do_count(sent, w, rw, d->right, re, rcost);
+						total += leftcount * do_count(mchxt, ctxt, w, rw, d->right, re, rcost);
 					}
 					if ((le == NULL) && (rightcount > 0)) {
 						/* evaluate using the right match, but not the left */
-						total += rightcount * do_count(sent, lw, w, le, d->left, lcost);
+						total += rightcount * do_count(mchxt, ctxt, lw, w, le, d->left, lcost);
 					}
 
 					/* Sigh. Overflows can and do occur, esp for the ANY language. */
@@ -517,13 +510,13 @@ static s64 do_count(Sentence sent, int lw, int rw,
 					{
 						total = INT_MAX;
 						t->count = total;
-						put_match_list(sent, m1);
+						put_match_list(mchxt, m1);
 						return total;
 					}
 				}
 			}
 		}
-		put_match_list(sent, m1);
+		put_match_list(mchxt, m1);
 	}
 	t->count = total;
 	return total;
@@ -536,23 +529,28 @@ static s64 do_count(Sentence sent, int lw, int rw,
  * initialized, and is freed later. The "null_count" here is the
  * number of words that are allowed to have no links to them.
  */
-s64 do_parse(Sentence sent, int null_count, Parse_Options opts)
+s64 do_parse(Sentence sent,
+             match_context_t *mchxt,
+             count_context_t *ctxt,
+             int null_count, Parse_Options opts)
 {
 	s64 total;
-	count_context_t *ctxt = sent->count_ctxt;
 
 	ctxt->current_resources = opts->resources;
 	ctxt->exhausted = resources_exhausted(ctxt->current_resources);
 	ctxt->checktimer = 0;
 	ctxt->local_sent = sent->word;
 #ifdef USE_FAT_LINKAGES
-	count_set_effective_distance(sent);
+	count_set_effective_distance(ctxt, sent);
 	ctxt->deletable = sent->deletable;
 #endif /* USE_FAT_LINKAGES */
-	ctxt->null_block = opts->null_block;
+
+	/* consecutive blocks of this many words are considered as
+	 * one null link. */
+	ctxt->null_block = 1;
 	ctxt->islands_ok = opts->islands_ok;
 
-	total = do_count(sent, -1, sent->length, NULL, NULL, null_count+1);
+	total = do_count(mchxt, ctxt, -1, sent->length, NULL, NULL, null_count+1);
 
 	ctxt->local_sent = NULL;
 	ctxt->current_resources = NULL;
@@ -607,7 +605,8 @@ static int x_prune_match(count_context_t *ctxt,
  * links.  Returns 1 if it can, and it's not been marked, and returns
  * 2 if it can and it has been marked.
  */
-static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector *re)
+static int region_valid(match_context_t *mchxt, count_context_t *ctxt,
+                        int lw, int rw, Connector *le, Connector *re)
 {
 	Disjunct * d;
 	int left_valid, right_valid, found;
@@ -615,9 +614,7 @@ static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector
 	int w;
 	Match_node * m, *m1;
 
-	count_context_t *ctxt = sent->count_ctxt;
-
-	i = table_lookup(sent, lw, rw, le, re, 0);
+	i = table_lookup(ctxt, lw, rw, le, re, 0);
 	if (i >= 0) return i;
 
 	if ((le == NULL) && (re == NULL) && ctxt->deletable[lw][rw]) {
@@ -640,7 +637,7 @@ static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector
 
 	for (w=start_word; w < end_word; w++)
 	{
-		m1 = m = form_match_list(sent, w, le, lw, re, rw);
+		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next)
 		{
 			d = m->d;
@@ -648,25 +645,25 @@ static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector
 			/* in the following expressions we use the fact that 0=FALSE. Could eliminate
 			   by always saying "region_valid(...) != 0"  */
 			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(ctxt, le, d->left, lw, w)) &&
-						  ((region_valid(sent, lw, w, le->next, d->left->next)) ||
-						   ((le->multi) && region_valid(sent, lw, w, le, d->left->next)) ||
-						   ((d->left->multi) && region_valid(sent, lw, w, le->next, d->left)) ||
-						   ((le->multi && d->left->multi) && region_valid(sent, lw, w, le, d->left))));
-			if (left_valid && region_valid(sent, w, rw, d->right, re)) {
+						  ((region_valid(mchxt, ctxt, lw, w, le->next, d->left->next)) ||
+						   ((le->multi) && region_valid(mchxt, ctxt, lw, w, le, d->left->next)) ||
+						   ((d->left->multi) && region_valid(mchxt, ctxt, lw, w, le->next, d->left)) ||
+						   ((le->multi && d->left->multi) && region_valid(mchxt, ctxt, lw, w, le, d->left))));
+			if (left_valid && region_valid(mchxt, ctxt, w, rw, d->right, re)) {
 				found = 1;
 				break;
 			}
 			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(ctxt, d->right, re, w, rw)) &&
-						   ((region_valid(sent, w, rw, d->right->next,re->next))	||
-							((d->right->multi) && region_valid(sent, w,rw,d->right,re->next))  ||
-							((re->multi) && region_valid(sent, w, rw, d->right->next, re))  ||
-							((d->right->multi && re->multi) && region_valid(sent, w, rw, d->right, re))));
-			if ((left_valid && right_valid) || (right_valid && region_valid(sent, lw, w, le, d->left))) {
+						   ((region_valid(mchxt, ctxt, w, rw, d->right->next,re->next))	||
+							((d->right->multi) && region_valid(mchxt, ctxt, w, rw, d->right,re->next))  ||
+							((re->multi) && region_valid(mchxt, ctxt, w, rw, d->right->next, re))  ||
+							((d->right->multi && re->multi) && region_valid(mchxt, ctxt, w, rw, d->right, re))));
+			if ((left_valid && right_valid) || (right_valid && region_valid(mchxt, ctxt, lw, w, le, d->left))) {
 				found = 1;
 				break;
 			}
 		}
-		put_match_list(sent, m1);
+		put_match_list(mchxt, m1);
 		if (found != 0) break;
 	}
 	table_store(ctxt, lw, rw, le, re, 0, found);
@@ -680,7 +677,7 @@ static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector
  * this region itself is not valid, then this fact will be recorded
  * in the table, and nothing else happens.
  */
-static void mark_region(Sentence sent,
+static void mark_region(match_context_t *mchxt, count_context_t *ctxt,
                         int lw, int rw, Connector *le, Connector *re)
 {
 
@@ -689,9 +686,8 @@ static void mark_region(Sentence sent,
 	int start_word, end_word;
 	int w;
 	Match_node * m, *m1;
-	count_context_t *ctxt = sent->count_ctxt;
 
-	i = region_valid(sent, lw, rw, le, re);
+	i = region_valid(mchxt, ctxt, lw, rw, le, re);
 	if ((i==0) || (i==2)) return;
 	/* we only reach this point if it's a valid unmarked region, i=1 */
 	table_update(ctxt, lw, rw, le, re, 0, 2);
@@ -699,12 +695,12 @@ static void mark_region(Sentence sent,
 	if ((le == NULL) && (re == NULL) && (ctxt->null_links) && (rw != 1+lw)) {
 		w = lw+1;
 		for (d = ctxt->local_sent[w].d; d != NULL; d = d->next) {
-			if ((d->left == NULL) && region_valid(sent, w, rw, d->right, NULL)) {
+			if ((d->left == NULL) && region_valid(mchxt, ctxt, w, rw, d->right, NULL)) {
 				d->marked = TRUE;
-				mark_region(sent, w, rw, d->right, NULL);
+				mark_region(mchxt, ctxt, w, rw, d->right, NULL);
 			}
 		}
-		mark_region(sent, w, rw, NULL, NULL);
+		mark_region(mchxt, ctxt, w, rw, NULL, NULL);
 		return;
 	}
 
@@ -721,60 +717,60 @@ static void mark_region(Sentence sent,
 
 	for (w=start_word; w < end_word; w++)
 	{
-		m1 = m = form_match_list(sent, w, le, lw, re, rw);
+		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next)
 		{
 			d = m->d;
 			/* mark_cost++;*/
 			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(ctxt, le, d->left, lw, w)) &&
-						  ((region_valid(sent, lw, w, le->next, d->left->next)) ||
-						   ((le->multi) && region_valid(sent, lw, w, le, d->left->next)) ||
-						   ((d->left->multi) && region_valid(sent, lw, w, le->next, d->left)) ||
-						   ((le->multi && d->left->multi) && region_valid(sent, lw, w, le, d->left))));
+						  ((region_valid(mchxt, ctxt, lw, w, le->next, d->left->next)) ||
+						   ((le->multi) && region_valid(mchxt, ctxt, lw, w, le, d->left->next)) ||
+						   ((d->left->multi) && region_valid(mchxt, ctxt, lw, w, le->next, d->left)) ||
+						   ((le->multi && d->left->multi) && region_valid(mchxt, ctxt, lw, w, le, d->left))));
 			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(ctxt, d->right, re, w, rw)) &&
-						   ((region_valid(sent, w, rw, d->right->next,re->next)) ||
-							((d->right->multi) && region_valid(sent, w,rw,d->right,re->next))  ||
-							((re->multi) && region_valid(sent, w, rw, d->right->next, re)) ||
-							((d->right->multi && re->multi) && region_valid(sent, w, rw, d->right, re))));
+						   ((region_valid(mchxt, ctxt, w, rw, d->right->next,re->next)) ||
+							((d->right->multi) && region_valid(mchxt, ctxt, w, rw, d->right,re->next))  ||
+							((re->multi) && region_valid(mchxt, ctxt, w, rw, d->right->next, re)) ||
+							((d->right->multi && re->multi) && region_valid(mchxt, ctxt, w, rw, d->right, re))));
 
 			/* The following if statements could be restructured to avoid superfluous calls
 			   to mark_region.  It didn't seem a high priority, so I didn't optimize this.
 			   */
 
-			if (left_valid && region_valid(sent, w, rw, d->right, re))
+			if (left_valid && region_valid(mchxt, ctxt, w, rw, d->right, re))
 			{
 				d->marked = TRUE;
-				mark_region(sent, w, rw, d->right, re);
-				mark_region(sent, lw, w, le->next, d->left->next);
-				if (le->multi) mark_region(sent, lw, w, le, d->left->next);
-				if (d->left->multi) mark_region(sent, lw, w, le->next, d->left);
-				if (le->multi && d->left->multi) mark_region(sent, lw, w, le, d->left);
+				mark_region(mchxt, ctxt, w, rw, d->right, re);
+				mark_region(mchxt, ctxt, lw, w, le->next, d->left->next);
+				if (le->multi) mark_region(mchxt, ctxt, lw, w, le, d->left->next);
+				if (d->left->multi) mark_region(mchxt, ctxt, lw, w, le->next, d->left);
+				if (le->multi && d->left->multi) mark_region(mchxt, ctxt, lw, w, le, d->left);
 			}
 
-			if (right_valid && region_valid(sent, lw, w, le, d->left))
+			if (right_valid && region_valid(mchxt, ctxt, lw, w, le, d->left))
 			{
 				d->marked = TRUE;
-				mark_region(sent, lw, w, le, d->left);
-				mark_region(sent, w, rw, d->right->next,re->next);
-				if (d->right->multi) mark_region(sent, w,rw,d->right,re->next);
-				if (re->multi) mark_region(sent, w, rw, d->right->next, re);
-				if (d->right->multi && re->multi) mark_region(sent, w, rw, d->right, re);
+				mark_region(mchxt, ctxt, lw, w, le, d->left);
+				mark_region(mchxt, ctxt, w, rw, d->right->next,re->next);
+				if (d->right->multi) mark_region(mchxt, ctxt, w, rw, d->right, re->next);
+				if (re->multi) mark_region(mchxt, ctxt, w, rw, d->right->next, re);
+				if (d->right->multi && re->multi) mark_region(mchxt, ctxt, w, rw, d->right, re);
 			}
 
 			if (left_valid && right_valid)
 			{
 				d->marked = TRUE;
-				mark_region(sent, lw, w, le->next, d->left->next);
-				if (le->multi) mark_region(sent, lw, w, le, d->left->next);
-				if (d->left->multi) mark_region(sent, lw, w, le->next, d->left);
-				if (le->multi && d->left->multi) mark_region(sent, lw, w, le, d->left);
-				mark_region(sent, w, rw, d->right->next,re->next);
-				if (d->right->multi) mark_region(sent, w,rw,d->right,re->next);
-				if (re->multi) mark_region(sent, w, rw, d->right->next, re);
-				if (d->right->multi && re->multi) mark_region(sent, w, rw, d->right, re);
+				mark_region(mchxt, ctxt, lw, w, le->next, d->left->next);
+				if (le->multi) mark_region(mchxt, ctxt, lw, w, le, d->left->next);
+				if (d->left->multi) mark_region(mchxt, ctxt, lw, w, le->next, d->left);
+				if (le->multi && d->left->multi) mark_region(mchxt, ctxt, lw, w, le, d->left);
+				mark_region(mchxt, ctxt, w, rw, d->right->next,re->next);
+				if (d->right->multi) mark_region(mchxt, ctxt, w, rw, d->right, re->next);
+				if (re->multi) mark_region(mchxt, ctxt, w, rw, d->right->next, re);
+				if (d->right->multi && re->multi) mark_region(mchxt, ctxt, w, rw, d->right, re);
 			}
 		}
-		put_match_list(sent, m1);
+		put_match_list(mchxt, m1);
 	}
 }
 #endif /* USE_FAT_LINKAGES */
@@ -811,15 +807,14 @@ void delete_unmarked_disjuncts(Sentence sent)
  * marking them all as irrelevant, and then marking the ones that
  * might be useable.  Finally, the unmarked ones are removed.
  */
-void conjunction_prune(Sentence sent, Parse_Options opts)
+void conjunction_prune(Sentence sent, count_context_t *ctxt, Parse_Options opts)
 {
 	Disjunct * d;
 	int w;
-	count_context_t *ctxt = sent->count_ctxt;
 
 	ctxt->current_resources = opts->resources;
 	ctxt->deletable = sent->deletable;
-	count_set_effective_distance(sent);
+	count_set_effective_distance(ctxt, sent);
 
 	/* We begin by unmarking all disjuncts.  This would not be necessary if
 	   whenever we created a disjunct we cleared its marked field.
@@ -832,29 +827,29 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 		}
 	}
 
-	init_fast_matcher(sent);
+	match_context_t *mchxt = alloc_fast_matcher(sent);
 	ctxt->local_sent = sent->word;
 	ctxt->null_links = (opts->min_null_count > 0);
 	/*
 	for (d = sent->word[0].d; d != NULL; d = d->next) {
-		if ((d->left == NULL) && region_valid(sent, 0, sent->length, d->right, NULL)) {
-			mark_region(sent, 0, sent->length, d->right, NULL);
+		if ((d->left == NULL) && region_valid(mchxt, ctxt, 0, sent->length, d->right, NULL)) {
+			mark_region(mchxt, ctxt, 0, sent->length, d->right, NULL);
 			d->marked = TRUE;
 		}
 	}
-	mark_region(sent, 0, sent->length, NULL, NULL);
+	mark_region(mchxt, ctxt, 0, sent->length, NULL, NULL);
 	*/
 
 	if (ctxt->null_links) {
-		mark_region(sent, -1, sent->length, NULL, NULL);
+		mark_region(mchxt, ctxt, -1, sent->length, NULL, NULL);
 	} else {
 		for (w=0; w<sent->length; w++) {
 		  /* consider removing the words [0,w-1] from the beginning
 			 of the sentence */
 			if (ctxt->deletable[-1][w]) {
 				for (d = sent->word[w].d; d != NULL; d = d->next) {
-					if ((d->left == NULL) && region_valid(sent, w, sent->length, d->right, NULL)) {
-						mark_region(sent, w, sent->length, d->right, NULL);
+					if ((d->left == NULL) && region_valid(mchxt, ctxt, w, sent->length, d->right, NULL)) {
+						mark_region(mchxt, ctxt, w, sent->length, d->right, NULL);
 						d->marked = TRUE;
 					}
 				}
@@ -864,30 +859,28 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 
 	delete_unmarked_disjuncts(sent);
 
-	free_fast_matcher(sent);
+	free_fast_matcher(mchxt);
 
 	ctxt->local_sent = NULL;
 	ctxt->current_resources = NULL;
 	ctxt->checktimer = 0;
 	ctxt->deletable = NULL;
-	count_unset_effective_distance(sent);
+	count_unset_effective_distance(ctxt);
 }
 #endif /* USE_FAT_LINKAGES */
 
-void init_count(Sentence sent)
+/* sent_length is used only as a hint for the hash table size ... */
+count_context_t * alloc_count_context(size_t sent_length)
 {
-	if (NULL == sent->count_ctxt)
-		sent->count_ctxt = (count_context_t *) xalloc (sizeof(count_context_t));
-	memset(sent->count_ctxt, 0, sizeof(count_context_t));
+	count_context_t *ctxt = (count_context_t *) xalloc (sizeof(count_context_t));
+	memset(ctxt, 0, sizeof(count_context_t));
 
-	init_table(sent);
+	init_table(ctxt, sent_length);
+	return ctxt;
 }
 
-void free_count(Sentence sent)
+void free_count_context(count_context_t *ctxt)
 {
-	if (NULL == sent->count_ctxt) return;
-
-	free_table(sent->count_ctxt);
-	xfree(sent->count_ctxt, sizeof(count_context_t));
-	sent->count_ctxt = NULL;
+	free_table(ctxt);
+	xfree(ctxt, sizeof(count_context_t));
 }
