@@ -112,6 +112,10 @@ Parse_info parse_info_new(int nwords)
 	pi = (Parse_info) xalloc(sizeof(struct Parse_info_struct));
 	memset(pi, 0, sizeof(struct Parse_info_struct));
 	pi->N_words = nwords;
+
+	/* very unlikely that more than this many links will be needed...*/
+	pi->lasz = 2 * nwords;
+	pi->link_array = (Link*) xalloc(pi->lasz * sizeof(Link));
 	pi->parse_set = NULL;
 
 	pi->chosen_disjuncts = (Disjunct **) xalloc(nwords * sizeof(Disjunct *));
@@ -154,6 +158,8 @@ void free_parse_info(Parse_info pi)
 	int len;
 	X_table_connector *t, *x;
 
+	xfree(pi->link_array, pi->lasz * sizeof(Link));
+
 	len = pi->N_words;
 	xfree(pi->chosen_disjuncts, len * sizeof(Disjunct *));
 #ifdef USE_FAT_LINKAGES
@@ -243,7 +249,9 @@ static void x_table_update(int lw, int rw, Connector *le, Connector *re,
  * This code is similar to code in count.c
  * (grep for end_word in these files).
  */
-static Parse_set * parse_set(Sentence sent,
+static
+Parse_set * mk_parse_set(Sentence sent, match_context_t *mchxt,
+                 count_context_t * ctxt,
                  Disjunct *ld, Disjunct *rd, int lw, int rw,
                  Connector *le, Connector *re, unsigned int cost,
                  Boolean islands_ok, Parse_info pi)
@@ -260,14 +268,14 @@ static Parse_set * parse_set(Sentence sent,
 	X_table_connector *xt;
 	s64 count;
 
-	assert(cost < 0x7fff, "parse_set() called with cost < 0.");
+	assert(cost < 0x7fff, "mk_parse_set() called with cost < 0.");
 
-	count = table_lookup(sent, lw, rw, le, re, cost);
+	count = table_lookup(ctxt, lw, rw, le, re, cost);
 
 	/*
-	  assert(count >= 0, "parse_set() called on params that were not in the table.");
+	  assert(count >= 0, "mk_parse_set() called on params that were not in the table.");
 	  Actually, we can't assert this, because of the pseudocount technique that's
-	  used in count().  It's not the case that every call to parse_set() has already
+	  used in count().  It's not the case that every call to mk_parse_set() has already
 	  been put into the table.
 	 */
 
@@ -297,7 +305,7 @@ static Parse_set * parse_set(Sentence sent,
 		{
 			if (dis->left == NULL)
 			{
-				rs[0] = parse_set(sent, dis, NULL, w, rw, dis->right,
+				rs[0] = mk_parse_set(sent, mchxt, ctxt, dis, NULL, w, rw, dis->right,
 				                  NULL, cost-1, islands_ok, pi);
 				if (rs[0] == NULL) continue;
 				a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
@@ -306,7 +314,7 @@ static Parse_set * parse_set(Sentence sent,
 				put_choice_in_set(xt->set, a_choice);
 			}
 		}
-		rs[0] = parse_set(sent, NULL, NULL, w, rw, NULL, NULL,
+		rs[0] = mk_parse_set(sent, mchxt, ctxt, NULL, NULL, w, rw, NULL, NULL,
 		                  cost-1, islands_ok, pi);
 		if (rs[0] != NULL)
 		{
@@ -338,7 +346,7 @@ static Parse_set * parse_set(Sentence sent,
 
 	for (w = start_word; w < end_word; w++)
 	{
-		m1 = m = form_match_list(sent, w, le, lw, re, rw);
+		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next)
 		{
 			d = m->d;
@@ -351,22 +359,22 @@ static Parse_set * parse_set(Sentence sent,
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
 
-				Lmatch = (le != NULL) && (d->left != NULL) && do_match(sent, le, d->left, lw, w);
-				Rmatch = (d->right != NULL) && (re != NULL) && do_match(sent, d->right, re, w, rw);
+				Lmatch = (le != NULL) && (d->left != NULL) && do_match(ctxt, le, d->left, lw, w);
+				Rmatch = (d->right != NULL) && (re != NULL) && do_match(ctxt, d->right, re, w, rw);
 				for (i=0; i<4; i++) {ls[i] = rs[i] = NULL;}
 				if (Lmatch)
 				{
-					ls[0] = parse_set(sent, ld, d, lw, w, le->next, d->left->next, lcost, islands_ok, pi);
-					if (le->multi) ls[1] = parse_set(sent, ld, d, lw, w, le, d->left->next, lcost, islands_ok, pi);
-					if (d->left->multi) ls[2] = parse_set(sent, ld, d, lw, w, le->next, d->left, lcost, islands_ok, pi);
-					if (le->multi && d->left->multi) ls[3] = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
+					ls[0] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left->next, lcost, islands_ok, pi);
+					if (le->multi) ls[1] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left->next, lcost, islands_ok, pi);
+					if (d->left->multi) ls[2] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le->next, d->left, lcost, islands_ok, pi);
+					if (le->multi && d->left->multi) ls[3] = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 				}
 				if (Rmatch)
 				{
-					rs[0] = parse_set(sent, d, rd, w, rw, d->right->next, re->next, rcost, islands_ok, pi);
-					if (d->right->multi) rs[1] = parse_set(sent, d, rd, w,rw,d->right,re->next, rcost, islands_ok, pi);
-					if (re->multi) rs[2] = parse_set(sent, d, rd, w, rw, d->right->next, re, rcost, islands_ok, pi);
-					if (d->right->multi && re->multi) rs[3] = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
+					rs[0] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re->next, rcost, islands_ok, pi);
+					if (d->right->multi) rs[1] = mk_parse_set(sent, mchxt, ctxt, d, rd, w,rw,d->right,re->next, rcost, islands_ok, pi);
+					if (re->multi) rs[2] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right->next, re, rcost, islands_ok, pi);
+					if (d->right->multi && re->multi) rs[3] = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
 				}
 
 				for (i=0; i<4; i++)
@@ -387,7 +395,7 @@ static Parse_set * parse_set(Sentence sent,
 				if (ls[0] != NULL || ls[1] != NULL || ls[2] != NULL || ls[3] != NULL)
 				{
 					/* evaluate using the left match, but not the right */
-					rset = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
+					rset = mk_parse_set(sent, mchxt, ctxt, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
 					if (rset != NULL)
 					{
 						for (i=0; i<4; i++)
@@ -407,7 +415,7 @@ static Parse_set * parse_set(Sentence sent,
 				     rs[1] != NULL || rs[2] != NULL || rs[3] != NULL))
 				{
 					/* evaluate using the right match, but not the left */
-					lset = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
+					lset = mk_parse_set(sent, mchxt, ctxt, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 
 					if (lset != NULL)
 					{
@@ -426,7 +434,7 @@ static Parse_set * parse_set(Sentence sent,
 				}
 			}
 		}
-		put_match_list(sent, m1);
+		put_match_list(mchxt, m1);
 	}
 	xt->set->current = xt->set->first;
 	return xt->set;
@@ -436,21 +444,21 @@ static Parse_set * parse_set(Sentence sent,
  * return TRUE if and only if overflow in the number of parses occured.
  * Use a 64-bit int for counting.
  */
-static int verify_set_node(Parse_set *set)
+static bool verify_set_node(Parse_set *set)
 {
 	Parse_choice *pc;
-	s64 n;
-	if (set == NULL || set->first == NULL) return FALSE;
-	n = 0;
+	s64 n = 0;
+	if (set == NULL || set->first == NULL) return false;
+
 	for (pc = set->first; pc != NULL; pc = pc->next)
 	{
 		n  += pc->set[0]->count * pc->set[1]->count;
-		if (PARSE_NUM_OVERFLOW < n) return TRUE;
+		if (PARSE_NUM_OVERFLOW < n) return true;
 	}
-	return FALSE;
+	return false;
 }
 
-static Boolean verify_set(Parse_info pi)
+static bool verify_set(Parse_info pi)
 {
 	unsigned int i;
 
@@ -460,10 +468,10 @@ static Boolean verify_set(Parse_info pi)
 		X_table_connector *t;
 		for(t = pi->x_table[i]; t != NULL; t = t->next)
 		{
-			if (verify_set_node(t->set)) return TRUE;
+			if (verify_set_node(t->set)) return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 /**
@@ -482,12 +490,15 @@ static Boolean verify_set(Parse_info pi)
  * This routine returns TRUE iff overflowed occurred.
  */
 
-int build_parse_set(Sentence sent, unsigned int cost, Parse_Options opts)
+bool build_parse_set(Sentence sent, match_context_t *mchxt,
+                    count_context_t *ctxt,
+                    unsigned int cost, Parse_Options opts)
 {
 	Parse_set * whole_set;
 
 	whole_set =
-		parse_set(sent, NULL, NULL, -1, sent->length, NULL, NULL, cost+1,
+		mk_parse_set(sent, mchxt, ctxt, 
+                NULL, NULL, -1, sent->length, NULL, NULL, cost+1,
 		          opts->islands_ok, sent->parse_info);
 
 	if ((whole_set != NULL) && (whole_set->current != NULL)) {
@@ -507,7 +518,12 @@ static void initialize_links(Parse_info pi)
 
 static void issue_link(Parse_info pi, Disjunct * ld, Disjunct * rd, Link link)
 {
-	assert(pi->N_links <= MAX_LINKS-1, "Too many links");
+	if (pi->lasz <= pi->N_links)
+	{
+		size_t oldsz = pi->lasz;
+		pi->lasz = 2 * pi->lasz + 10;
+		pi->link_array = xrealloc(pi->link_array, oldsz * sizeof(Link), pi->lasz * sizeof(Link));
+	}
 	pi->link_array[pi->N_links] = link;
 	pi->N_links++;
 
