@@ -15,5 +15,5 @@
 
 Parse_info parse_info_new(int nwords);
 void free_parse_info(Parse_info);
-int   build_parse_set(Sentence sent, unsigned int cost, Parse_Options opts);
-void  extract_links(int index, Parse_info pi);
+bool build_parse_set(Sentence, match_context_t*, count_context_t*, unsigned int cost, Parse_Options);
+void extract_links(int index, Parse_info);
