@@ -74,10 +74,9 @@ static Match_node * get_match_node(match_context_t *ctxt)
 /**
  * Put these nodes back onto my free list
  */
-void put_match_list(Sentence sent, Match_node *m)
+void put_match_list(match_context_t *ctxt, Match_node *m)
 {
 	Match_node * xm;
-	match_context_t *ctxt = sent->match_ctxt;
 
 	for (; m != NULL; m = xm)
 	{
@@ -99,33 +98,31 @@ static void free_match_list(Match_node * t)
 /**
  * Free all of the hash tables and Match_nodes 
  */
-void free_fast_matcher(Sentence sent)
+void free_fast_matcher(match_context_t *mchxt)
 {
 	size_t w;
 	unsigned int i;
-	match_context_t *ctxt = sent->match_ctxt;
 
-	if (verbosity > 1) printf("%d Match cost\n", ctxt->match_cost);
-	for (w = 0; w < sent->length; w++)
+	if (verbosity > 1) printf("%d Match cost\n", mchxt->match_cost);
+	for (w = 0; w < mchxt->size; w++)
 	{
-		for (i = 0; i < ctxt->l_table_size[w]; i++)
+		for (i = 0; i < mchxt->l_table_size[w]; i++)
 		{
-			free_match_list(ctxt->l_table[w][i]);
+			free_match_list(mchxt->l_table[w][i]);
 		}
-		xfree((char *)ctxt->l_table[w], ctxt->l_table_size[w] * sizeof (Match_node *));
-		for (i = 0; i < ctxt->r_table_size[w]; i++)
+		xfree((char *)mchxt->l_table[w], mchxt->l_table_size[w] * sizeof (Match_node *));
+		for (i = 0; i < mchxt->r_table_size[w]; i++)
 		{
-			free_match_list(ctxt->r_table[w][i]);
+			free_match_list(mchxt->r_table[w][i]);
 		}
-		xfree((char *)ctxt->r_table[w], ctxt->r_table_size[w] * sizeof (Match_node *));
+		xfree((char *)mchxt->r_table[w], mchxt->r_table_size[w] * sizeof (Match_node *));
 	}
-	free_match_list(ctxt->mn_free_list);
-	ctxt->mn_free_list = NULL;
+	free_match_list(mchxt->mn_free_list);
+	mchxt->mn_free_list = NULL;
 
-	xfree(ctxt->l_table_size, ctxt->size * sizeof(unsigned int));
-	xfree(ctxt->l_table, ctxt->size * sizeof(Match_node **));
-	xfree(ctxt, sizeof(match_context_t));
-	sent->match_ctxt = NULL;
+	xfree(mchxt->l_table_size, mchxt->size * sizeof(unsigned int));
+	xfree(mchxt->l_table, mchxt->size * sizeof(Match_node **));
+	xfree(mchxt, sizeof(match_context_t));
 }
 
 /**
@@ -212,9 +209,9 @@ static void put_into_match_table(unsigned int size, Match_node ** t,
 	}
 }
 
-void init_fast_matcher(Sentence sent)
+match_context_t* alloc_fast_matcher(const Sentence sent)
 {
-   unsigned int size, i;
+	unsigned int size;
 	size_t w;
 	int len;
 	Match_node ** t;
@@ -227,18 +224,17 @@ void init_fast_matcher(Sentence sent)
 	ctxt->r_table_size = ctxt->l_table_size + sent->length;
 	ctxt->l_table = xalloc(2 * sent->length * sizeof(Match_node **));
 	ctxt->r_table = ctxt->l_table + sent->length;
+	memset(ctxt->l_table, 0, 2 * sent->length * sizeof(Match_node **));
 	ctxt->match_cost = 0;
 	ctxt->mn_free_list = NULL;
 
-	sent->match_ctxt = ctxt;
-
 	for (w=0; w<sent->length; w++)
 	{
 		len = left_disjunct_list_length(sent->word[w].d);
 		size = next_power_of_two_up(len);
 		ctxt->l_table_size[w] = size;
 		t = ctxt->l_table[w] = (Match_node **) xalloc(size * sizeof(Match_node *));
-		for (i = 0; i < size; i++) t[i] = NULL;
+		memset(t, 0, size * sizeof(Match_node *));
 
 		for (d = sent->word[w].d; d != NULL; d = d->next)
 		{
@@ -252,7 +248,7 @@ void init_fast_matcher(Sentence sent)
 		size = next_power_of_two_up(len);
 		ctxt->r_table_size[w] = size;
 		t = ctxt->r_table[w] = (Match_node **) xalloc(size * sizeof(Match_node *));
-		for (i = 0; i < size; i++) t[i] = NULL;
+		memset(t, 0, size * sizeof(Match_node *));
 
 		for (d = sent->word[w].d; d != NULL; d = d->next)
 		{
@@ -262,6 +258,8 @@ void init_fast_matcher(Sentence sent)
 			}
 		}
 	}
+
+	return ctxt;
 }
 
 /**
@@ -274,14 +272,12 @@ void init_fast_matcher(Sentence sent)
  * to fix this.
  */
 Match_node * 
-form_match_list(Sentence sent, int w, 
+form_match_list(match_context_t *ctxt, int w, 
                 Connector *lc, int lw,
                 Connector *rc, int rw)
 {
 	Match_node *ml, *mr, *mx, *my, * mz, *front, *free_later;
 
-	match_context_t *ctxt = sent->match_ctxt;
-
 	if (lc != NULL) {
 		ml = ctxt->l_table[w][connector_hash(lc) & (ctxt->l_table_size[w]-1)];
 	} else {
@@ -337,7 +333,7 @@ form_match_list(Sentence sent, int w,
 		}
 	}
 	mr = front;  /* mr is now the abbreviated right list */
-	put_match_list(sent, free_later);
+	put_match_list(ctxt, free_later);
 
 	/* now catenate the two lists */
 	if (mr == NULL) return ml;
