@@ -15,11 +15,13 @@
 #include <stdlib.h>
 
 #include "analyze-linkage.h"
+#include "and.h"
 #include "api-structures.h"
 #include "disjuncts.h"
 #include "extract-links.h"
 #include "idiom.h"
 #include "link-includes.h"
+#include "linkage.h"
 #include "post-process.h"
 #include "print.h"
 #include "print-util.h"
@@ -34,24 +36,21 @@
  */
 
 /*
- * The "empty word" is an ugly, hacky concept used to work around
- * fixed-length parsing within the parser. That is, the tokenizer
- * may or may not split a word into morphemes, and the number
- * of morphemes may be variable.  However, the parser, as currently
- * written, expects sentences to be of fixed length. Thus, the "empty
- * word" is automatically inserted into a sentence, so as to balance
- * the total count.  However, its resence is entirely bogus, and so,
- * below, we remove it from the linkage, as well as the links that
- * connect to it.
+ * The "empty word" is a concept used in order to make the current parser able
+ * to parse "alternatives" within a sentence. The "empty word" can link to any
+ * word, hence it is issued when a word is optional.  Currently, for coding
+ * convenience, the "empty * word" is also automatically inserted into a
+ * sentence, so as to balance the total count of words in alternatives.
  *
- * The empty-word device is emplyed by every non-English dictionary.
- * (viz. Russian, German, Lithuanian, ...)
+ * However, the empty words are not needed after the linkage step, and so,
+ * below, we remove them from the linkage, as well as the links that
+ * connect to them.
  *
- * XXX TODO ... the = sign should be replaced by something else, say,
- * 0x4, during dictionary read, and then converted back to equals sign
- * during printing.  The problem is tht equals signs can also show up
- * in equations, which makes things confusing during the printing steps
- * done below, where we try to glue words back together again.
+ * The empty word device is employed by every non-English dictionary. (viz.
+ * Russian, German, Lithuanian, ...). Recently it has been added to the English
+ * dictionary, and is issued in cases of contraction splitting, spell guesses,
+ * and unit-strip alternatives. For more information, see EMPTY_WORD.zzz in the
+ * dict file.
  */
 #define EMPTY_WORD_SUPPRESS ("ZZZ") /* link to pure whitespace */
 
@@ -65,6 +64,11 @@
 
 #define HIDE_MORPHO   (!display_morphology)
 
+/* FIXME for is_*:
+ * - Use INFIX_MARK and STEMSUBSCR, to match the corresponding affix classes.
+ * - There are versions of these functions in api.c - unify them.
+ */
+
 /**
  * Return TRUE if the word is a suffix.
  *
@@ -88,13 +92,19 @@ static bool is_suffix(const char* w)
 
 /* Return TRUE if the word seems to be in stem form.
  * Stems have the distinctive 'shape', that the end with the = sign
- * and are preceeded by the subscript mark.
+ * and are preceded by the subscript mark.
+ * Examples (. represented the subscript mark): word.= word.=[!]
  */
 static bool is_stem(const char* w)
 {
 	size_t l = strlen(w);
-	if (l < 2) return FALSE;
-   if (strcmp(STEM_MARK, w+l-2)) return FALSE;
+	const char *subscrmark;
+
+	if (l < 3) return FALSE;
+
+	subscrmark = strchr(w, SUBSCRIPT_MARK);
+	if (NULL == subscrmark) return FALSE;
+	if (0 != strncmp(subscrmark, STEM_MARK, sizeof(STEM_MARK)-1)) return FALSE;
 
 	return TRUE;
 }
@@ -111,7 +121,7 @@ static bool is_stem(const char* w)
  *    chosen_disjunct[].string in that the idiom symbols have been removed.
  *
  */
-static void compute_chosen_words(Sentence sent, Linkage linkage)
+void compute_chosen_words(Sentence sent, Linkage linkage)
 {
 	size_t i, j, l;
 	char * s, *u;
@@ -120,6 +130,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 	size_t remap[sent->length];
 	Parse_Options opts = linkage->opts;
 	bool display_morphology = opts->display_morphology;
+	bool display_word_subscripts = true;
 
 	for (i=0; i<sent->length; i++)
 	{
@@ -129,7 +140,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 		if (pi->chosen_disjuncts[i] == NULL)
 		{
 			/* The unsplit_word is the original word; if its been split
-			 * into stem+suffix, and either one hasn't been choosen, then
+			 * into stem+suffix, and either one hasn't been chosen, then
 			 * neither should be printed.  Do, however, put brackets around
 			 * the original word, and print that.
 			 */
@@ -152,6 +163,8 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 
 				append_string(s, "[%zu", i);
 				for (a = sent->word[i].alternatives; *a; a++) {
+					/* Don't show an empty word - it is not an island. */
+					if (0 == strcmp(*a, EMPTY_WORD_MARK)) continue;
 					append_string(s, " %s", *a);
 				}
 				append_string(s, "]");
@@ -161,8 +174,9 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 				exfree(a_list, strlen(a_list)+1);
 			}
 		}
-		else if (opts->display_word_subscripts)
+		else if (display_word_subscripts) 
 		{
+			/* print the subscript, as in "dog.n" as opposed to "dog" */
 			t = pi->chosen_disjuncts[i]->string;
 			/* get rid of those ugly ".Ixx" */
 			if (is_idiom_word(t)) {
@@ -232,7 +246,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 			 * be alternative 0.  We should do like the above, and then
 			 * manually strip the subscript.
 			 * Except that this code is never ever reached, because
-			 * opts->display_word_subscripts is always true...
+			 * display_word_subscripts is always true...
 			 */
 			t = sent->word[i].alternatives[0];
 		}
@@ -269,15 +283,23 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 	 * is being suppressed, then all links connecting morphemes will
 	 * be discarded as well.
 	 */
+#ifdef USE_FAT_LINKAGES
+	for (i=0, j=0; i<linkage->sublinkage[linkage->current].num_links; i++)
+#else
 	for (i=0, j=0; i<linkage->sublinkage.num_links; i++)
+#endif
 	{
+#ifdef USE_FAT_LINKAGES
+		Link * lnk = linkage->sublinkage[linkage->current].link[i];
+#else
 		Link * lnk = linkage->sublinkage.link[i];
+#endif
 		if (NULL == chosen_words[lnk->lw] ||
 		    NULL == chosen_words[lnk->rw])
 		{
 			bool sane = (0 == strcmp(lnk->link_name, EMPTY_WORD_SUPPRESS));
 			sane = sane || (HIDE_MORPHO &&
-             0 == strncmp(lnk->link_name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L));
+			     0 == strncmp(lnk->link_name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L));
 			assert(sane, "Something wrong with linkage processing");
 			exfree_link(lnk);
 		}
@@ -285,19 +307,28 @@ static void compute_chosen_words(Sentence sent, Linkage linkage)
 		{
 			lnk->lw = remap[lnk->lw];
 			lnk->rw = remap[lnk->rw];
+#ifdef USE_FAT_LINKAGES
+			linkage->sublinkage[linkage->current].link[j] = lnk;
+#else
 			linkage->sublinkage.link[j] = lnk;
+#endif
 			j++;
 		}
 	}
+#ifdef USE_FAT_LINKAGES
+	linkage->sublinkage[linkage->current].num_links = j;
+#else
 	linkage->sublinkage.num_links = j;
+#endif
 }
 
 
 
-Linkage linkage_create(size_t k, Sentence sent, Parse_Options opts)
+Linkage linkage_create(LinkageIdx k, Sentence sent, Parse_Options opts)
 {
 	Linkage linkage;
 
+	if (sent->num_linkages_alloced <= k) return NULL; /* bounds check */
 	if (opts->use_sat_solver)
 	{
 		return sat_create_linkage(k, sent, opts);
@@ -351,17 +382,16 @@ int linkage_get_current_sublinkage(const Linkage linkage)
 #endif /* USE_FAT_LINKAGES */
 }
 
-int linkage_set_current_sublinkage(Linkage linkage, int index)
+bool linkage_set_current_sublinkage(Linkage linkage, LinkageIdx index)
 {
 #ifdef USE_FAT_LINKAGES
-	if ((index < 0) ||
-		(index >= linkage->num_sublinkages))
+	if (index >= linkage->num_sublinkages)
 	{
-		return 0;
+		return false;
 	}
 	linkage->current = index;
 #endif /* USE_FAT_LINKAGES */
-	return 1;
+	return true;
 }
 
 static void exfree_pp_info(PP_info *ppi)
@@ -421,9 +451,9 @@ void linkage_delete(Linkage linkage)
 }
 
 #ifdef USE_FAT_LINKAGES
-static int links_are_equal(Link *l, Link *m)
+static bool links_are_equal(Link *l, Link *m)
 {
-	return ((l->l == m->l) && (l->r == m->r) && (strcmp(l->name, m->name)==0));
+	return ((l->lw == m->lw) && (l->rw == m->rw) && (strcmp(l->link_name, m->link_name)==0));
 }
 
 static int link_already_appears(Linkage linkage, Link *link, int a)
@@ -548,7 +578,7 @@ int linkage_get_num_sublinkages(const Linkage linkage)
 #endif /* USE_FAT_LINKAGES */
 }
 
-int linkage_get_num_words(const Linkage linkage)
+size_t linkage_get_num_words(const Linkage linkage)
 {
 	if (!linkage) return 0;
 	return linkage->num_words;
@@ -567,7 +597,7 @@ size_t linkage_get_num_links(const Linkage linkage)
 #endif /* USE_FAT_LINKAGES */
 }
 
-static inline Boolean verify_link_index(const Linkage linkage, size_t index)
+static inline bool verify_link_index(const Linkage linkage, LinkIdx index)
 {
 	if (!linkage) return false;
 #ifdef USE_FAT_LINKAGES
@@ -581,7 +611,7 @@ static inline Boolean verify_link_index(const Linkage linkage, size_t index)
 	return true;
 }
 
-int linkage_get_link_length(const Linkage linkage, int index)
+int linkage_get_link_length(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	bool word_has_link[linkage->num_words+1];
@@ -621,7 +651,7 @@ int linkage_get_link_length(const Linkage linkage, int index)
 	return length;
 }
 
-size_t linkage_get_link_lword(const Linkage linkage, size_t index)
+WordIdx linkage_get_link_lword(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return SIZE_MAX;
@@ -633,7 +663,7 @@ size_t linkage_get_link_lword(const Linkage linkage, size_t index)
 	return link->lw;
 }
 
-size_t linkage_get_link_rword(const Linkage linkage, size_t index)
+WordIdx linkage_get_link_rword(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return SIZE_MAX;
@@ -645,7 +675,7 @@ size_t linkage_get_link_rword(const Linkage linkage, size_t index)
 	return link->rw;
 }
 
-const char * linkage_get_link_label(const Linkage linkage, size_t index)
+const char * linkage_get_link_label(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -657,7 +687,7 @@ const char * linkage_get_link_label(const Linkage linkage, size_t index)
 	return link->link_name;
 }
 
-const char * linkage_get_link_llabel(const Linkage linkage, size_t index)
+const char * linkage_get_link_llabel(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -669,7 +699,7 @@ const char * linkage_get_link_llabel(const Linkage linkage, size_t index)
 	return link->lc->string;
 }
 
-const char * linkage_get_link_rlabel(const Linkage linkage, size_t index)
+const char * linkage_get_link_rlabel(const Linkage linkage, LinkIdx index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -691,7 +721,7 @@ Sentence linkage_get_sentence(const Linkage linkage)
 	return linkage->sent;
 }
 
-const char * linkage_get_disjunct_str(const Linkage linkage, int w)
+const char * linkage_get_disjunct_str(const Linkage linkage, WordIdx w)
 {
 	Disjunct *dj;
 
@@ -701,6 +731,9 @@ const char * linkage_get_disjunct_str(const Linkage linkage, int w)
 		lg_compute_disjunct_strings(linkage->sent, linkage->info);
 	}
 
+	/* XXX FIXME in the future, linkage->num_words might not match sent->length */
+	if (linkage->num_words <= w) return NULL; /* bounds-check */
+
 	/* dj will be null if the word wasn't used in the parse. */
 	dj = linkage->sent->parse_info->chosen_disjuncts[w];
 	if (NULL == dj) return "";
@@ -708,18 +741,26 @@ const char * linkage_get_disjunct_str(const Linkage linkage, int w)
 	return linkage->info->disjunct_list_str[w];
 }
 
-double linkage_get_disjunct_cost(const Linkage linkage, int w)
+double linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
 {
-	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
+	Disjunct *dj;
+	/* XXX FIXME in the future, linkage->num_words might not match sent->length */
+	if (linkage->num_words <= w) return 0.0; /* bounds-check */
+
+	dj = linkage->sent->parse_info->chosen_disjuncts[w];
 
 	/* dj may be null, if the word didn't participate in the parse. */
 	if (dj) return dj->cost;
 	return 0.0;
 }
 
-double linkage_get_disjunct_corpus_score(const Linkage linkage, int w)
+double linkage_get_disjunct_corpus_score(const Linkage linkage, WordIdx w)
 {
-	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
+	Disjunct *dj;
+
+	/* XXX FIXME in the future, linkage->num_words might not match sent->length */
+	if (linkage->num_words <= w) return 99.999; /* bounds-check */
+	dj = linkage->sent->parse_info->chosen_disjuncts[w];
 
 	/* dj may be null, if the word didn't participate in the parse. */
 	if (NULL == dj) return 99.999;
@@ -727,9 +768,10 @@ double linkage_get_disjunct_corpus_score(const Linkage linkage, int w)
 	return lg_corpus_disjunct_score(linkage, w);
 }
 
-const char * linkage_get_word(const Linkage linkage, int w)
+const char * linkage_get_word(const Linkage linkage, WordIdx w)
 {
 	if (!linkage) return NULL;
+	if (linkage->num_words <= w) return NULL; /* bounds-check */
 	return linkage->word[w];
 }
 
@@ -783,7 +825,7 @@ double linkage_corpus_cost(const Linkage linkage)
 	return linkage->info->corpus_cost;
 }
 
-int linkage_get_link_num_domains(const Linkage linkage, int index)
+int linkage_get_link_num_domains(const Linkage linkage, LinkIdx index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return -1;
@@ -795,7 +837,7 @@ int linkage_get_link_num_domains(const Linkage linkage, int index)
 	return pp_info->num_domains;
 }
 
-const char ** linkage_get_link_domain_names(const Linkage linkage, int index)
+const char ** linkage_get_link_domain_names(const Linkage linkage, LinkIdx index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -900,7 +942,7 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 		else
 #endif /* USE_FAT_LINKAGES */
 		{
-			pp = post_process(postprocessor, opts, sent, subl, false);
+			pp = do_post_process(postprocessor, opts, sent, subl, false);
 			/* This can return NULL, for example if there is no
 			   post-processor */
 		}
