@@ -38,9 +38,10 @@
  * ordinary char for matching purposes.
  */
 
-int post_process_match(const char *s, const char *t)
+bool post_process_match(const char *s, const char *t)
 {
 	char c;
+	if (islower((int)*t)) t++; /* Skip head-dependent indicator */
 	while (isupper((int)*s) || isupper((int)*t))
 	{
 		if (*s != *t) return FALSE;
@@ -52,12 +53,12 @@ int post_process_match(const char *s, const char *t)
 		if (*s != '#')
 		{
 			if (*t == '\0') c = '*'; else c = *t;
-			if (*s != c) return FALSE;
+			if (*s != c) return false;
 		}
 		s++;
 		if (*t != '\0') t++;
 	}
-	return TRUE;
+	return true;
 }
 
 /***************** utility routines (not exported) ***********************/
@@ -121,7 +122,7 @@ static bool check_domain_nesting(Postprocessor *pp, int num_links)
 {
 	Domain * d1, * d2;
 	int counts[4];
-	char mark[MAX_LINKS];
+	char mark[MAX_NUM_LINKS];
 	List_o_links * lol;
 	int i;
 	for (d1=pp->pp_data.domain_array; d1 < pp->pp_data.domain_array + pp->pp_data.N_domains; d1++) {
@@ -231,6 +232,16 @@ static bool is_connected(Postprocessor *pp)
 	return TRUE;
 }
 
+static void chk_d_type(PP_node* ppn, size_t idx)
+{
+	if (ppn->dtsz <= idx)
+	{
+		size_t oldsz = ppn->dtsz;
+		ppn->dtsz = 2 * ppn->dtsz + 10;
+		ppn->d_type_array = xrealloc(ppn->d_type_array, 
+			oldsz * sizeof(D_type_list*), ppn->dtsz * sizeof(D_type_list*));
+	}
+}
 
 static void build_type_array(Postprocessor *pp)
 {
@@ -241,10 +252,11 @@ static void build_type_array(Postprocessor *pp)
 	{
 		for (lol=pp->pp_data.domain_array[d].lol; lol != NULL; lol = lol->next)
 		{
+			chk_d_type(pp->pp_node, lol->link);
 			dtl = (D_type_list *) xalloc(sizeof(D_type_list));
 			dtl->next = pp->pp_node->d_type_array[lol->link];
-			pp->pp_node->d_type_array[lol->link] = dtl;
 			dtl->type = pp->pp_data.domain_array[d].type;
+			pp->pp_node->d_type_array[lol->link] = dtl;
 		}
 	}
 }
@@ -282,35 +294,42 @@ D_type_list * copy_d_type(const D_type_list * dtl)
 /** free the pp node from last time */
 static void free_pp_node(Postprocessor *pp)
 {
-	int i;
+	size_t i;
 	PP_node *ppn = pp->pp_node;
 	pp->pp_node = NULL;
 	if (ppn == NULL) return;
 
-	for (i=0; i<MAX_LINKS; i++)
+	for (i=0; i<ppn->dtsz; i++)
 	{
 		free_d_type(ppn->d_type_array[i]);
 	}
+	xfree(ppn->d_type_array, ppn->dtsz * sizeof(D_type_list *));
 	xfree((void*) ppn, sizeof(PP_node));
 }
 
-
 /** set up a fresh pp_node for later use */
 static void alloc_pp_node(Postprocessor *pp)
 {
-	int i;
-	pp->pp_node=(PP_node *) xalloc(sizeof(PP_node));
-	pp->pp_node->violation = NULL;
-	for (i=0; i<MAX_LINKS; i++)
-		pp->pp_node->d_type_array[i] = NULL;
+	PP_node *ppn = (PP_node *) xalloc(sizeof(PP_node));
+
+	/* highly unlikely that the number of links will ever exceed this */
+	ppn->dtsz = 2 * pp->pp_data.length;
+	ppn->d_type_array = (D_type_list **) xalloc (ppn->dtsz * sizeof(D_type_list*));
+	pp->pp_node = ppn;
 }
 
-static void reset_pp_node(Postprocessor *pp)
+static void clear_pp_node(Postprocessor *pp)
 {
-	free_pp_node(pp);
-	alloc_pp_node(pp);
+	size_t i;
+	PP_node *ppn = pp->pp_node;
+	if (NULL == ppn) { alloc_pp_node(pp); ppn = pp->pp_node; }
+
+	ppn->violation = NULL;
+	for (i=0; i<ppn->dtsz; i++)
+		ppn->d_type_array[i] = NULL;
 }
 
+
 /************************ rule application *******************************/
 
 static bool apply_rules(Postprocessor *pp,
@@ -423,8 +442,11 @@ apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *ru
 {
 	size_t i,j,count;
 	for (i=0; i<sublinkage->num_links; i++) {
-		// if (sublinkage->link[i]->lw == SIZE_MAX) continue;
-		assert (sublinkage->link[i]->lw != SIZE_MAX);
+#ifdef USE_FAT_LINKAGES
+		if (sublinkage->link[i]->lw == SIZE_MAX) continue;
+#else
+		assert(sublinkage->link[i]->lw != SIZE_MAX);
+#endif
 		if (post_process_match(rule->selector,sublinkage->link[i]->link_name)) break;
 	}
 	if (i == sublinkage->num_links) return TRUE;
@@ -432,8 +454,11 @@ apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *ru
 	/* selector link of rule appears in sentence */
 	count = 0;
 	for (j=0; j<sublinkage->num_links && count==0; j++) {
-		// if (sublinkage->link[j]->lw == SIZE_MAX) continue;
-		assert (sublinkage->link[j]->lw != SIZE_MAX);
+#ifdef USE_FAT_LINKAGES
+		if (sublinkage->link[j]->lw == SIZE_MAX) continue;
+#else
+		assert(sublinkage->link[j]->lw != SIZE_MAX);
+#endif
 		if (string_in_list(sublinkage->link[j]->link_name, rule->link_array))
 		{
 			count=1;
@@ -452,34 +477,12 @@ apply_connected(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 	return true;
 }
 
-#if 0
-/* replaced in 3/98 with a slightly different algorithm shown below	---DS*/
-static int
-apply_connected_without(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
-{
-	/* Returns true if the linkage is connected when ignoring the links
-		 whose names are in the given list of link names.
-		 Actually, what it does is this: it returns FALSE if the connectivity
-		 of the subgraph reachable from word 0 changes as a result of deleting
-		 these links. */
-	int i;
-	memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
-	mark_reachable_words(pp, 0);
-	for (i=0; i<pp->pp_data.length; i++)
-		pp->visited[i] = !pp->visited[i];
-	connectivity_dfs(pp, sublinkage, 0, rule->link_set);
-	for (i=0; i<pp->pp_data.length; i++)
-		if (pp->visited[i] == FALSE) return FALSE;
-	return TRUE;
-}
-#else
-
-/* Here's the new algorithm: For each link in the linkage that is in the
-	 must_form_a_cycle list, we want to make sure that that link
-	 is in a cycle.	We do this simply by deleting the link, then seeing if the
-	 end points of that link are still connected.
-*/
-
+/**
+ * For each link in the linkage that is in the must_form_a_cycle list,
+ * we want to make sure that that link is in a cycle.  We do this
+ * simply by deleting the link, then seeing if the end points of that
+ * link are still connected.
+ */
 static void reachable_without_dfs(Postprocessor *pp,
                 Sublinkage *sublinkage, size_t a, size_t b, size_t w)
 {
@@ -489,7 +492,9 @@ static void reachable_without_dfs(Postprocessor *pp,
 	pp->visited[w] = TRUE;
 	for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
 	{
-		if (!pp->visited[lol->word] && !(w == a && lol->word == b) && ! (w == b && lol->word == a))
+		if (!pp->visited[lol->word] && 
+          !(w == a && lol->word == b) && 
+          !(w == b && lol->word == a))
 		{
 				reachable_without_dfs(pp, sublinkage, a, b, lol->word);
 		}
@@ -527,14 +532,12 @@ apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage, pp_rule *rule)
 		if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->link_name)) continue;
 		memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
 		reachable_without_dfs(pp, sublinkage, w, lol->word, w);
-		if (!pp->visited[lol->word]) return FALSE;
+		if (!pp->visited[lol->word]) return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
-#endif
-
 /**
  * Checks to see that all domains with this name have the property that
  * all of the words that touch a link in the domain are not to the left
@@ -575,8 +578,11 @@ static void build_graph(Postprocessor *pp, Sublinkage *sublinkage)
 
 	for (link=0; link<sublinkage->num_links; link++)
 	{
-		// if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+#ifdef USE_FAT_LINKAGES
+		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+#else
 		assert (sublinkage->link[link]->lw != SIZE_MAX);
+#endif
 		if (pp_linkset_match(pp->knowledge->ignore_these_links, sublinkage->link[link]->link_name))
 		{
 			lol = (List_o_links *) xalloc(sizeof(List_o_links));
@@ -706,54 +712,56 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 	pp->pp_data.N_domains = 0;
 
 	for (link = 0; link<sublinkage->num_links; link++) {
-		// if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+#ifdef USE_FAT_LINKAGES
+		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+#else
 		assert (sublinkage->link[link]->lw != SIZE_MAX);
+#endif
 		s = sublinkage->link[link]->link_name;
 
 		if (pp_linkset_match(pp->knowledge->ignore_these_links, s)) continue;
 		if (pp_linkset_match(pp->knowledge->domain_starter_links, s))
-			{
-		setup_domain_array(pp, pp->pp_data.N_domains, s, link);
-				if (pp_linkset_match(pp->knowledge->domain_contains_links, s))
-			add_link_to_domain(pp, link);
-		depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
+		{
+			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
+			if (pp_linkset_match(pp->knowledge->domain_contains_links, s))
+				add_link_to_domain(pp, link);
+			depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
 							 sublinkage->link[link]->lw, link);
 
-		pp->pp_data.N_domains++;
-				assert(pp->pp_data.N_domains<PP_MAX_DOMAINS, "raise value of PP_MAX_DOMAINS");
-			}
-		else {
-			if (pp_linkset_match(pp->knowledge->urfl_domain_starter_links,s))
+			pp->pp_data.N_domains++;
+			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS, "raise value of PP_MAX_DOMAINS");
+		}
+		else
+		if (pp_linkset_match(pp->knowledge->urfl_domain_starter_links,s))
 		{
 			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 			/* always add the starter link to its urfl domain */
 			add_link_to_domain(pp, link);
 			bad_depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
-								 sublinkage->link[link]->lw, link);
+							 sublinkage->link[link]->lw, link);
 			pp->pp_data.N_domains++;
 			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
-			else
-		if (pp_linkset_match(pp->knowledge->urfl_only_domain_starter_links,s))
-			{
-				setup_domain_array(pp, pp->pp_data.N_domains, s, link);
-				/* do not add the starter link to its urfl_only domain */
-				d_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
-								 sublinkage->link[link]->lw,
-								 sublinkage->link[link]->rw, link);
-				pp->pp_data.N_domains++;
-				assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
-			}
 		else
-			if (pp_linkset_match(pp->knowledge->left_domain_starter_links, s))
-				{
-					setup_domain_array(pp, pp->pp_data.N_domains, s, link);
-					/* do not add the starter link to a left domain */
-					left_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
-									 sublinkage->link[link]->rw, link);
-					pp->pp_data.N_domains++;
-					assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
-				}
+		if (pp_linkset_match(pp->knowledge->urfl_only_domain_starter_links,s))
+		{
+			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
+			/* do not add the starter link to its urfl_only domain */
+			d_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
+							 sublinkage->link[link]->lw,
+							 sublinkage->link[link]->rw, link);
+			pp->pp_data.N_domains++;
+			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
+		}
+		else
+		if (pp_linkset_match(pp->knowledge->left_domain_starter_links, s))
+		{
+			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
+			/* do not add the starter link to a left domain */
+			left_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
+							 sublinkage->link[link]->rw, link);
+			pp->pp_data.N_domains++;
+			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
 	}
 
@@ -800,7 +808,11 @@ static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 		pp->pp_data.domain_array[d].child = NULL;
 	}
 	for (link=0; link < sublinkage->num_links; link++) {
+#ifdef USE_FAT_LINKAGES
+		if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+#else
 		assert (sublinkage->link[link]->lw != SIZE_MAX);
+#endif
 		for (d=0; d<pp->pp_data.N_domains; d++) {
 			if (link_in_domain(link, &pp->pp_data.domain_array[d])) {
 				dtl = (DTreeLeaf *) xalloc(sizeof(DTreeLeaf));
@@ -1000,7 +1012,7 @@ void post_process_scan_linkage(Postprocessor *pp, Parse_Options opts,
  * NB: sublinkage->link[i]->l=-1 means that this connector
  * is to be ignored
  */
-PP_node *post_process(Postprocessor *pp, Parse_Options opts,
+PP_node *do_post_process(Postprocessor *pp, Parse_Options opts,
 							Sentence sent, Sublinkage *sublinkage, int cleanup)
 {
 	const char *msg;
@@ -1013,13 +1025,13 @@ PP_node *post_process(Postprocessor *pp, Parse_Options opts,
 	/* In the name of responsible memory management, we retain a copy of the
 	 * returned data structure pp_node as a field in pp, so that we can clear
 	 * it out after every call, without relying on the user to do so. */
-	reset_pp_node(pp);
+	clear_pp_node(pp);
 
 	/* The first time we see a sentence, prune the rules which we won't be
 	 * needing during postprocessing the linkages of this sentence */
-	if (sent->q_pruned_rules==FALSE && sent->length >= opts->twopass_length)
+	if (sent->q_pruned_rules == FALSE && sent->length >= opts->twopass_length)
 		prune_irrelevant_rules(pp);
-	sent->q_pruned_rules=TRUE;
+	sent->q_pruned_rules = TRUE;
 
 	switch (internal_process(pp, sublinkage, &msg))
 	{
