@@ -76,7 +76,7 @@ void build_deletable(Sentence sent, int has_conjunction)
 	sent->dptr = (char **) xalloc((sent->length + 1) * sizeof(char *));
 	sent->deletable = sent->dptr + 1;  /* we need to be able to access the [-1] position in this array */
 
-	for (i = -1; i<sent->length; i++)
+	for (i = -1; i< (int) sent->length; i++)
 	{
 		sent->deletable[i] = (char *) xalloc((sent->length+1) * sizeof(char));
 		/* the +1 is to allow us to have the info for the last word
@@ -246,10 +246,9 @@ static void install_fat_connectors(Sentence sent)
 		}
 	}
 }
-#endif /* USE_FAT_LINKAGES */
 
 static void 
-set_connector_list_length_limit(Sentence sent,
+set_connector_list_length_limit(count_context_t * ctxt,
                                 Connector *c,
                                 Connector_set *conset,
                                 int short_len,
@@ -259,7 +258,7 @@ set_connector_list_length_limit(Sentence sent,
 		if (parse_options_get_all_short_connectors(opts)) {
 			c->length_limit = short_len;
 		}
-		else if (conset == NULL || match_in_connector_set(sent, conset, c, '+')) {
+		else if (conset == NULL || match_in_connector_set(ctxt, conset, c, '+')) {
 			c->length_limit = UNLIMITED_LEN;
 		} else {
 			c->length_limit = short_len;
@@ -267,24 +266,26 @@ set_connector_list_length_limit(Sentence sent,
 	}
 }
 
-static void set_connector_length_limits(Sentence sent, Parse_Options opts)
+static void 
+set_connector_length_limits(Sentence sent, count_context_t * ctxt,
+                            Parse_Options opts)
 {
 	size_t i;
 	size_t len;
 	Disjunct *d;
+	Connector_set * ucs = sent->dict->unlimited_connector_set;
 
 	len = opts->short_length;
 	if (len > UNLIMITED_LEN) len = UNLIMITED_LEN;
 
 	for (i=0; i<sent->length; i++) {
 		for (d = sent->word[i].d; d != NULL; d = d->next) {
-			set_connector_list_length_limit(sent, d->left, sent->dict->unlimited_connector_set, len, opts);
-			set_connector_list_length_limit(sent, d->right, sent->dict->unlimited_connector_set, len, opts);
+			set_connector_list_length_limit(ctxt, d->left, ucs, len, opts);
+			set_connector_list_length_limit(ctxt, d->right, ucs, len, opts);
 		}
 	}
 }
 
-#ifdef USE_FAT_LINKAGES
 /**
  * Return true if the sentence contains a conjunction.  Assumes
  * is_conjunction[] has been initialized.
@@ -306,16 +307,6 @@ int sentence_contains_conjunction(Sentence sent)
 	return FALSE;
 }
 
-#else
-
-int sentence_contains_conjunction(Sentence sent)
-{
-	return FALSE;
-}
-
-#endif /* USE_FAT_LINKAGES */
-
-
 /**
  * Assumes that the sentence expression lists have been generated.
  * This does all the necessary pruning and building of fat-link structures.
@@ -323,9 +314,7 @@ int sentence_contains_conjunction(Sentence sent)
 void prepare_to_parse(Sentence sent, Parse_Options opts)
 {
 	size_t i;
-#ifdef USE_FAT_LINKAGES
 	bool has_conjunction;
-#endif /* USE_FAT_LINKAGES */
 
 	build_sentence_disjuncts(sent, opts->disjunct_cost);
 	if (verbosity > 2) {
@@ -336,6 +325,10 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 
 	for (i=0; i<sent->length; i++) {
 		sent->word[i].d = eliminate_duplicate_disjuncts(sent->word[i].d);
+
+		/* Some long Russian sentences can really blow up, here. */
+		if (resources_exhausted(opts->resources))
+			return;
 	}
 	print_time(opts, "Eliminated duplicate disjuncts");
 
@@ -346,7 +339,6 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 
 	sent->null_links = (opts->min_null_count > 0);
 
-#ifdef USE_FAT_LINKAGES
 	if (opts->use_fat_links)
 	{
 		has_conjunction = sentence_contains_conjunction(sent);
@@ -355,20 +347,18 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 	{
 		has_conjunction = FALSE;
 	}
-#endif /* USE_FAT_LINKAGES */
 
 	/* The deletable region depends on whether null links are in use;
 	 * with null_links everything is deletable. Thus, this processing
 	 * cannot be done earlier than here.
 	 */
-	set_connector_length_limits(sent, opts);
-#ifdef USE_FAT_LINKAGES
 	build_deletable(sent, has_conjunction);
 	build_effective_dist(sent, has_conjunction);
-#endif /* USE_FAT_LINKAGES */
-	init_count(sent);
 
-#ifdef USE_FAT_LINKAGES
+	ctxt = alloc_count_context(sent->length);
+	set_connector_length_limits(sent, ctxt, opts);
+	sent->count_ctxt = ctxt;
+
 	if (!has_conjunction)
 	{
 		pp_and_power_prune(sent, RUTHLESS, opts);
@@ -383,7 +373,7 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 		}
 		*/
 		/* already reported -- print_time(opts, "Finished gentle power pruning"); */
-		conjunction_prune(sent, opts);
+		conjunction_prune(sent, ctxt, opts);
 		if (verbosity > 2) {
 			printf("\nAfter conjunction pruning:\n");
 			print_disjunct_counts(sent);
@@ -396,13 +386,13 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 			printf("After conjunctions, disjuncts counts:\n");
 			print_disjunct_counts(sent);
 		}
-		set_connector_length_limits(sent, opts);
+		set_connector_length_limits(sent, ctxt, opts);
 		/* have to do this again cause of the
 		 * new fat connectors and disjuncts */
 
 		print_time(opts, "Constructed fat disjuncts");
 
-		prune(sent);
+		fat_prune(sent);
 		print_time(opts, "Pruned fat disjuncts");
 
 		for (i=0; i<sent->length; i++) {
@@ -418,19 +408,82 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 
 		power_prune(sent, RUTHLESS, opts);
 	}
+	free_count_context(ctxt);
+}
+
 #else
-	pp_and_power_prune(sent, RUTHLESS, opts);
-#endif /* USE_FAT_LINKAGES */
 
-	free_count(sent);
+static void 
+set_connector_list_length_limit(Connector *c,
+                                Connector_set *conset,
+                                int short_len,
+                                Parse_Options opts)
+{
+	for (; c!=NULL; c=c->next) {
+		if (parse_options_get_all_short_connectors(opts)) {
+			c->length_limit = short_len;
+		}
+		else if (conset == NULL || match_in_connector_set(conset, c, '+')) {
+			c->length_limit = UNLIMITED_LEN;
+		} else {
+			c->length_limit = short_len;
+		}
+	}
+}
 
-	/*
+static void 
+set_connector_length_limits(Sentence sent, Parse_Options opts)
+{
+	size_t i;
+	size_t len = opts->short_length;
+	Connector_set * ucs = sent->dict->unlimited_connector_set;
+
+	if (len > UNLIMITED_LEN) len = UNLIMITED_LEN;
+
+	for (i=0; i<sent->length; i++) {
+		Disjunct *d;
+		for (d = sent->word[i].d; d != NULL; d = d->next) {
+			set_connector_list_length_limit(d->left, ucs, len, opts);
+			set_connector_list_length_limit(d->right, ucs, len, opts);
+		}
+	}
+}
+
+/**
+ * Assumes that the sentence expression lists have been generated.
+ */
+void prepare_to_parse(Sentence sent, Parse_Options opts)
+{
+	size_t i;
+
+	build_sentence_disjuncts(sent, opts->disjunct_cost);
 	if (verbosity > 2) {
-		printf("\nAfter RUTHLESS power-pruning:\n");
+		printf("After expanding expressions into disjuncts:");
 		print_disjunct_counts(sent);
 	}
-	*/
-	/* print time for power pruning used to be here */
-	/* now done in power_prune itself */
-	print_time(opts, "Initialized fast matcher and hash table");
+	print_time(opts, "Built disjuncts");
+
+	for (i=0; i<sent->length; i++) {
+		sent->word[i].d = eliminate_duplicate_disjuncts(sent->word[i].d);
+
+		/* Some long Russian sentences can really blow up, here. */
+		if (resources_exhausted(opts->resources))
+			return;
+	}
+	print_time(opts, "Eliminated duplicate disjuncts");
+
+	if (verbosity > 2) {
+		printf("\nAfter expression pruning and duplicate elimination:\n");
+		print_disjunct_counts(sent);
+	}
+
+	set_connector_length_limits(sent, opts);
+	pp_and_power_prune(sent, RUTHLESS, opts);
 }
+
+int sentence_contains_conjunction(Sentence sent)
+{
+	return FALSE;
+}
+
+#endif /* USE_FAT_LINKAGES */
