@@ -29,6 +29,9 @@
 #define RIGHT_WALL_SUPPRESS ("RW")/* If this connector is used on the wall, */
                                   /* then suppress the display of the wall. */
 
+#define HEAD_CHR ('h') /* Single char marking head-word. */
+#define DEPT_CHR ('d') /* Single char marking dependent word */
+
 static void
 set_centers(const Linkage linkage, int center[],
             Boolean print_word_0, int N_words_to_print)
@@ -84,11 +87,11 @@ static void left_append_string(String * string, const char * s, const char * t)
 	}
 }
 
-static void print_a_link(String * s, const Linkage linkage, int link)
+static void print_a_link(String * s, const Linkage linkage, LinkIdx link)
 {
 	Sentence sent = linkage_get_sentence(linkage);
 	Dictionary dict = sent->dict;
-	int l, r;
+	WordIdx l, r;
 	const char *label, *llabel, *rlabel;
 
 	l      = linkage_get_link_lword(linkage, link);
@@ -97,18 +100,38 @@ static void print_a_link(String * s, const Linkage linkage, int link)
 	llabel = linkage_get_link_llabel(linkage, link);
 	rlabel = linkage_get_link_rlabel(linkage, link);
 
-	if ((l == 0) && dict->left_wall_defined) {
-		left_append_string(s, LEFT_WALL_DISPLAY,"               ");
-	} else if ((l == (linkage_get_num_words(linkage)-1)) && dict->right_wall_defined) {
-		left_append_string(s, RIGHT_WALL_DISPLAY,"               ");
-	} else {
+	if ((l == 0) && dict->left_wall_defined)
+	{
+		left_append_string(s, LEFT_WALL_DISPLAY, "               ");
+	}
+	else if ((l == (linkage_get_num_words(linkage)-1)) &&
+	         dict->right_wall_defined)
+	{
+		left_append_string(s, RIGHT_WALL_DISPLAY, "               ");
+	}
+	else
+	{
 		left_append_string(s, linkage_get_word(linkage, l), "               ");
 	}
-	left_append_string(s, llabel, "     ");
-	append_string(s, "   <---");
+
+	/* XXX FIXME -- the space allocated to a link name should depend
+	 * on the longest link-name in the sentence! */
+	left_append_string(s, llabel, "           ");
+	if (DEPT_CHR == llabel[0])
+		append_string(s, "   <---");
+	else if (HEAD_CHR == llabel[0])
+		append_string(s, "   >---");
+	else
+		append_string(s, "   ----");
+
 	left_append_string(s, label, "-----");
-	append_string(s, "->  ");
-	left_append_string(s, rlabel, "     ");
+	if (DEPT_CHR == rlabel[0])
+		append_string(s, "->  ");
+	else if (HEAD_CHR == rlabel[0])
+		append_string(s, "-<  ");
+	else
+		append_string(s, "--  ");
+	left_append_string(s, rlabel, "           ");
 	append_string(s, "     %s\n", linkage_get_word(linkage, r));
 }
 
@@ -127,20 +150,22 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 	const char ** dname;
 
 	longest = 0;
-	for (link=0; link<N_links; link++) {
+	for (link=0; link<N_links; link++)
+	{
 		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
 		assert (linkage_get_link_lword(linkage, link) != SIZE_MAX);
 		if (linkage_get_link_num_domains(linkage, link) > longest)
 			longest = linkage_get_link_num_domains(linkage, link);
 	}
-	for (link=0; link<N_links; link++) {
+	for (link=0; link<N_links; link++)
+	{
 		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
 		assert (linkage_get_link_lword(linkage, link) != SIZE_MAX);
 		dname = linkage_get_link_domain_names(linkage, link);
 		for (j=0; j<linkage_get_link_num_domains(linkage, link); ++j) {
 			append_string(s, " (%s)", dname[j]);
 		}
-		for (;j<longest; j++) {
+		for (; j<longest; j++) {
 			append_string(s, "    ");
 		}
 		append_string(s, "   ");
@@ -228,9 +253,9 @@ char * linkage_print_disjuncts(const Linkage linkage)
 
 #ifdef USE_CORPUS
 		score = linkage_get_disjunct_corpus_score(linkage, w);
-		append_string(s, "%*s    %5.1f %6.3f %s\n", pad, infword, cost, score, dj);
+		append_string(s, "%*s    %5.3f %6.3f %s\n", pad, infword, cost, score, dj);
 #else
-		append_string(s, "%*s    %5.1f  %s\n", pad, infword, cost, dj);
+		append_string(s, "%*s    %5.3f  %s\n", pad, infword, cost, dj);
 #endif
 	}
 	djs = string_copy(s);
@@ -241,7 +266,9 @@ char * linkage_print_disjuncts(const Linkage linkage)
 /**
  * postscript printing ...
  */
-static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *pctx)
+static char *
+build_linkage_postscript_string(const Linkage linkage,
+                                bool display_walls, ps_ctxt_t *pctx)
 {
 	int link, i,j;
 	int d;
@@ -256,7 +283,6 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	String  * string;
 	char * ps_string;
 	Dictionary dict = linkage->sent->dict;
-	Parse_Options opts = linkage->opts;
 	int N_words_to_print;
 
 	string = string_new();
@@ -264,7 +290,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	N_wall_connectors = 0;
 	if (dict->left_wall_defined) {
 		suppressor_used = FALSE;
-		if (!opts->display_walls)
+		if (!display_walls)
 			for (j=0; j<N_links; j++) {
 				if (ppla[j]->lw == 0) {
 					if (ppla[j]->rw == linkage->num_words-1) continue;
@@ -275,7 +301,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 				}
 			}
 		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors > 1) || opts->display_walls);
+						|| (N_wall_connectors > 1) || display_walls);
 	} else {
 		print_word_0 = TRUE;
 	}
@@ -292,7 +318,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 			}
 		}
 		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors > 1) || opts->display_walls);
+						|| (N_wall_connectors > 1) || display_walls);
 	}
 	else {
 		print_word_N = TRUE;
@@ -363,15 +389,19 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
  * Returned string is allocated with exalloc.
  * Needs to be freed with linkage_free_diagram()
  */
-static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
+static char *
+linkage_print_diagram_ctxt(const Linkage linkage,
+                           bool display_walls,
+                           size_t x_screen_width,
+                           ps_ctxt_t *pctx)
 {
+	bool display_short = true;
 	unsigned int i, j, k, cl, cr, row, top_row, top_row_p1;
 	const char *s;
 	char *t;
-	Boolean print_word_0 = 0, print_word_N = 0;
+	bool print_word_0 = 0, print_word_N = 0;
 	int N_wall_connectors, suppressor_used;
 	int center[linkage->num_words+1];
-	char connector[MAX_TOKEN_LENGTH];
 	unsigned int line_len, link_length;
 #ifdef USE_FAT_LINKAGES
 	Sublinkage *sublinkage = &(linkage->sublinkage[linkage->current]);
@@ -383,8 +413,6 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	String * string;
 	char * gr_string;
 	Dictionary dict = linkage->sent->dict;
-	Parse_Options opts = linkage->opts;
-	size_t x_screen_width = parse_options_get_screen_width(opts);
 	unsigned int N_words_to_print;
 
 	char picture[MAX_HEIGHT][MAX_LINE];
@@ -398,7 +426,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	if (dict->left_wall_defined)
 	{
 		suppressor_used = FALSE;
-		if (!opts->display_walls)
+		if (!display_walls)
 		{
 			for (j=0; j<N_links; j++)
 			{
@@ -414,7 +442,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			}
 		}
 		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors > 1) || opts->display_walls);
+						|| (N_wall_connectors > 1) || display_walls);
 	}
 	else
 	{
@@ -438,7 +466,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			}
 		}
 		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
-						|| (N_wall_connectors > 1) || opts->display_walls);
+						|| (N_wall_connectors > 1) || display_walls);
 	}
 	else
 	{
@@ -465,10 +493,10 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			if (((unsigned int) (ppla[j]->rw - ppla[j]->lw)) != link_length)
 			  continue;
 			if (!print_word_0 && (ppla[j]->lw == 0)) continue;
-			/* gets rid of the irrelevant link to the left wall */
+			/* Gets rid of the irrelevant link to the left wall */
 			if (!print_word_N && (ppla[j]->rw == linkage->num_words-1)) continue;
 
-			/* put it into the lowest position */
+			/* Put it into the lowest position */
 			cl = center[ppla[j]->lw];
 			cr = center[ppla[j]->rw];
 			for (row=0; row < MAX_HEIGHT; row++)
@@ -479,8 +507,8 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 				}
 				if (k == cr) break;
 			}
-			/* we know it fits, so put it in this row */
 
+			/* We know it fits, so put it in this row */
 			pctx->link_heights[j] = row;
 
 			if (2*row+2 > MAX_HEIGHT-1) {
@@ -498,39 +526,37 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			}
 			s = ppla[j]->link_name;
 
-			/* display_link_subscripts is always true ... hard-coded */
-			if (opts->display_link_subscripts)
-			{
 #ifdef USE_FAT_LINKAGES
-				if (!is_utf8_alpha(s))
-				  s = "";  /* Don't print fat link connector name */
+			if (!is_utf8_alpha(s))
+			  s = "";  /* Don't print fat link connector name */
 #endif /* USE_FAT_LINKAGES */
-			}
-			else
-			{
-				if (!is_utf8_upper(s)) {
-				  s = "";
-				}
-			}
-			strncpy(connector, s, MAX_TOKEN_LENGTH-1);
-			connector[MAX_TOKEN_LENGTH-1] = '\0';
-			k=0;
-			if (opts->display_link_subscripts)
-				k = strlen(connector);
-			else
-				for (t = connector; isupper((int)*t); t++) k++; /* uppercase len of conn*/
+
+			k = strlen(s);
 			if ((cl+cr-k)/2 + 1 <= cl) {
 				t = picture[row] + cl + 1;
 			} else {
 				t = picture[row] + (cl + cr + 2 - k)/2;
 			}
-			s = connector;
-			if (opts->display_link_subscripts)
-				while ((*s != '\0') && (*t == '-')) *t++ = *s++;
-			else
-				while (isupper((int)*s) && (*t == '-')) *t++ = *s++;
 
-			/* now put in the | below this one, where needed */
+			/* Add direction indicator */
+			// if (DEPT_CHR == ppla[j]->lc->string[0]) { *(t-1) = '<'; }
+			if (DEPT_CHR == ppla[j]->lc->string[0] &&
+			    (t > &picture[row][cl])) { picture[row][cl+1] = '<'; }
+			if (HEAD_CHR == ppla[j]->lc->string[0]) { *(t-1) = '>'; }
+
+			/* Copy connector name; stop short if no room */
+			while ((*s != '\0') && (*t == '-')) *t++ = *s++;
+
+			/* Add direction indicator */
+			// if (DEPT_CHR == ppla[j]->rc->string[0]) { *t = '>'; }
+			if (DEPT_CHR == ppla[j]->rc->string[0]) { picture[row][cr-1] = '>'; }
+			if (HEAD_CHR == ppla[j]->rc->string[0]) { *t = '<'; }
+
+			/* The direction indicators maye have clobbered these. */
+			picture[row][cl] = '+';
+			picture[row][cr] = '+';
+
+			/* Now put in the | below this one, where needed */
 			for (k=0; k<row; k++) {
 				if (picture[k][cl] == ' ') {
 					picture[k][cl] = '|';
@@ -542,8 +568,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		}
 	}
 
-	/* we have the link picture, now put in the words and extra "|"s */
-
+	/* We have the link picture, now put in the words and extra "|"s */
 	t = xpicture[0];
 	if (print_word_0) k = 0; else k = 1;
 	for (; k<N_words_to_print; k++) {
@@ -557,7 +582,10 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	}
 	*t = '\0';
 
-	if (opts->display_short) {
+	/* If display_short is NOT true, then the linkage diagram is printed
+	 * in the "tall" style, with an extra row of vertical descenders
+	 * between each level. */
+	if (display_short) {
 		for (k=0; picture[0][k] != '\0'; k++) {
 			if ((picture[0][k] == '+') || (picture[0][k] == '|')) {
 				xpicture[1][k] = '|';
@@ -659,11 +687,11 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
  * The returned string is malloced, and needs to be freed with
  * linkage_free_diagram()
  */
-char * linkage_print_diagram(const Linkage linkage)
+char * linkage_print_diagram(const Linkage linkage, bool display_walls, size_t screen_width)
 {
 	ps_ctxt_t ctx;
 	if (!linkage) return NULL;
-	return linkage_print_diagram_ctxt(linkage, &ctx);
+	return linkage_print_diagram_ctxt(linkage, display_walls, screen_width, &ctx);
 }
 
 void linkage_free_diagram(char * s)
@@ -687,24 +715,24 @@ void linkage_free_senses(char * s)
 }
 
 /* Forward declarations, the gunk is at the bottom. */
-static const char * trailer(int mode);
-static const char * header(int mode);
+static const char * trailer(bool print_ps_header);
+static const char * header(bool print_ps_header);
 
-char * linkage_print_postscript(Linkage linkage, int mode)
+char * linkage_print_postscript(Linkage linkage, bool display_walls, bool print_ps_header)
 {
 	char * ps, * qs;
 	int size;
 
 	/* call the ascii printer to initialize the row size stuff. */
 	ps_ctxt_t ctx;
-	char * ascii = linkage_print_diagram_ctxt(linkage, &ctx);
+	char * ascii = linkage_print_diagram_ctxt(linkage, display_walls, 8000, &ctx);
 	linkage_free_diagram(ascii);
 
-	ps = build_linkage_postscript_string(linkage, &ctx);
-	size = strlen(header(mode)) + strlen(ps) + strlen(trailer(mode)) + 1;
+	ps = build_linkage_postscript_string(linkage, display_walls, &ctx);
+	size = strlen(header(print_ps_header)) + strlen(ps) + strlen(trailer(print_ps_header)) + 1;
 
 	qs = (char *) exalloc(sizeof(char)*size);
-	sprintf(qs, "%s%s%s", header(mode), ps, trailer(mode));
+	sprintf(qs, "%s%s%s", header(print_ps_header), ps, trailer(print_ps_header));
 	exfree(ps, strlen(ps)+1);
 
 	return qs;
@@ -758,7 +786,7 @@ void print_expression_sizes(Sentence sent)
 	printf("\n\n");
 }
 
-static const char * trailer(int mode)
+static const char * trailer(bool print_ps_header)
 {
 	static const char * trailer_string=
 		"diagram\n"
@@ -766,11 +794,11 @@ static const char * trailer(int mode)
 		"%%EndDocument\n"
 		;
 
-	if (mode==1) return trailer_string;
+	if (print_ps_header) return trailer_string;
 	else return "";
 }
 
-static const char * header(int mode)
+static const char * header(bool print_ps_header)
 {
 	static const char * header_string=
 		"%!PS-Adobe-2.0 EPSF-1.2\n"
@@ -1126,7 +1154,7 @@ static const char * header(int mode)
 		"     end\n"
 		"} def \n"
 		;
-	if (mode==1) return header_string;
+	if (print_ps_header) return header_string;
 	else return "";
 }
 
