@@ -1,7 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright (c) 2009, 2013 Linas Vepstas                                */
+/* Copyright (c) 2009, 2013, 2014 Linas Vepstas                          */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -158,7 +158,7 @@ int prune_match(int dist, Connector *a, Connector *b)
 	s = a->string;
 	t = b->string;
 
-	while(isupper((int)*s) || isupper((int)*t))
+	while (isupper((int)*s) || isupper((int)*t))
 	{
 		if (*s != *t) return FALSE;
 		s++;
@@ -252,33 +252,9 @@ int prune_match(int dist, Connector *a, Connector *b)
 
 #else /* not USE_FAT_LINKAGES */
 
-int prune_match(int dist, Connector *a, Connector *b)
+static inline bool prune_match(int dist, Connector *a, Connector *b)
 {
-	const char *s, *t;
-
-	s = a->string;
-	t = b->string;
-
-	while(isupper((int)*s) || isupper((int)*t))
-	{
-		if (*s != *t) return FALSE;
-		s++;
-		t++;
-	}
-
-	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
-
-	while ((*s != '\0') && (*t != '\0'))
-	{
-		if ((*s == '*') || (*t == '*') || (*s == *t))
-		{
-			s++;
-			t++;
-		}
-		else
-			return FALSE;
-	}
-	return TRUE;
+	return easy_match(a->string, b->string);
 }
 #endif /* USE_FAT_LINKAGES */
 
@@ -314,8 +290,8 @@ static void insert_connector(connector_table *ct, Connector * c)
 	ct[h] = c;
 }
 
-#if 0 /* DEAD_CODE_NOT_USED_ANYWHERE */
-void prune(Sentence sent)
+#ifdef USE_FAT_LINKAGES
+void fat_prune(Sentence sent)
 {
 	Connector *e, *f;
 	int w;
@@ -323,10 +299,8 @@ void prune(Sentence sent)
 	Connector *ct[CONTABSZ];
 	Disjunct fake_head, *d, *d1;
 
-#ifdef USE_FAT_LINKAGES
 	/* XXX why is this here ?? */
-	count_set_effective_distance(sent);
-#endif /* USE_FAT_LINKAGES */
+	count_set_effective_distance(sent->count_ctxt, sent);
 
 	N_deleted = 1;  /* a lie to make it always do at least 2 passes */
 	while (1)
@@ -448,7 +422,7 @@ void prune(Sentence sent)
 		N_deleted = 0;
 	}
 }
-#endif /* DEAD_CODE_NOT_USED_ANYWHERE */
+#endif /* USE_FAT_LINKAGES */
 
 /*
    The second algorithm eliminates disjuncts that are dominated by
@@ -803,7 +777,7 @@ static Exp* purge_Exp(Exp *e)
 			return NULL;
 		}
 	}
-	else
+	else /* if we are here, its OR_type */
 	{
 		e->u.l = or_purge_E_list(e->u.l);
 		if (e->u.l == NULL)
@@ -831,7 +805,7 @@ static Exp* purge_Exp(Exp *e)
 }
 
 /**
- * Returns TRUE if c can match anything in the set S.
+ * Returns TRUE if c can match anything in the set S (err. the connector table ct).
  */
 static inline bool matches_S(connector_table *ct, Connector * c, char dir)
 {
@@ -1330,7 +1304,7 @@ static Boolean possible_connection(prune_context *pc,
 				return FALSE;
 			}
 		}
-		return do_match(pc->sent, lc, rc, lword, rword);
+		return do_match(pc->sent->count_ctxt, lc, rc, lword, rword);
 	} else {
 		if (lword == rword-1) {
 			if (!((lc->next == NULL) && (rc->next == NULL))) return FALSE;
@@ -1348,20 +1322,21 @@ static Boolean possible_connection(prune_context *pc,
 }
 #else /* not USE_FAT_LINKAGES */
 
-static Boolean possible_connection(prune_context *pc,
+static bool possible_connection(prune_context *pc,
                                Connector *lc, Connector *rc,
                                Boolean lshallow, Boolean rshallow,
                                int lword, int rword)
 {
-	if ((!lshallow) && (!rshallow)) return FALSE;
+	int dist;
+	if ((!lshallow) && (!rshallow)) return false;
 	  /* two deep connectors can't work */
-	if ((lc->word > rword) || (rc->word < lword)) return FALSE;
+	if ((lc->word > rword) || (rc->word < lword)) return false;
 	  /* word range constraints */
 
 	assert(lword < rword, "Bad word order in possible connection.");
 
 	if (lword == rword-1) {
-		if (!((lc->next == NULL) && (rc->next == NULL))) return FALSE;
+		if (!((lc->next == NULL) && (rc->next == NULL))) return false;
 	}
 	else
 	if ((!pc->null_links) &&
@@ -1369,9 +1344,13 @@ static Boolean possible_connection(prune_context *pc,
 	    (rc->next == NULL) &&
 	    (!lc->multi) && (!rc->multi))
 	{
-		return FALSE;
+		return false;
 	}
-	return prune_match(rword - lword, lc, rc);
+
+	dist = rword - lword;
+	if (dist > lc->length_limit || dist > rc->length_limit) return false;
+
+	return easy_match(lc->string, rc->string);
 }
 #endif /* USE_FAT_LINKAGES */
 
@@ -1513,7 +1492,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 	pc->power_prune_mode = mode;
 	pc->deletable = sent->deletable;
 	pc->effective_dist = sent->effective_dist;
-	count_set_effective_distance(sent);
+	count_set_effective_distance(sent->count_ctxt, sent);
 #endif /* USE_FAT_LINKAGES */
 
 	pt = power_table_new(sent);
@@ -1732,7 +1711,8 @@ static void cms_table_delete(multiset_table *mt)
 static unsigned int cms_hash(const char * s)
 {
 	unsigned int i = 5381;
-	while (isupper((int) *s)) /* connector names are not yet UTF8-capable */
+	if (islower((int) *s)) s++; /* skip head-dependent indicator */
+	while (isupper((int) *s))
 	{
 		i = ((i << 5) + i) + *s;
 		s++;
@@ -1816,9 +1796,16 @@ static Boolean rule_satisfiable(multiset_table *cmt, pp_linkset *ls)
 			/* now we want to see if we can satisfy this criterion link */
 			/* with a collection of the links in the cms table */
 
-			for (s = name; isupper((int)*s); s++) {}
+			s = name;
+			if (islower((int)*s)) s++; /* skip head-dependent indicator */
+			for (; isupper((int)*s); s++) {}
 			for (;*s != '\0'; s++) if (*s != '*') *s = '#';
-			for (s = name, t = p->str; isupper((int) *s); s++, t++) {}
+
+			s = name;
+			t = p->str;
+			if (islower((int)*s)) s++; /* skip head-dependent indicator */
+			if (islower((int)*t)) t++; /* skip head-dependent indicator */
+			for (; isupper((int) *s); s++, t++) {}
 
 			/* s and t remain in lockstep */
 			bad = 0;
