@@ -33,14 +33,16 @@ public:
   /* Abstract functions that calculate params for each type of variable */
 
   /* string variables */
-  virtual void setStringParameters  (int var, const char* str) {
+  virtual void setStringParameters  (int var, const char* str)
+  {
     bool isDecision = false;
     setParameters(var, isDecision, 0.0, 0.0);
   }
-  virtual void setStringParameters  (int var, const char* str, int cost) = 0;
+  virtual void setStringParameters  (int var, const char* str, double cost) = 0;
 
   /* epsilon variables */
-  virtual void setEpsilonParameters (int var) {
+  virtual void setEpsilonParameters (int var)
+  {
     bool isDecision = false;
     setParameters(var, isDecision, 0.0, 0.0);
   }
@@ -48,15 +50,17 @@ public:
   /* link_cc variables */
   virtual void setLinkParameters    (int var, int wi, const char* ci, int wj, const char* cj, const char* label) = 0;
   virtual void setLinkParameters    (int var, int wi, const char* ci, int wj, const char* cj, const char* label,
-                                     int cost) = 0;
+                                     double cost) = 0;
 
   /* linked_variables */
-  virtual void setLinkedParameters  (int var, int wi, int wj) {
+  virtual void setLinkedParameters  (int var, int wi, int wj)
+  {
     bool isDecision = false;
     setParameters(var, isDecision, 0.0, 0.0);
   }
 
-  virtual void setLinkedMinMaxParameters  (int var, int wi, int wj) {
+  virtual void setLinkedMinMaxParameters  (int var, int wi, int wj)
+  {
     bool isDecision = false;
     setParameters(var, isDecision, 0.0, 0.0);
   }
@@ -79,12 +83,6 @@ public:
     setParameters(var, isDecision, 0.0, 0.0);
   }
 
-#ifdef USE_FAT_LINKAGES
-  /* fat_link variables */
-  virtual void setFatLinkParameters (int var, int wi, int wj) = 0;
-  /* neighbor fat link variables */
-  virtual void setNeighborFatLinkParameters (int var, int w) = 0;
-#endif /* USE_FAT_LINKAGES */
   /* thin_link variables */
   virtual void setThinLinkParameters (int var, int wi, int wj) = 0;
 
@@ -121,24 +119,27 @@ protected:
 };
 
 ////////////////////////////////////////////////////////////////////////////
-class CostDistanceGuiding : public Guiding {
+class CostDistanceGuiding : public Guiding
+{
 public:
-  double cost2priority(int cost) const {
-    return cost == 0 ? 0.0 : (double)(_sent->length + cost);
+  double cost2priority(double cost) const {
+    return cost == 0.0 ? 0.0 : ((double)(_sent->length) + cost);
   }
 
   CostDistanceGuiding(Sentence sent)
     : Guiding(sent) {
   }
 
-  virtual void setStringParameters  (int var, const char* str, int cost) {
+  virtual void setStringParameters(int var, const char* str, double cost)
+  {
     bool isDecision = cost > 0.0;
     double priority = cost2priority(cost);
     double polarity = 0.0;
     setParameters(var, isDecision, priority, polarity);
   }
 
-  virtual void setLinkParameters    (int var, int wi, const char* ci, int wj, const char* cj, const char* label) {
+  virtual void setLinkParameters(int var, int wi, const char* ci, int wj, const char* cj, const char* label)
+  {
     bool isDecision = true;
     double priority = 0.0;
     double polarity = 0.0;
@@ -146,36 +147,24 @@ public:
   }
 
   virtual void setLinkParameters(int var, int i, const char* ci, int j, const char* cj, const char* label,
-                                 int cost)  {
+                                 double cost)
+  {
     bool isDecision = true;
     double priority = cost2priority(cost);
     double polarity = 0.0;
     setParameters(var, isDecision, priority, polarity);
   }
 
-#ifdef USE_FAT_LINKAGES
-  void setFatLinkParameters(int var, int i, int j) {
-    bool isDecision = true;
-    double priority = (double)abs(j - i);
-
-    if (_sent->is_conjunction[i])
-      priority = 0.0;
-
-    double polarity = abs(j - i) == 1 ? 1.0 : 0.0;
-    setParameters(var, isDecision, priority, polarity);
-  }
-#endif /* USE_FAT_LINKAGES */
-
   void setThinLinkParameters(int var, int i, int j)
   {
     bool isDecision = true;
 
     double priority = (double)(j - i);
-#ifdef USE_FAT_LINKAGES
-    if (_sent->is_conjunction[i] || _sent->is_conjunction[j])
-      priority /= 2;
-#endif /* USE_FAT_LINKAGES */
 
+    // XXX this is wrong, isEndingInterpunction is just wrong, and
+    // must be killed FIXME I think that this is just looking for the
+    // one single long link from left-wall to right-punctuation.
+    // seems like a hack, why is such an exception needed???
     double polarity = j - i == 1 ? 1.0 : 0.0;
     if (i == 0 && j == (int) _sent->length - 2 &&
        isEndingInterpunction(_sent->word[j].alternatives[0])) {
@@ -188,22 +177,13 @@ public:
 
     setParameters(var, isDecision, priority, polarity);
   }
-
-#ifdef USE_FAT_LINKAGES
-  void setNeighborFatLinkParameters(int var, int w) {
-    bool isDecision = true;
-    double priority = _sent->length;
-    double polarity = 1.0;
-    setParameters(var, isDecision, priority, polarity);
-  }
-#endif /* USE_FAT_LINKAGES */
-
 };
 
 
 
 ////////////////////////////////////////////////////////////////////////////
-class CostDistanceGuidingOnlyLink : public Guiding {
+class CostDistanceGuidingOnlyLink : public Guiding
+{
 public:
   double cost2priority(int cost) const {
     return cost == 0 ? 0.0 : (double)(_sent->length + cost);
@@ -213,14 +193,16 @@ public:
     : Guiding(sent) {
   }
 
-  virtual void setStringParameters  (int var, const char* str, int cost) {
+  virtual void setStringParameters  (int var, const char* str, double cost)
+  {
     bool isDecision = cost > 0.0;
     double priority = cost2priority(cost);
     double polarity = 0.0;
     setParameters(var, isDecision, priority, polarity);
   }
 
-  virtual void setLinkParameters    (int var, int wi, const char* ci, int wj, const char* cj, const char* label) {
+  virtual void setLinkParameters    (int var, int wi, const char* ci, int wj, const char* cj, const char* label)
+  {
     bool isDecision = true;
     double priority = _sent->length - (wj - wi);
     double polarity = wj - wi <= 3 ? 1.0 : 0.0;;
@@ -228,39 +210,17 @@ public:
   }
 
   virtual void setLinkParameters(int var, int wi, const char* ci, int wj, const char* cj, const char* label,
-                                 int cost)  {
+                                 double cost)
+  {
     bool isDecision = true;
-    double priority = cost > 0 ? cost2priority(cost) : wj - wi;
+    double priority = cost > 0.0 ? cost2priority(cost) : wj - wi;
     double polarity = wj - wi <= 3 ? 1.0 : 0.0;
     setParameters(var, isDecision, priority, polarity);
   }
 
-#ifdef USE_FAT_LINKAGES
-  void setFatLinkParameters(int var, int i, int j) {
-    bool isDecision = true;
-    double priority = (double)abs(j - i);
-
-    if (_sent->is_conjunction[i])
-      priority = 0.0;
-
-    double polarity = abs(j - i) == 1 ? 1.0 : 0.0;
-    setParameters(var, isDecision, priority, polarity);
-  }
-#endif /* USE_FAT_LINKAGES */
-
   void setThinLinkParameters(int var, int i, int j) {
     bool isDecision = false;
     setParameters(var, isDecision, 0.0, 0.0);
   }
-
-#ifdef USE_FAT_LINKAGES
-  void setNeighborFatLinkParameters(int var, int w) {
-    bool isDecision = true;
-    double priority = _sent->length;
-    double polarity = 1.0;
-    setParameters(var, isDecision, priority, polarity);
-  }
-#endif /* USE_FAT_LINKAGES */
-
 };
 #endif
