@@ -5,19 +5,25 @@
 #include <map>
 #include <set>
 
+#include <link-grammar/link-includes.h>
+
 extern "C" {
-#include <link-grammar/api.h>
-}
+#include "count.h"
+#include "prune.h"
+#include "word-utils.h"
+};
 
 #include "variables.hpp"
 
 
-struct PositionConnector {
-  PositionConnector(Connector* c, char d, int w, int p, int cst, int pcst,
+struct PositionConnector
+{
+  PositionConnector(Connector* c, char d, int w, int p, double cst, double pcst,
                     bool lr, bool ll, const std::vector<int>& er, const std::vector<int>& el)
-    : connector(c), dir(d), word(w), position(p), cost(cst), parrent_cost(pcst),
+    : connector(c), dir(d), word(w), position(p), cost(cst), parent_cost(pcst),
       leading_right(lr), leading_left(ll),
-      eps_right(er), eps_left(el) {
+      eps_right(er), eps_left(el)
+  {
     /*
     cout << c->string << " : ." << w << ". : ." << p << ". ";
     if (leading_right) {
@@ -37,17 +43,17 @@ struct PositionConnector {
   // Direction
   char dir;
   // word in a sentence that this connector belongs to
-  int word;
+  size_t word;
   // position in the word tag
   int position;
   // cost of the connector
-  int cost;
-  // parrent cost
-  int parrent_cost;
+  double cost;
+  // parent cost
+  double parent_cost;
 
   bool leading_right;
-  std::vector<int> eps_right;
   bool leading_left;
+  std::vector<int> eps_right;
   std::vector<int> eps_left;
 
   // Matches with other words
@@ -93,16 +99,19 @@ public:
     return _right_connectors;
   }
 
-  PositionConnector* get(int dfs_position) {
+  PositionConnector* get(int dfs_position)
+  {
     switch (_dir[dfs_position - 1]) {
     case '+':
       return &_right_connectors[_position[dfs_position - 1]];
     case '-':
       return &_left_connectors[_position[dfs_position - 1]];
     }
+    return NULL;
   }
 
-  void set_connector_length_limit(Connector* c) {
+  void set_connector_length_limit(Connector* c)
+  {
     int short_len = _opts->short_length;
     if (short_len > UNLIMITED_LEN)
       short_len = UNLIMITED_LEN;
@@ -111,33 +120,26 @@ public:
     if (parse_options_get_all_short_connectors(_opts)) {
       c->length_limit = short_len;
     }
-    else if (conset == NULL || match_in_connector_set(_sent, conset, c, '+')) {
+    else if (conset == NULL || match_in_connector_set(conset, c, '+')) {
       c->length_limit = UNLIMITED_LEN;
     } else {
       c->length_limit = short_len;
     }
   }
 
-  int match(int w1, Connector& cntr1, char dir, int w2, Connector& cntr2, bool conjunction) {
-    if (conjunction) {
-      switch (dir) {
-      case '+':
-        return ::prune_match(0, &cntr1, &cntr2);
-      case '-':
-        return ::prune_match(0, &cntr2, &cntr1);
-      default:
-        throw std::string("Unknown connector direction: ") + dir;
-      }
-    } else {
-      return ::do_match(_sent, &cntr1, &cntr2, w1, w2);
-    }
+  bool match(int w1, Connector& cntr1, char dir, int w2, Connector& cntr2)
+  {
+      int dist = w2 - w1;
+      assert(0 < dist, "match() did not receive words in the natural order.");
+      if (dist > cntr1.length_limit || dist > cntr2.length_limit) return false;
+      return easy_match(cntr1.string, cntr2.string);
   }
 
   void insert_connectors(Exp* exp, int& dfs_position,
                          bool& leading_right, bool& leading_left,
                          std::vector<int>& eps_right,
                          std::vector<int>& eps_left,
-                         char* var, bool root, int parrent_cost);
+                         char* var, bool root, double parent_cost);
 
   // Caches information about the found matches to the _matches vector, and also
   // updates the _matches vector of all connectors in the given tag.
@@ -151,7 +153,8 @@ public:
 
   // A simpler function: Can any connector in this word match a connector wi, pi?
   // It is assumed that
-  bool match_possible(int wi, int pi) {
+  bool match_possible(int wi, int pi)
+  {
     return _match_possible[wi].find(pi) != _match_possible[wi].end();
   }
 
