@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -66,7 +65,7 @@ static unsigned int stride_hash_string(const char *str, const String_set *ss)
 static unsigned int next_prime_up(unsigned int start)
 {
 	unsigned int i;
-	start = start | 1; /* make it odd */
+	start |= 1; /* make it odd */
 	for (;;) {
 		for (i=3; (i <= (start/i)); i += 2) {
 			if (start % i == 0) break;
