@@ -28,6 +28,7 @@
 #include <pthread.h>
 #endif
 
+#include "string-set.h"
 #include "structures.h"
 #include "utilities.h"
 
@@ -272,6 +273,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 	char low[MB_LEN_MAX];
 	mbstate_t mbs;
 
+	to[0] = '\0'; /* Make sure it doesn't contain garbage in case of an error */
 	memset(&mbs, 0, sizeof(mbs));
 	nbh = mbrtowc (&c, from, MB_CUR_MAX, &mbs);
 	if (nbh < 0)
@@ -1163,6 +1165,13 @@ char * get_default_locale(void)
 /* ============================================================= */
 /* Alternatives utilities */
 
+static const char ** resize_alts(const char **arr, size_t len)
+{
+	arr = (const char **)realloc(arr, (len+2) * sizeof(const char *));
+	arr[len+1] = NULL;
+	return arr;
+}
+
 size_t altlen(const char **arr)
 {
 	size_t len = 0;
@@ -1171,4 +1180,12 @@ size_t altlen(const char **arr)
 	return len;
 }
 
+void altappend(Sentence sent, const char ***altp, const char *w)
+{
+	int n = altlen(*altp);
+
+	*altp = resize_alts(*altp, n);
+	(*altp)[n] = string_set_add(w, sent->string_set);
+}
+
 /* ========================== END OF FILE =================== */
