@@ -306,6 +306,8 @@ void exfree_link(Link * l)
 static unsigned int connector_set_hash(Connector_set *conset, const char * s, int d)
 {
 	unsigned int i;
+	if (islower((int)*s)) s++; /* skip head-dependent indicator */
+
 	/* djb2 hash */
 	i = 5381;
 	i = ((i << 5) + i) + d;
@@ -365,45 +367,38 @@ void connector_set_delete(Connector_set * conset)
  * d='+' means this connector is on the right side of the disjunct.
  * d='-' means this connector is on the left side of the disjunct.
  */
-Boolean match_in_connector_set(Sentence sent, Connector_set *conset, Connector * c, int d)
+#ifdef USE_FAT_LINKAGES
+bool match_in_connector_set(count_context_t * ctxt, Connector_set *conset, Connector * c, int d)
 {
 	unsigned int h;
 	Connector * c1;
-	if (conset == NULL) return FALSE;
+	if (conset == NULL) return false;
 	h = connector_set_hash(conset, c->string, d);
 	for (c1 = conset->hash_table[h]; c1 != NULL; c1 = c1->next)
 	{
-		if (do_match(sent, c1, c, 0, 0) && (d == c1->word)) return TRUE;
+		if (do_match(ctxt, c1, c, 0, 0) && (d == c1->word)) return true;
 	}
-	return FALSE;
+	return false;
 }
+#else /* USE_FAT_LINKAGES */
+
+bool match_in_connector_set(Connector_set *conset, Connector * c, int dir)
+{
+	unsigned int h;
+	Connector * c1;
+	if (conset == NULL) return false;
+	h = connector_set_hash(conset, c->string, dir);
+	for (c1 = conset->hash_table[h]; c1 != NULL; c1 = c1->next)
+	{
+		if (easy_match(c1->string, c->string) && (dir == c1->word)) return true;
+	}
+	return false;
+}
+#endif /* not USE_FAT_LINKAGES */
 
 /* ======================================================== */
 /* More connector utilities ... */
 
-/**
- * This is like the basic "match" function in count.c - the basic
- * connector-matching function used in parsing - except it ignores
- * "priority" (used to handle fat links)
- */
-static Boolean easy_match(const char * s, const char * t)
-{
-	while(isupper((int)*s) || isupper((int)*t)) {
-		if (*s != *t) return FALSE;
-		s++;
-		t++;
-	}
-
-	while ((*s!='\0') && (*t!='\0')) {
-		if ((*s == '*') || (*t == '*') ||
-		   ((*s == *t) && (*s != '^'))) {
-		s++;
-		t++;
-		} else return FALSE;
-	}
-	return TRUE;
-}
-
 /**
  * word_has_connector() -- return TRUE if dictionary expression has connector
  * This function takes a dict_node (corresponding to an entry in a
@@ -438,23 +433,23 @@ Boolean word_has_connector(Dict_node * dn, const char * cs, char direction)
 }
 #else /* CRAZY_OBESE_CHECKING_AGLO */
 
-static Boolean exp_has_connector(Exp * e, const char * cs, char direction)
+static bool exp_has_connector(Exp * e, const char * cs, char direction)
 {
 	E_list * el;
 	if (e->type == CONNECTOR_type)
 	{
 		if (direction == e->dir)
 			return easy_match(e->u.string, cs);
-		return FALSE;
+		return false;
 	}
 	for (el = e->u.l; el != NULL; el = el->next)
 	{
-		if (exp_has_connector(el->e, cs, direction)) return TRUE;
+		if (exp_has_connector(el->e, cs, direction)) return false;
 	}
-	return FALSE;
+	return false;
 }
 
-Boolean word_has_connector(Dict_node * dn, const char * cs, char direction)
+bool word_has_connector(Dict_node * dn, const char * cs, char direction)
 {
 	return exp_has_connector(dn->exp, cs, direction);
 }
@@ -466,10 +461,10 @@ Boolean word_has_connector(Dict_node * dn, const char * cs, char direction)
  */
 const char * word_only_connector(Dict_node * dn)
 {
-   Exp * e = dn->exp;
-   if (CONNECTOR_type == e->type)
-      return e->u.string;
-   return NULL;
+	Exp * e = dn->exp;
+	if (CONNECTOR_type == e->type)
+		return e->u.string;
+	return NULL;
 }
 
 /* ======================================================== */
