@@ -48,9 +48,66 @@ void        exfree_link(Link *);
 /* Connector-set utilities ... */
 Connector_set * connector_set_create(Exp *e);
 void connector_set_delete(Connector_set * conset);
-Boolean match_in_connector_set(Sentence, Connector_set *conset, Connector * c, int d);
 Boolean word_has_connector(Dict_node *, const char *, char);
 const char * word_only_connector(Dict_node *);
+#ifdef USE_FAT_LINKAGES
+bool match_in_connector_set(count_context_t*, Connector_set*, Connector*, int);
+#else
+bool match_in_connector_set(Connector_set*, Connector*, int dir);
+#endif
+
+
+/**
+ * Returns TRUE if s and t match according to the connector matching
+ * rules.  The connector strings must be properly formed, starting with
+ * zero or one lower case letters, followed by one or more upper case 
+ * letters, followed by some other letters.
+ *
+ * The algorithm is symmetric with respect to a and b.
+ *
+ * Connectors starting with lower-case letters match ONLY if the initial
+ * letters are DIFFERENT.  Otherwise, connectors only match if the
+ * upper-case letters are the same, and the trailing lower case letters
+ * are the same (or have wildcards).
+ *
+ * The initial lower-case letters allow an initial 'h' (denoting 'head
+ * word') to match an initial 'd' (denoting 'dependent word'), while
+ * rejecting a match 'h' to 'h' or 'd' to 'd'.  This allows the parser
+ * to work with catena, instead of just links.
+ */
+static inline bool easy_match(const char * s, const char * t)
+{
+	char is = 0, it = 0;
+   if (islower((int) *s)) { is = *s; s++; }
+   if (islower((int) *t)) { it = *t; t++; }
+
+   if (is != 0 && it != 0 && is == it) return false;
+
+	while (isupper((int)*s) || isupper((int)*t))
+	{
+		if (*s != *t) return false;
+		s++;
+		t++;
+	}
+
+	while ((*s!='\0') && (*t!='\0'))
+	{
+		if ((*s == '*') || (*t == '*') || 
+#ifdef USE_FAT_LINKAGES
+			((*s == *t) && (*s != '^'))
+#else
+			(*s == *t)
+#endif
+		)
+		{
+			s++;
+			t++;
+		}
+		else
+			return false;
+	}
+	return true;
+}
 
 
 /* Dictionary utilities ... */
@@ -96,6 +153,7 @@ static inline int connector_hash(Connector * c)
 	i = ((i << 5) + i) + (0xff & c->label);
 #endif /* USE_FAT_LINKAGES */
 	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
 	while (isupper((int) *s)) /* connector tables cannot contain UTF8, yet */
 	{
 		i = ((i << 5) + i) + *s;
@@ -109,6 +167,7 @@ static inline int connector_hash(Connector * c)
 	/* Jenkins one-at-a-time hash */
 	i = 0;
 	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
 	while (isupper((int) *s)) /* connector tables cannot contain UTF8, yet */
 	{
 		i += *s;
@@ -129,6 +188,7 @@ static inline int connector_hash(Connector * c)
 	i = 0;
 #endif /* USE_FAT_LINKAGES */
 	s = c->string;
+	if (islower((int) *s)) s++; /* ignore head-dependent indicator */
 	while (isupper((int) *s))
 	{
 		i = *s + (i << 6) + (i << 16) - i;
@@ -173,6 +233,6 @@ static inline unsigned int pair_hash(unsigned int log2_table_size,
 	i = ((unsigned long) re) + (i << 6) + (i << 16) - i;
 #endif
 
-   return i & (table_size-1);
+	return i & (table_size-1);
 }
 #endif /* _LINK_GRAMMAR_WORD_UTILS_H_ */
