@@ -1,7 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright (c) 2008, 2009, 2011 Linas Vepstas                          */
+/* Copyright (c) 2008, 2009, 2011, 2014 Linas Vepstas                    */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -17,6 +17,9 @@
 #include "command-line.h"
 #include <link-grammar/link-includes.h>
 #include "../link-grammar/utilities.h"  /* For MSWindows portability */
+#ifdef USE_REGEX_TOKENIZER
+#include "../link-grammar/regex-tokenizer.h" /* For testing */
+#endif
 
 static struct
 {
@@ -26,7 +29,6 @@ static struct
 	int timeout;
 	int memory;
 	int linkage_limit;
-	int null_block;
 	int islands_ok;
 	int spell_guess;
 	int short_length;
@@ -46,7 +48,6 @@ static struct
 	int screen_width;
 	int display_on;
 	ConstituentDisplayStyle display_constituents;
-	int max_sentence_length;
 	int display_postscript;
 	int display_bad;
 	int display_links;
@@ -86,11 +87,9 @@ static Switch default_switches[] =
    {"islands-ok", Bool, "Use of null-linked islands",      &local.islands_ok},
    {"limit",      Int,  "The maximum linkages processed",  &local.linkage_limit},
    {"links",      Bool, "Display of complete link data",   &local.display_links},
-   {"max-length", Int,  "Maximum sentence length",         &local.max_sentence_length},
    {"memory",     Int,  "Max memory allowed",              &local.memory},
    {"morphology", Bool, "Display word morphology",         &local.display_morphology},
    {"null",       Bool, "Allow null links",                &local.allow_null},
-   {"null-block", Int,  "Size of blocks with null count 1", &local.null_block},
    {"panic",      Bool, "Use of \"panic mode\"",           &local.panic_mode},
    {"postscript", Bool, "Generate postscript output",      &local.display_postscript},
    {"senses",     Bool, "Display of word senses",          &local.display_senses},
@@ -190,11 +189,12 @@ static inline void setival(Switch s, int val)
 	*((int *) s.ptr) = val;
 }
 
-static int x_issue_special_command(const char * line, Parse_Options opts, Dictionary dict)
+static int x_issue_special_command(const char * line, Command_Options *copts, Dictionary dict)
 {
 	char *s, myline[1000], *x, *y;
 	int i, count, j, k;
 	Switch * as = default_switches;
+	Parse_Options opts = copts->popts;
 
 	strncpy(myline, line, sizeof(myline));
 	myline[sizeof(myline)-1] = '\0';
@@ -309,6 +309,14 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 		dict_display_word_expr(dict, s+1, opts);
 		return 0;
 	}
+#ifdef USE_REGEX_TOKENIZER
+	if (s[0] == '/')
+	{
+		int rc = regex_tokenizer_test(dict, s+1);
+		if (0 != rc) printf("regex_tokenizer_test: rc %d\n", rc);
+		return 0;
+	}
+#endif
 
 	/* Test here for an equation i.e. does the command line hold an equals sign? */
 	for (x=s; (*x != '=') && (*x != '\0') ; x++)
@@ -413,87 +421,92 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 	return -1;
 }
 
-static void put_opts_in_local_vars(Parse_Options opts)
+static void put_opts_in_local_vars(Command_Options* copts)
 {
+	Parse_Options opts = copts->popts;
 	local.verbosity = parse_options_get_verbosity(opts);
 	local.debug = parse_options_get_debug(opts);
 	local.test = parse_options_get_test(opts);
 	local.timeout = parse_options_get_max_parse_time(opts);;
 	local.memory = parse_options_get_max_memory(opts);;
 	local.linkage_limit = parse_options_get_linkage_limit(opts);
-	local.null_block = parse_options_get_null_block(opts);
 	local.islands_ok = parse_options_get_islands_ok(opts);
 	local.spell_guess = parse_options_get_spell_guess(opts);
 	local.short_length = parse_options_get_short_length(opts);
 	local.cost_model = parse_options_get_cost_model_type(opts);
 	local.max_cost = parse_options_get_disjunct_cost(opts);
-	local.echo_on = parse_options_get_echo_on(opts);
-	local.batch_mode = parse_options_get_batch_mode(opts);
-	local.panic_mode = parse_options_get_panic_mode(opts);
-	local.screen_width = parse_options_get_screen_width(opts);
-	local.allow_null = parse_options_get_allow_null(opts);
 	local.use_cluster_disjuncts = parse_options_get_use_cluster_disjuncts(opts);
 #ifdef USE_FAT_LINKAGES
 	local.use_fat_links = parse_options_get_use_fat_links(opts);
-	local.display_union = parse_options_get_display_union(opts);
 #endif /* USE_FAT_LINKAGES */
 	local.use_sat_solver = parse_options_get_use_sat_parser(opts);
 	local.use_viterbi = parse_options_get_use_viterbi(opts);
-	local.screen_width = parse_options_get_screen_width(opts);
-	local.display_on = parse_options_get_display_on(opts);
-	local.display_postscript = parse_options_get_display_postscript(opts);
-	local.display_constituents = parse_options_get_display_constituents(opts);
-	local.max_sentence_length = parse_options_get_max_sentence_length(opts);
-	local.display_bad = parse_options_get_display_bad(opts);
-	local.display_disjuncts = parse_options_get_display_disjuncts(opts);
-	local.display_links = parse_options_get_display_links(opts);
+
+	local.screen_width = copts->screen_width;
+	local.echo_on = copts->echo_on;
+	local.batch_mode = copts->batch_mode;
+	local.panic_mode = copts->panic_mode;
+	local.allow_null = copts->allow_null;
+#ifdef USE_FAT_LINKAGES
+	local.display_union = copts->display_union;
+#endif /* USE_FAT_LINKAGES */
+	local.display_on = copts->display_on;
+	local.display_walls = copts->display_walls;
+	local.display_postscript = copts->display_postscript;
+	local.display_constituents = copts->display_constituents;
+
+	local.display_bad = copts->display_bad;
+	local.display_disjuncts = copts->display_disjuncts;
+	local.display_links = copts->display_links;
+	local.display_senses = copts->display_senses;
+
 	local.display_morphology = parse_options_get_display_morphology(opts);
-	local.display_senses = parse_options_get_display_senses(opts);
-	local.display_walls = parse_options_get_display_walls(opts);
 }
 
-static void put_local_vars_in_opts(Parse_Options opts)
+static void put_local_vars_in_opts(Command_Options* copts)
 {
+	Parse_Options opts = copts->popts;
 	parse_options_set_verbosity(opts, local.verbosity);
 	parse_options_set_debug(opts, local.debug);
 	parse_options_set_test(opts, local.test);
 	parse_options_set_max_parse_time(opts, local.timeout);
 	parse_options_set_max_memory(opts, local.memory);
 	parse_options_set_linkage_limit(opts, local.linkage_limit);
-	parse_options_set_null_block(opts, local.null_block);
 	parse_options_set_islands_ok(opts, local.islands_ok);
 	parse_options_set_spell_guess(opts, local.spell_guess);
 	parse_options_set_short_length(opts, local.short_length);
-	parse_options_set_echo_on(opts, local.echo_on);
 	parse_options_set_cost_model_type(opts, local.cost_model);
 	parse_options_set_disjunct_cost(opts, local.max_cost);
-	parse_options_set_batch_mode(opts, local.batch_mode);
-	parse_options_set_panic_mode(opts, local.panic_mode);
-	parse_options_set_screen_width(opts, local.screen_width);
-	parse_options_set_allow_null(opts, local.allow_null);
 	parse_options_set_use_cluster_disjuncts(opts, local.use_cluster_disjuncts);
 #ifdef USE_FAT_LINKAGES
 	parse_options_set_use_fat_links(opts, local.use_fat_links);
-	parse_options_set_display_union(opts, local.display_union);
 #endif /* USE_FAT_LINKAGES */
 #ifdef USE_SAT_SOLVER
 	parse_options_set_use_sat_parser(opts, local.use_sat_solver);
 #endif
 	parse_options_set_use_viterbi(opts, local.use_viterbi);
-	parse_options_set_screen_width(opts, local.screen_width);
-	parse_options_set_display_on(opts, local.display_on);
-	parse_options_set_display_postscript(opts, local.display_postscript);
-	parse_options_set_display_constituents(opts, local.display_constituents);
-	parse_options_set_max_sentence_length(opts, local.max_sentence_length);
-	parse_options_set_display_bad(opts, local.display_bad);
-	parse_options_set_display_disjuncts(opts, local.display_disjuncts);
-	parse_options_set_display_links(opts, local.display_links);
 	parse_options_set_display_morphology(opts, local.display_morphology);
-	parse_options_set_display_senses(opts, local.display_senses);
-	parse_options_set_display_walls(opts, local.display_walls);
+
+	copts->screen_width = local.screen_width;
+	copts->echo_on = local.echo_on;
+	copts->batch_mode = local.batch_mode;
+	copts->panic_mode = local.panic_mode;
+	copts->allow_null = local.allow_null;
+#ifdef USE_FAT_LINKAGES
+	copts->display_union = local.display_union;
+#endif /* USE_FAT_LINKAGES */
+	copts->display_on = local.display_on;
+	copts->display_walls = local.display_walls;
+	copts->display_postscript = local.display_postscript;
+	copts->display_constituents = local.display_constituents;
+
+	copts->display_bad = local.display_bad;
+	copts->display_disjuncts = local.display_disjuncts;
+	copts->display_links = local.display_links;
+	copts->display_senses = local.display_senses;
 }
 
-int issue_special_command(const char * line, Parse_Options opts, Dictionary dict)
+int issue_special_command(const char * line, Command_Options* opts, Dictionary dict)
 {
 	int rc;
 	put_opts_in_local_vars(opts);
@@ -503,3 +516,34 @@ int issue_special_command(const char * line, Parse_Options opts, Dictionary dict
 }
 
 
+Command_Options* command_options_create(void)
+{
+	Command_Options* co = malloc(sizeof (Command_Options));
+	co->popts = parse_options_create();
+	co->panic_opts = parse_options_create();
+
+	co->screen_width = 79;
+#ifdef USE_FAT_LINKAGES
+	co->display_union = false;
+#endif /* USE_FAT_LINKAGES */
+	co->allow_null = true;
+	co->echo_on = false;
+	co->panic_mode = false;
+	co->display_on = true;
+	co->display_walls = false;
+	co->display_postscript = false;
+	co->display_constituents = NO_DISPLAY;
+
+	co->display_bad = false;
+	co->display_disjuncts = false;
+	co->display_links = false;
+	co->display_senses = false;
+	return co;
+}
+
+void command_options_delete(Command_Options* co)
+{
+	parse_options_delete(co->panic_opts);
+	parse_options_delete(co->popts);
+	free(co);
+}
