@@ -1,7 +1,7 @@
 /***************************************************************************/
 /* Copyright (c) 2004                                                      */
 /* Daniel Sleator, David Temperley, and John Lafferty                      */
-/* Copyright (c) 2008 Linas Vepstas                                        */
+/* Copyright (c) 2008, 2014 Linas Vepstas                                  */
 /* All rights reserved                                                     */
 /*                                                                         */
 /* Use of the link grammar parsing system is subject to the terms of the   */
@@ -61,20 +61,19 @@
 #define COMMENT_CHAR '%'  /* input lines beginning with this are ignored */
 
 static int batch_errors = 0;
-static int input_pending=FALSE;
-static Parse_Options  opts;
-static Parse_Options  panic_parse_opts;
+static int input_pending = FALSE;
 static int verbosity = 0;
 static char * debug = (char *)"";
 static char * test = (char *)"";
 
 typedef enum
 {
-	UNGRAMMATICAL='*',
-	PARSE_WITH_DISJUNCT_COST_GT_0=':',  /* Not used anywhere, currently ... */
-	NO_LABEL=' '
+	UNGRAMMATICAL = '*',
+	PARSE_WITH_DISJUNCT_COST_GT_0 = ':',  /* Not used anywhere, currently ... */
+	NO_LABEL = ' '
 } Label;
 
+
 #if defined(_MSC_VER) || defined(__MINGW32__)
 /* Windows console (cmd.exe) input to utf8 */
 static char* oem_to_utf8(char *instring)
@@ -106,7 +105,7 @@ static char* oem_to_utf8(char *instring)
 #endif
 
 static char *
-fget_input_string(FILE *in, FILE *out, Parse_Options opts)
+fget_input_string(FILE *in, FILE *out, Command_Options* copts)
 {
 #ifdef HAVE_EDITLINE
 	static char * pline = NULL;
@@ -131,9 +130,7 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 		input_pending = FALSE;
 		return pline;
 	}
-	if (parse_options_get_batch_mode(opts) ||
-	    (verbosity == 0) ||
-	    input_pending)
+	if (copts->batch_mode || verbosity == 0 || input_pending)
 	{
 		prompt = "";
 	}
@@ -148,9 +145,7 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 
 	if (NULL == in) return NULL;
 
-	if ((!parse_options_get_batch_mode(opts)) &&
-	    (verbosity > 0) &&
-	    (!input_pending))
+	if (!copts->batch_mode && verbosity > 0 && !input_pending)
 	{
 		fprintf(out, "linkparser> ");
 		fflush(out);
@@ -207,10 +202,10 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 #endif
 }
 
-static int fget_input_char(FILE *in, FILE *out, Parse_Options opts)
+static int fget_input_char(FILE *in, FILE *out, Command_Options* copts)
 {
 #ifdef HAVE_EDITLINE
-	char * pline = fget_input_string(in, out, opts);
+	char * pline = fget_input_string(in, out, copts);
 	if (NULL == pline) return EOF;
 	if (*pline)
 	{
@@ -222,7 +217,7 @@ static int fget_input_char(FILE *in, FILE *out, Parse_Options opts)
 #else
 	int c;
 
-	if (!parse_options_get_batch_mode(opts) && (verbosity > 0))
+	if (!copts->batch_mode && verbosity > 0)
 		fprintf(out, "linkparser> ");
 	fflush(out);
 
@@ -246,11 +241,12 @@ static int fget_input_char(FILE *in, FILE *out, Parse_Options opts)
 *
 **************************************************************************/
 
-static void process_linkage(Linkage linkage, Parse_Options opts)
+static void process_linkage(Linkage linkage, Command_Options* copts)
 {
 	char * string;
 	ConstituentDisplayStyle mode;
 #ifdef USE_FAT_LINKAGES
+	Parse_Options opts = copts->popts;
 	int j, first_sublinkage;
 	int nlink;
 #endif /* USE_FAT_LINKAGES */
@@ -274,46 +270,46 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 	{
 		linkage_set_current_sublinkage(linkage, j);
 #endif /* USE_FAT_LINKAGES */
-		if (parse_options_get_display_bad(opts))
+		if (copts->display_bad)
 		{
 			string = linkage_print_pp_msgs(linkage);
 			fprintf(stdout, "%s\n", string);
 			linkage_free_pp_msgs(string);
 		}
-		if (parse_options_get_display_on(opts))
+		if (copts->display_on)
 		{
-			string = linkage_print_diagram(linkage);
+			string = linkage_print_diagram(linkage, copts->display_walls, copts->screen_width);
 			fprintf(stdout, "%s", string);
 			linkage_free_diagram(string);
 		}
-		if (parse_options_get_display_links(opts))
+		if (copts->display_links)
 		{
 			string = linkage_print_links_and_domains(linkage);
 			fprintf(stdout, "%s", string);
 			linkage_free_links_and_domains(string);
 		}
-		if (parse_options_get_display_senses(opts))
+		if (copts->display_senses)
 		{
 			string = linkage_print_senses(linkage);
 			fprintf(stdout, "%s", string);
 			linkage_free_senses(string);
 		}
-		if (parse_options_get_display_disjuncts(opts))
+		if (copts->display_disjuncts)
 		{
 			string = linkage_print_disjuncts(linkage);
 			fprintf(stdout, "%s", string);
 			linkage_free_disjuncts(string);
 		}
-		if (parse_options_get_display_postscript(opts))
+		if (copts->display_postscript)
 		{
-			string = linkage_print_postscript(linkage, FALSE);
+			string = linkage_print_postscript(linkage, copts->display_walls, false);
 			fprintf(stdout, "%s\n", string);
 			linkage_free_postscript(string);
 		}
 #ifdef USE_FAT_LINKAGES
 	}
 #endif /* USE_FAT_LINKAGES */
-	if ((mode = parse_options_get_display_constituents(opts)))
+	if ((mode = copts->display_constituents))
 	{
 		string = linkage_print_constituent_tree(linkage, mode);
 		if (string != NULL)
@@ -359,31 +355,30 @@ static void print_parse_statistics(Sentence sent, Parse_Options opts)
 }
 
 
-static int process_some_linkages(Sentence sent, Parse_Options opts)
+static int process_some_linkages(Sentence sent, Command_Options* copts)
 {
 	int c;
 	int i, num_to_query, num_to_display, num_displayed;
 	Linkage linkage;
 	double corpus_cost;
+	Parse_Options opts = copts->popts;
 
 	if (verbosity > 0) print_parse_statistics(sent, opts);
-	num_to_query = MIN(sentence_num_linkages_post_processed(sent),
-	                   DISPLAY_MAX);
-	if (!parse_options_get_display_bad(opts))
+	num_to_query = sentence_num_linkages_post_processed(sent);
+	if (!copts->display_bad)
 	{
 		num_to_display = MIN(sentence_num_valid_linkages(sent),
 		                     DISPLAY_MAX);
 	}
 	else
 	{
-		num_to_display = MIN(sentence_num_linkages_post_processed(sent),
-		                     DISPLAY_MAX);
+		num_to_display = MIN(num_to_query, DISPLAY_MAX);
 	}
 
 	for (i=0, num_displayed=0; i<num_to_query; i++)
 	{
 		if ((sentence_num_violations(sent, i) > 0) &&
-			(!parse_options_get_display_bad(opts)))
+			!copts->display_bad)
 		{
 			continue;
 		}
@@ -396,11 +391,11 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 		if (verbosity > 0)
 		{
 			if ((sentence_num_valid_linkages(sent) == 1) &&
-				(!parse_options_get_display_bad(opts)))
+				!copts->display_bad)
 			{
 				fprintf(stdout, "	Unique linkage, ");
 			}
-			else if ((parse_options_get_display_bad(opts)) &&
+			else if (copts->display_bad &&
 			         (sentence_num_violations(sent, i) > 0))
 			{
 				fprintf(stdout, "	Linkage %d (bad), ", num_displayed+1);
@@ -465,7 +460,7 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 #endif /* USE_FAT_LINKAGES */
 		}
 
-		process_linkage(linkage, opts);
+		process_linkage(linkage, copts);
 		linkage_delete(linkage);
 
 		if (++num_displayed < num_to_display)
@@ -474,7 +469,7 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 			{
 				fprintf(stdout, "Press RETURN for the next linkage.\n");
 			}
-			c = fget_input_char(stdin, stdout, opts);
+			c = fget_input_char(stdin, stdout, copts);
 			if (c != '\n') return c;
 		}
 		else
@@ -508,9 +503,12 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts)
 
 static void batch_process_some_linkages(Label label,
                                         Sentence sent,
-                                        Parse_Options opts)
+                                        Command_Options* copts)
 {
-	if (there_was_an_error(label, sent, opts)) {
+	Parse_Options opts = copts->popts;
+
+	if (there_was_an_error(label, sent, opts))
+	{
 		/* If linkages were found, print them */
 		if (sentence_num_linkages_found(sent) > 0) {
 			Linkage linkage = NULL;
@@ -531,7 +529,7 @@ static void batch_process_some_linkages(Label label,
 				/* This linkage will be bad; no good ones were found. */
 				linkage = linkage_create(0, sent, opts);
 			}
-			process_linkage(linkage, opts);
+			process_linkage(linkage, copts);
 			linkage_delete(linkage);
 		}
 		fprintf(stdout, "+++++ error %d\n", batch_errors);
@@ -545,18 +543,21 @@ static void batch_process_some_linkages(Label label,
 	}
 }
 
-static int special_command(char *input_string, Dictionary dict)
+static bool special_command(char *input_string, Command_Options* copts, Dictionary dict)
 {
 	if (input_string[0] == '\n') return TRUE;
 	if (input_string[0] == COMMENT_CHAR) return TRUE;
 	if (input_string[0] == '!') {
 		if (strncmp(input_string, "!panic_", 7) == 0)
 		{
-			issue_special_command(input_string+7, panic_parse_opts, dict);
+			Parse_Options save = copts->popts;
+			copts->popts = copts->panic_opts;
+			issue_special_command(input_string+7, copts, dict);
+			copts->popts = save;
 			return TRUE;
 		}
 
-		issue_special_command(input_string+1, opts, dict);
+		issue_special_command(input_string+1, copts, dict);
 		return TRUE;
 	}
 	return FALSE;
@@ -580,7 +581,7 @@ static Label strip_off_label(char * input_string)
 
 static void setup_panic_parse_options(Parse_Options opts)
 {
-	parse_options_set_disjunct_cost(opts, 3.0f);
+	parse_options_set_disjunct_cost(opts, 4.0f);
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, 100);
 	parse_options_set_max_parse_time(opts, 60);
@@ -596,14 +597,15 @@ static void setup_panic_parse_options(Parse_Options opts)
 
 static void print_usage(char *str)
 {
+	Command_Options *copts;
 	fprintf(stderr,
 			"Usage: %s [language|dictionary location]\n"
 			"                   [-<special \"!\" command>]\n"
 			"                   [--version]\n", str);
 
 	fprintf(stderr, "\nSpecial commands are:\n");
-	opts = parse_options_create();
-	issue_special_command("var", opts, NULL);
+	copts = command_options_create();
+	issue_special_command("var", copts, NULL);
 	exit(-1);
 }
 
@@ -612,7 +614,7 @@ static void print_usage(char *str)
  * and sets the output screen width accordingly.
  * Not sure how MS Windows does this.
  */
-static void check_winsize(Parse_Options popts)
+static void check_winsize(Command_Options* copts)
 {
 /* Neither windows nor MSYS have the ioctl support needed for this. */
 #ifdef _WIN32
@@ -637,7 +639,7 @@ static void check_winsize(Parse_Options popts)
 	 */
 	if ((10 < ws.ws_col) && (16123 > ws.ws_col))
 	{
-		parse_options_set_screen_width(popts, ws.ws_col - 1);
+		copts->screen_width = ws.ws_col - 1;
 	}
 #endif /* _WIN32 */
 }
@@ -651,6 +653,9 @@ int main(int argc, char * argv[])
 	Label           label = NO_LABEL;
 	const char      *codeset;
 	const char      *locale = NULL;
+	Command_Options *copts;
+	Parse_Options   opts;
+	bool batch_in_progress = false;
 
 #if LATER
 	/* Try to catch the SIGWINCH ... except this is not working. */
@@ -702,29 +707,14 @@ int main(int argc, char * argv[])
 		}
 	}
 
-	opts = parse_options_create();
-	if (opts == NULL)
+	copts = command_options_create();
+	opts = copts->popts;
+	if (copts == NULL || opts == NULL || copts->panic_opts == NULL)
 	{
 		fprintf(stderr, "%s: Fatal error: unable to create parse options\n", argv[0]);
 		exit(-1);
 	}
 
-	panic_parse_opts = parse_options_create();
-	if (panic_parse_opts == NULL)
-	{
-		fprintf(stderr, "%s: Fatal error: unable to create panic parse options\n", argv[0]);
-		exit(-1);
-	}
-	setup_panic_parse_options(panic_parse_opts);
-	parse_options_set_max_sentence_length(opts, 230);
-	parse_options_set_panic_mode(opts, TRUE);
-	parse_options_set_max_parse_time(opts, 30);
-	parse_options_set_linkage_limit(opts, 1000);
-	parse_options_set_short_length(opts, 10);
-	parse_options_set_disjunct_cost(opts, 2.0);
-	parse_options_set_min_null_count(opts, 0);
-	parse_options_set_max_null_count(opts, 0);
-
 	if (language && *language)
 		dict = dictionary_create_lang(language);
 	else
@@ -736,6 +726,21 @@ int main(int argc, char * argv[])
 		exit(-1);
 	}
 
+	setup_panic_parse_options(copts->panic_opts);
+	copts->panic_mode = true;
+
+	parse_options_set_max_parse_time(opts, 30);
+	parse_options_set_linkage_limit(opts, 1000);
+	parse_options_set_min_null_count(opts, 0);
+	parse_options_set_max_null_count(opts, 0);
+	parse_options_set_short_length(opts, 10);
+
+	/* The English and Russian dicts use a cost of 2.7, which allows
+	 * regexes with a fractional cost of less than 1 to be used with
+	 * rules that have a cost of 2.0.
+	 */
+	parse_options_set_disjunct_cost(opts, 2.7);
+
 	/* Process the command line commands */
 	for (i = 1; i<argc; i++)
 	{
@@ -743,20 +748,16 @@ int main(int argc, char * argv[])
 		{
 			int rc;
 			if (argv[i][1] == '!' || argv[i][1] == '-')
-				rc = issue_special_command(argv[i]+2, opts, dict);
+				rc = issue_special_command(argv[i]+2, copts, dict);
 			else
-				rc = issue_special_command(argv[i]+1, opts, dict);
+				rc = issue_special_command(argv[i]+1, copts, dict);
 
 			if (rc)
 				print_usage(argv[0]);
 		}
 	}
 
-	verbosity = parse_options_get_verbosity(opts);
-	debug = parse_options_get_debug(opts);
-	test = parse_options_get_test(opts);
-
-	check_winsize(opts);
+	check_winsize(copts);
 #ifdef _WIN32
 	parse_options_set_screen_width(opts, 79);
 #endif
@@ -775,8 +776,12 @@ int main(int argc, char * argv[])
 		char *input_string;
 		Sentence sent = NULL;
 
-		input_string = fget_input_string(input_fh, stdout, opts);
-		check_winsize(opts);
+		verbosity = parse_options_get_verbosity(opts);
+		debug = parse_options_get_debug(opts);
+		test = parse_options_get_test(opts);
+
+		input_string = fget_input_string(input_fh, stdout, copts);
+		check_winsize(copts);
 
 		if (NULL == input_string)
 		{
@@ -809,13 +814,27 @@ int main(int argc, char * argv[])
 		/* If the input string is just whitespace, then ignore it. */
 		if (strspn(input_string, " \t\v") == strlen(input_string)) continue;
 
-		if (special_command(input_string, dict)) continue;
-		if (parse_options_get_echo_on(opts))
+		if (special_command(input_string, copts, dict)) continue;
+
+		if (!copts->batch_mode) batch_in_progress = false;
+		if ('\0' != test[0])
+		{
+			/* In batch mode warn only once */
+			if (!batch_in_progress)
+			{
+				fflush(stdout);
+				/* Remind the developer this is a test mode. */
+				fprintf(stderr, "Warning: Tests enabled: %s\n", test);
+				if (copts->batch_mode) batch_in_progress = true;
+			}
+		}
+		
+		if (copts->echo_on)
 		{
 			printf("%s\n", input_string);
 		}
 
-		if (parse_options_get_batch_mode(opts))
+		if (copts->batch_mode)
 		{
 			label = strip_off_label(input_string);
 		}
@@ -832,7 +851,7 @@ int main(int argc, char * argv[])
 			sent = sentence_create(input_string, dict);
 
 			/* First parse with cost 0 or 1 and no null links */
-			// parse_options_set_disjunct_cost(opts, 2.0);
+			// parse_options_set_disjunct_cost(opts, 2.7);
 			parse_options_set_min_null_count(opts, 0);
 			parse_options_set_max_null_count(opts, 0);
 			parse_options_reset_resources(opts);
@@ -861,7 +880,7 @@ int main(int argc, char * argv[])
 			 */
 			if (num_linkages == 0)
 			{
-				parse_options_set_disjunct_cost(opts, 3.5);
+				parse_options_set_disjunct_cost(opts, 4.5);
 				num_linkages = sentence_parse(sent, opts);
 				if (num_linkages < 0) continue;
 			}
@@ -876,7 +895,7 @@ int main(int argc, char * argv[])
 			{
 				int expanded;
 				if (verbosity > 0) fprintf(stdout, "No standard linkages, expanding disjunct set.\n");
-				parse_options_set_disjunct_cost(opts, 2.9);
+				parse_options_set_disjunct_cost(opts, 3.9);
 				expanded = lg_expand_disjunct_list(sent);
 				if (expanded)
 				{
@@ -886,20 +905,20 @@ int main(int argc, char * argv[])
 			}
 
 			/* If asked to show bad linkages, then show them. */
-			if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts)))
+			if ((num_linkages == 0) && (!copts->batch_mode))
 			{
-				if (parse_options_get_display_bad(opts))
+				if (copts->display_bad)
 				{
 					num_linkages = sentence_num_linkages_found(sent);
 				}
 			}
 
 			/* Now parse with null links */
-			if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts)))
+			if (num_linkages == 0 && !copts->batch_mode)
 			{
 				if (verbosity > 0) fprintf(stdout, "No complete linkages found.\n");
 
-				if (parse_options_get_allow_null(opts))
+				if (copts->allow_null)
 				{
 					/* XXX should use expanded disjunct list here too */
 					parse_options_set_min_null_count(opts, 1);
@@ -918,31 +937,31 @@ int main(int argc, char * argv[])
 			}
 
 			if ((num_linkages == 0) &&
-				parse_options_resources_exhausted(opts) &&
-				parse_options_get_panic_mode(opts))
+				copts->panic_mode &&
+				parse_options_resources_exhausted(opts))
 			{
 				/* print_total_time(opts); */
 				batch_errors++;
 				if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
-				parse_options_reset_resources(panic_parse_opts);
-				parse_options_set_verbosity(panic_parse_opts, verbosity);
-				num_linkages = sentence_parse(sent, panic_parse_opts);
+				parse_options_reset_resources(copts->panic_opts);
+				parse_options_set_verbosity(copts->panic_opts, verbosity);
+				num_linkages = sentence_parse(sent, copts->panic_opts);
 				if (verbosity > 0)
 				{
-					if (parse_options_timer_expired(panic_parse_opts))
+					if (parse_options_timer_expired(copts->panic_opts))
 						fprintf(stdout, "Panic timer is expired!\n");
 				}
 			}
 
 			/* print_total_time(opts); */
 
-			if (parse_options_get_batch_mode(opts))
+			if (copts->batch_mode)
 			{
-				batch_process_some_linkages(label, sent, opts);
+				batch_process_some_linkages(label, sent, copts);
 			}
 			else
 			{
-				int c = process_some_linkages(sent, opts);
+				int c = process_some_linkages(sent, copts);
 				if (c == EOF)
 				{
 					sentence_delete(sent);
@@ -957,7 +976,7 @@ int main(int argc, char * argv[])
 		}
 	}
 
-	if (parse_options_get_batch_mode(opts))
+	if (copts->batch_mode)
 	{
 		/* print_time(opts, "Total"); */
 		fprintf(stderr,
@@ -965,8 +984,7 @@ int main(int argc, char * argv[])
 	}
 
 	/* Free stuff, so that mem-leak detectors don't commplain. */
-	parse_options_delete(panic_parse_opts);
-	parse_options_delete(opts);
+	command_options_delete(copts);
 	dictionary_delete(dict);
 	fget_input_string(NULL, NULL, NULL);
 
